#ifndef _ENGINE_CORE_CONCURRENCY_ATOMIC32_HPP
#define _ENGINE_CORE_CONCURRENCY_ATOMIC32_HPP

#ifdef _EDITOR_BUILD

#include <Windows.h>
#include <malloc.h>

namespace Engine
{
	namespace Core
	{
		namespace Concurrency
		{
			template <typename T>
			class Atomic<T, 4>
			{
			public:
				Atomic() : data((T*)_aligned_malloc(4, 4)) { *data = 0; }
				Atomic(const T& _value) : data((T*)_aligned_malloc(4, 4)) { *data = _value; }
				Atomic(const Atomic<T, 4>& _value) : data((T*)_aligned_malloc(4, 4)) { *data = *(_value.data); }
				template <typename U> Atomic(const Atomic<U, 4>& _value) : data((T*)_aligned_malloc(4, 4))
				{
					volatile int *v = (volatile T*)_value.Get();
					*data = *v;
				}
				~Atomic() { _aligned_free((void*)data); }

				T CompareExchange(const T& _exchange, const T& _compare) const
				{
					LONG result = InterlockedCompareExchange((LONG*)data, *(LONG*)(&_exchange), *(LONG*)(&_compare));
					return *((T*)(&result));
				}
				T Exchange(const T& _exchange) const
				{ 
					LONG result = InterlockedExchange((LONG*)data, *(LONG*)(&_exchange));
					return *((T*)(&result));
				}

				inline T operator ++() { return (T)InterlockedIncrement((LONG*)data); }
				T operator --() { return (T)InterlockedDecrement((LONG*)data); }

				bool operator == (const T& _compare) { return (*data == _compare); }
				bool operator == (const Atomic<T, 4> _compare) { return (*data == *_compare.data); }
				bool operator != (const T& _compare) { return (*data != _compare); }
				bool operator != (const Atomic<T, 4> _compare) { return (*data != *_compare.data); }

				T operator = (const T& _exchange) { return Exchange(_exchange); }
				T operator = (const Atomic<T, 4>& _other) { return Exchange(*_other.data); }

				T* Get() const { return (T*)data; }

			private:
				volatile T* data;
			};
		}
	}
}

#endif /* if _EDITOR_BUILD */

#endif