#ifndef _ENGINE_CORE_CONCURRENCY_ATOMIC64_HPP
#define _ENGINE_CORE_CONCURRENCY_ATOMIC64_HPP

#ifdef _EDITOR_BUILD

#include <Windows.h>
#include <malloc.h>

namespace Engine
{
	namespace Core
	{
		namespace Concurrency
		{
			template <typename T>
			class Atomic<T, 8>
			{
			public:
				Atomic() : data((T*)_aligned_malloc(8, 8)) { *data = 0; }
				Atomic(const T& _value) : data((T*)_aligned_malloc(8, 8)) { *data = _value; }
				Atomic(const Atomic<T, 8>& _value) : data((T*)_aligned_malloc(8, 8)) { *data = *(_value.data); }
				template <typename U> Atomic(const Atomic<U, 8>& _value) : data((T*)_aligned_malloc(8, 8))
				{
					volatile double *v = (volatile T*)_value.Get();
					*data = *v;
				}
				~Atomic() { _aligned_free((void*)data); }

				T CompareExchange(const T& _exchange, const T& _compare) const { return (T)InterlockedCompareExchange64((LONGLONG*)data, *((LONGLONG*)&_exchange), *((LONGLONG*)&_compare)); }
				T Exchange(const T& _exchange) const { return (T)InterlockedExchange64((LONGLONG*)data, *((LONGLONG*)&_exchange)); }

				T operator ++() { return (T)InterlockedIncrement64((LONGLONG*)data); }
				T operator --() { return (T)InterlockedDecrement64((LONGLONG*)data); }

				bool operator == (const T& _compare) { return (*data == _compare); }
				bool operator == (const Atomic<T, 8> _compare) { return (*data == *_compare.data); }
				bool operator != (const T& _compare) { return (*data != _compare); }
				bool operator != (const Atomic<T, 8> _compare) { return (*data != *_compare.data); }

				T& operator = (const T& _exchange) { return Exchange(_exchange); }
				T& operator = (const Atomic<T, 8>& _other) { return Exchange(*_other.data); }

				T* Get() const { return (T*)data; }

			private:
				volatile T* data;
			};
		}
	}
}

#endif /* if _EDITOR_BUILD */

#endif