#ifndef _ENGINE_CORE_CONCURRENCY_ATOMICARRAY32_HPP
#define _ENGINE_CORE_CONCURRENCY_ATOMICARRAY32_HPP

#include <Windows.h>
#include <malloc.h>
#include <cassert>

namespace Engine
{
	namespace Core
	{
		namespace Concurrency
		{
			template <typename T>
			class AtomicArray<T, 4>
			{
			public:
				AtomicArray(unsigned int _capacity) : data(nullptr), capacity(_capacity)
				{
					if (capacity > 0)
					{
						data = (T*)_aligned_malloc(4 * _capacity, 4);
						ZeroMemory(data, 4 * capacity);
					}
				}
				~AtomicArray() { _aligned_free(data); }

				void Reserve(unsigned int _capacity)
				{
					if (_capacity > capacity)
					{
						T* tempData = (T*)_aligned_malloc(_capacity * 4, 4);

						if (data != nullptr)
						{
							memcpy(tempData, data, size*4);
							_aligned_free(data);
						}

						data = tempData;
						capacity = _capacity;
						size = _capacity;
					}
					else
						size = _capacity;
				}

				inline T CompareExchange(const unsigned int _index, const T& _exchange, const T& _compare) const
				{
					assert(_index < size);

					LONG result = InterlockedCompareExchange((LONG*)(&data[_index]), *(LONG*)(&_exchange), *(LONG*)(&_compare));
					return *((T*)(&result));
				}
				inline T Exchange(const unsigned int _index, const T& _exchange) const
				{
					assert(_index < size);

					LONG result = InterlockedExchange((LONG*)(&data[_index]), *(LONG*)(&_exchange));
					return *((T*)(&result));
				}

				inline T Increment(const unsigned int _index)
				{
					assert(_index < size);

					LONG result = InterlockedIncrement((LONG*)(&data[_index]));
					return *((T*)(&result));
				}
				inline T Decrement(const unsigned int _index)
				{
					assert(_index < size);

					LONG result = InterlockedDecrement((LONG*)(&data[_index]));
					return *((T*)(&result));
				}

				inline const T& operator[] (const unsigned int _index) const
				{
					assert(_index < size)
					return data[_index];
				}
				inline T& operator[] (const unsigned int _index)
				{
					assert(_index < size);
					return data[_index];
				}

				unsigned int Size() const
				{
					return size;
				}

			private:
				unsigned int capacity;
				unsigned int size;
				T* volatile data;
			};
		}
	}
}

#endif