#ifndef _ENGINE_CORE_CONCURRENCY_FIFOQUEUE_HPP
#define _ENGINE_CORE_CONCURRENCY_FIFOQUEUE_HPP

#include "../../dllconfig.hpp"

#include "Lockable.hpp"
#include "BlockLocker.hpp"

#include <queue>

namespace Engine
{
	namespace Core
	{
		namespace Concurrency
		{
			template <typename T>
			class /*_DLL_DECLSPEC*/ FIFOQueue : private Lockable
			{
			public:
				bool Empty()
				{
					BlockLocker b(this);
					return data.empty();
				}

				unsigned int Size()
				{
					BlockLocker b(this);
					return (unsigned int)data.size();
				}

				void Push(T _element)
				{
					BlockLocker b(this);
					data.push(_element);
				}

				T Pop()
				{
					BlockLocker b(this);

					if (data.empty())
						return T();

					T top = data.front();					
					data.pop();
					return top;
				}

				T Front()
				{
					BlockLocker b(this);
					return data.front();
				}

			private:
				std::queue<T> data;
			};


			template <typename T> class FIFOQueue<T*> : private Lockable
			{
			public:
				T* Pop()
				{
					BlockLocker b(this);

					if (data.empty())
						return nullptr;

					T* top = data.front();					
					data.pop();
					return top;
				}

				bool Empty()
				{
					BlockLocker b(this);
					return data.empty();
				}

				unsigned int Size()
				{
					BlockLocker b(this);
					return (unsigned int)data.size();
				}

				void Push(T* _element)
				{
					BlockLocker b(this);
					data.push(_element);
				}

				T* Front()
				{
					BlockLocker b(this);
					return data.front();
				}

			private:
				std::queue<T*> data;
			};
		}
	}
}

#endif