#define DLL_EXPORT

#include "Lockable.hpp"

using namespace Engine::Core::Concurrency;

volatile LockableID Lockable::nextLockID = 1;

Lockable::Lockable() : lockID(Lockable::nextLockID++)
{
	InitializeCriticalSectionAndSpinCount(&section, 0); //Sollte unter Windows XP gearbeitet werden, muss hier eine Fehlerberprfung hin
}

Lockable::~Lockable()
{
	DeleteCriticalSection(&section);
}

void Lockable::Lock()
{
	//Insert-Position finden
	LockList::iterator insertIterator = ThreadManager::LockedObjects()->begin();

	while (insertIterator != ThreadManager::LockedObjects()->end() && (*insertIterator)->lockID > this->lockID)
		insertIterator++;

	//Einfgen
	LockList::iterator newElement = ThreadManager::LockedObjects()->insert(insertIterator, this);

	if (!TryEnterCriticalSection(&section))
	{
		//Da fehlgeschlagen -> alle mit grerer ID freigeben
		for (auto it = ThreadManager::LockedObjects()->begin(); it != newElement; it++)
			LeaveCriticalSection(&(*it)->section);

		//Warten bis dieses wieder frei ist
		EnterCriticalSection(&section);

		//Die anderen wieder locken
		for (auto it = ThreadManager::LockedObjects()->begin(); it != newElement; it++)
			EnterCriticalSection(&(*it)->section);
	}
}

bool Lockable::TryLock()
{
	bool isLocked = (TryEnterCriticalSection(&section) != 0);

	if (isLocked)
	{
		//Insert-Position finden
		LockList::iterator insertIterator = ThreadManager::LockedObjects()->begin();

		while (insertIterator != ThreadManager::LockedObjects()->end() && (*insertIterator)->lockID > this->lockID)
			insertIterator++;

		//Einfgen
		LockList::iterator newElement = ThreadManager::LockedObjects()->insert(insertIterator, this);
	}

	return isLocked;
}

void Lockable::Unlock()
{
	//Element aus der Liste entfernen
	LockList::iterator it = ThreadManager::LockedObjects()->begin();
	
	while ((*it)->lockID != lockID)
		it++;

	ThreadManager::LockedObjects()->erase(it);
	LeaveCriticalSection(&section);
}