#ifndef _ENGINE_CORE_CONCURRENCY_PRIORITYQUEUE_HPP
#define _ENGINE_CORE_CONCURRENCY_PRIORITYQUEUE_HPP

#include <queue>
#include "Lockable.hpp"
#include "BlockLocker.hpp"

namespace Engine
{
	namespace Core
	{
		namespace Concurrency
		{
			template <typename T, typename Compare = std::less<T> >
			class PriorityQueue : private Lockable
			{
			public:
				bool Empty()
				{
					BlockLocker b(this);
					return data.empty();
				}

				unsigned int Size()
				{
					BlockLocker b(this);
					return (unsigned int)data.size();
				}

				void Push(T _element)
				{
					BlockLocker b(this);
					data.push(_element);
				}

				T Pop()
				{
					BlockLocker b(this);
					T element = data.top();
					data.pop();
					return element;
				}

				T Top()
				{
					BlockLocker b(this);
					return data.top();
				}

			private:
				std::priority_queue<T, std::vector<T>, Compare> data;
			};
		}
	}
}

#endif