#define DLL_EXPORT

#include "RenderControl.hpp"
#include "../../Graphic/GraphicContext.hpp"

#include <gl\glew.h>
#include <gl\wglew.h>
#include <cassert>
#include <iostream>

using namespace Engine;
using namespace Engine::Core::Control;
using namespace Engine::Core;
using namespace Engine::Core::EventHandling;

RenderControl::RenderControl(unsigned int _width, unsigned int _height) : width(_width), height(_height), hasMouseFocus(false), showTimes(true), showBoundings(true),
	showLightVolumes(true), showBloom(true), showTonemapper(true), showFXAA(true)
{ }

RenderControl::~RenderControl()
{
	if (context != nullptr)
		delete context;
}

void RenderControl::Init(HWND _hwnd)
{
	context = new Graphic::GraphicContext(_hwnd, width, height);
}

void RenderControl::SwapBuffer()
{
	assert(context != nullptr);

	glViewport(0, 0, width, height);
	context->SwapBuffer();

	keyboard.InitFrame();
	mouse.InitFrame();
}

void RenderControl::Resize(unsigned int _width, unsigned int _height)
{
	context->Activate();

	Height(_height);
	Width(_width);
	context->Resize(_width, _height);

	SizeChanged(this, EventArgs::Empty());
}

inline unsigned int RenderControl::Width()
{
	return width;
}

inline void RenderControl::Width(unsigned int _width)
{
	width = _width;
}

inline unsigned int RenderControl::Height()
{
	return height;
}

inline void RenderControl::Height(unsigned int _height)
{
	height = _height;
}

inline Graphic::GraphicContext* RenderControl::Context() const
{
	return context;
}

inline IO::KeyboardInput& RenderControl::Keyboard()
{
	return keyboard;
}

inline IO::MouseInput& RenderControl::Mouse()
{
	return mouse;
}

bool RenderControl::HasMouseFocus() const
{
	return hasMouseFocus;
}

void RenderControl::HasMouseFocus(bool _focus)
{
	hasMouseFocus = _focus;
}

inline bool RenderControl::ShowTimes() const
{
	return showTimes;
}
inline void RenderControl::ShowTimes(bool _value)
{
	showTimes = _value;
}
inline bool RenderControl::ShowBoundings() const
{
	return showBoundings;
}
inline void RenderControl::ShowBoundings(bool _value)
{
	showBoundings = _value;
}
inline bool RenderControl::ShowLightVolumes() const
{
	return showLightVolumes;
}
inline void RenderControl::ShowLightVolumes(bool _value)
{
	showLightVolumes = _value;
}
inline bool RenderControl::ShowTonemapper() const
{
	return showTonemapper;
}
inline void RenderControl::ShowTonemapper(bool _value)
{
	showTonemapper = _value;
}
inline bool RenderControl::ShowBloom() const
{
	return showBloom;
}
inline void RenderControl::ShowBloom(bool _value)
{
	showBloom = _value;
}
inline bool RenderControl::ShowFXAA() const
{
	return showFXAA;
}
inline void RenderControl::ShowFXAA(bool _value)
{
	showFXAA = _value;
}