#define DLL_EXPORT

#include "SystemConsole.hpp"

#include <stdio.h>
#include <tchar.h>
#include <strsafe.h>
#include "../../Util/Logger.hpp"
#include "../../Util/Convert.hpp"
#include "../System/SystemConfiguration.hpp"

using namespace Engine;
using namespace Engine::Core;
using namespace Engine::Core::Control;
using namespace Engine::Core::System;
using namespace Engine::Util;

HANDLE SystemConsole::hOutputConsole = nullptr;
HWND SystemConsole::consoleWindow;



void SystemConsole::OpenConsole()
{
	AllocConsole();
	freopen("CONOUT$", "a", stderr);
	freopen("CONOUT$", "a", stdout);

	consoleWindow = GetConsoleWindow();

	hOutputConsole = GetStdHandle(STD_OUTPUT_HANDLE);

	CONSOLE_SCREEN_BUFFER_INFOEX consoleInfo;
	consoleInfo.cbSize = sizeof(CONSOLE_SCREEN_BUFFER_INFOEX);

	BOOL result = GetConsoleScreenBufferInfoEx(hOutputConsole, &consoleInfo);

	//Modify colors
	consoleInfo.ColorTable[0] = RGB(0,0,0); 
	consoleInfo.ColorTable[1] = RGB(50, 50, 50);
	consoleInfo.ColorTable[2] = RGB(255, 255, 255);
	consoleInfo.ColorTable[3] = RGB(255, 0, 0);
	consoleInfo.ColorTable[4] = RGB(0, 255, 0);
	consoleInfo.ColorTable[5] = RGB(0, 0, 255);
	consoleInfo.ColorTable[6] = RGB(192, 192, 192); //Default Gray
	consoleInfo.ColorTable[7] = RGB(240, 205, 0);
	consoleInfo.ColorTable[8] = RGB(80, 80, 80);
	for (int i = 9; i < 16; i++)
		consoleInfo.ColorTable[i] = RGB(192, 192, 192);

	//Modify size
	consoleInfo.dwSize.X = SystemConfiguration::Get<int>("SystemConsoleWidth") - 1;
	consoleInfo.dwSize.Y = 500;
	consoleInfo.dwMaximumWindowSize.X = 200;
	consoleInfo.dwMaximumWindowSize.Y = 500;
	consoleInfo.srWindow.Right = SystemConfiguration::Get<int>("SystemConsoleWidth");
	consoleInfo.srWindow.Bottom = SystemConfiguration::Get<int>("SystemConsoleHeight");

	if (result != 0)
	{
		result = SetConsoleScreenBufferInfoEx(hOutputConsole, &consoleInfo);
	}

	if (result != 0)
	{
		//Fonts????
	}

	if (result == FALSE) //Error Check
	{
		LPVOID lpMsgBuf;
		DWORD dw = GetLastError(); 

		FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM |
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			dw,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,
			0, NULL );

		Logger::Fatal(std::string("Unable to set console settings"));
		MessageBox(NULL, (LPCTSTR)lpMsgBuf, TEXT("Error"), MB_OK); 

		
	}

	SetColor(ConsoleColor::DefaultConsoleColor);
	
}

void SystemConsole::SetColor(ConsoleColor _color)
{
	SetConsoleTextAttribute(hOutputConsole, _color);
}

void SystemConsole::PrintInfo()
{
	SetColor(Core::Control::ConsoleColor::DefaultConsoleColor);
	std::cout << "ConsoleColor::DefaultConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::DarkGrayConsoleColor);
	std::cout << "ConsoleColor::DarkGrayConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::WhiteConsoleColor);
	std::cout << "ConsoleColor::WhiteConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::RedConsoleColor);
	std::cout << "ConsoleColor::RedConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::GreenConsoleColor);
	std::cout << "ConsoleColor::GreenConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::BlueConsoleColor);
	std::cout << "ConsoleColor::BlueConsoleColor" << std::endl;
	SetColor(Core::Control::ConsoleColor::BlackConsoleColor);
	std::cout << "ConsoleColor::BlackConsoleColor" << std::endl;
	SetColor(ConsoleColor::YellowConsoleColor);
	std::cout << "ConsoleColor::YellowConsoleColor" << std::endl;
	SetColor(ConsoleColor::GrayConsoleColor);
	std::cout << "ConsoleColor::GrayConsoleColor" << std::endl;

	SetColor(ConsoleColor::UndefinedConsoleColor1);
	std::cout << "ConsoleColor::UndefinedConsoleColor1" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor2);
	std::cout << "ConsoleColor::UndefinedConsoleColor2" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor3);
	std::cout << "ConsoleColor::UndefinedConsoleColor3" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor4);
	std::cout << "ConsoleColor::UndefinedConsoleColor4" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor5);
	std::cout << "ConsoleColor::UndefinedConsoleColor5" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor6);
	std::cout << "ConsoleColor::UndefinedConsoleColor6" << std::endl;
	SetColor(ConsoleColor::UndefinedConsoleColor7);
	std::cout << "ConsoleColor::UndefinedConsoleColor7" << std::endl;

	SetColor(ConsoleColor::DefaultConsoleColor);
}

void SystemConsole::Focus()
{
	SetForegroundWindow(consoleWindow);
}