#define DLL_EXPORT

#include "Window.hpp"

#include <windowsx.h>
#include <GL\glew.h>
#include <gl\GL.h>
#include <gl\GLU.h>

#include "../../Util/Logger.hpp"

#define LONGPTR_ID_THIS 0
#define EXTRA_BYTES (sizeof(LONG_PTR))

using namespace Engine::Core::IO;
using namespace Engine::Core::Control;
using namespace Engine::Util;

std::unordered_map<HWND, Window*> Window::windowLookupTable;

Window::Window(HINSTANCE _hInstance, LPCWSTR _title, unsigned int _width, unsigned int _height) : RenderControl(_width, _height), hwnd(nullptr)
{
	WNDCLASSEX wndClass;
	wndClass.cbSize = sizeof(WNDCLASSEX);
	wndClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC | CS_DBLCLKS;
	wndClass.lpfnWndProc = &WndProc;
	wndClass.cbClsExtra = 0;
	wndClass.cbWndExtra = EXTRA_BYTES;
	wndClass.hInstance = _hInstance;
	wndClass.hIcon = LoadIcon(_hInstance,  MAKEINTRESOURCE(101));
	wndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndClass.hbrBackground = (HBRUSH)(COLOR_BACKGROUND + 1);
	wndClass.lpszMenuName = NULL;
	wndClass.lpszClassName = _title;
	wndClass.hIconSm = NULL;

	if (!RegisterClassEx(&wndClass))
		Logger::Fatal("Cannot register window");

	DWORD windowStyle = WS_OVERLAPPEDWINDOW;

	hwnd = CreateWindow(
		_title, _title,
		windowStyle,
		CW_USEDEFAULT, CW_USEDEFAULT,
		_width, _height,
		NULL, NULL, _hInstance, NULL);

	SetWindowLongPtr(hwnd, LONGPTR_ID_THIS, (LONG_PTR)this);

	RECT clientRect, windowRect;
	GetClientRect(hwnd, &clientRect);
	GetWindowRect(hwnd, &windowRect);
	POINT difference;
	difference.x = (windowRect.right - windowRect.left) - clientRect.right;
	difference.y = (windowRect.bottom - windowRect.top) - clientRect.bottom;

	MoveWindow(hwnd,windowRect.left, windowRect.top, _width + difference.x, _height + difference.y, FALSE);

	//Fr Messages registrieren
	windowLookupTable.emplace(hwnd, this);

	this->Init(hwnd);
}

Window::~Window()
{
	if (hwnd != nullptr)
	{
		windowLookupTable.erase(hwnd);
		DestroyWindow(hwnd);
	}
}

void Window::Show()
{
	ShowWindow(hwnd, SW_SHOW);
}

void Window::Hide()
{
	ShowWindow(hwnd, SW_HIDE);
}

bool Window::CheckMessages(bool& _running)
{
	MSG msg;

	if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE )  )
	{	
		//Messages verarbeiten
		TranslateMessage(&msg);
		DispatchMessage(&msg);
				
		if (msg.message == WM_QUIT)
			_running = false;

		return true;
	}

	_running = true;
	return false;
}

LRESULT CALLBACK Window::WndProc(HWND _hwnd, UINT _message, WPARAM _wParam, LPARAM _lParam)
{
	auto it = Window::windowLookupTable.find(_hwnd);
	if (it != Window::windowLookupTable.end())
		return it->second->ProcessSystemEvent(_message, _wParam, _lParam);
	else
		return DefWindowProc(_hwnd, _message, _wParam, _lParam );
}

LRESULT Window::ProcessSystemEvent(UINT _message, WPARAM _wParam, LPARAM _lParam)
{
	LRESULT result = 0;

	switch (_message)
	{
	case WM_CLOSE:
		PostQuitMessage(0);
		break;
	case WM_KEYUP:
		Keyboard().KeyMessage(_wParam, _lParam, false);
		break;
	case WM_KEYDOWN:
		Keyboard().KeyMessage(_wParam, _lParam, true);

		if (Keyboard().IsKeyPressed(Keys::Escape))
		{
			if (!HasMouseFocus()) //Otherwise the contoller has to keep track of escapes
				PostQuitMessage(0);
		}

		break;
	case WM_LBUTTONDBLCLK:
		Mouse().MouseDoubleClickMessage(MouseButton::Left);
		break;
	case WM_RBUTTONDBLCLK:
		Mouse().MouseDoubleClickMessage(MouseButton::Right);
		break;
	case WM_MBUTTONDBLCLK:
		Mouse().MouseDoubleClickMessage(MouseButton::Middle);
		break;
	case WM_LBUTTONDOWN:
		Mouse().MouseMessage(MouseButton::Left, true);
		break;
	case WM_RBUTTONDOWN:
		Mouse().MouseMessage(MouseButton::Right, true);
		break;
	case WM_MBUTTONDOWN:
		Mouse().MouseMessage(MouseButton::Middle, true);
		break;
	case WM_LBUTTONUP:
		Mouse().MouseMessage(MouseButton::Left, false);
		break;
	case WM_RBUTTONUP:
		Mouse().MouseMessage(MouseButton::Right, false);
		break;
	case WM_MBUTTONUP:
		Mouse().MouseMessage(MouseButton::Middle, false);
		break;
	case WM_MOUSEWHEEL:
		Mouse().MouseWheelMessage(_wParam);
		break;
	case WM_SIZE:
		RECT clientRect;
		GetClientRect(hwnd, &clientRect);
		this->Resize(clientRect.right - clientRect.left, clientRect.bottom - clientRect.top);
		break;
	default:
		result = DefWindowProc( hwnd, _message, _wParam, _lParam );
	}

	return result;
}