#ifndef _ENGINE_CORE_EVENTHANDLING_EVENT_HPP
#define _ENGINE_CORE_EVENTHANDLING_EVENT_HPP

#include "../../dllconfig.hpp"
#include "IEventHandler.hpp"
#include "EventArgs.hpp"
#include <vector>

namespace Engine
{
	namespace Core
	{
		namespace EventHandling
		{
			template <typename ArgsType = EventArgs>
			class Event
			{
			public:
				void operator += (IEventHandler* _handler)
				{
					RegisterHandler(_handler);
				}
				void operator -= (IEventHandler* _handler)
				{
					RemoveHandler(_handler);
				}
				void operator () (void* _sender, ArgsType& _args)
				{
					Call(_sender, _args);
				}

			private:
				void RegisterHandler(IEventHandler* _handler)
				{
					handler.push_back(_handler);
				}

				void RemoveHandler(IEventHandler* _handler)
				{
					for (auto it = handler.begin(); it != handler.end(); it++)
					{
						if (*(*it) == *_handler)
						{
							handler.erase(it);
							break;
						}
					}
				}

				void Call(void* _sender, ArgsType& _args)
				{
					for (auto it = handler.begin(); it != handler.end(); it++)
					(*it)->Call(_sender, _args);
				}

				std::vector<IEventHandler*> handler;
			};
		}
	}
}

#endif