#ifndef _ENGINE_CORE_EVENTHANDLING_EVENTHANDLER_HPP
#define _ENGINE_CORE_EVENTHANDLING_EVENTHANDLER_HPP

#include "../../dllconfig.hpp"
#include "IEventHandler.hpp"
#include "EventArgs.hpp"
#include "FunctionPointerCaster.hpp"

namespace Engine
{
	namespace Core
	{
		namespace EventHandling
		{
			template <typename ClassType, typename ArgsType>
			class EventHandler : public IEventHandler
			{
			public:
				typedef void (ClassType::*EventFunction)(void*, ArgsType&);

				EventHandler(ClassType* _object, EventFunction _function) : object(_object), function(_function)
				{ }
				virtual ~EventHandler()
				{ }

				virtual long long GetCallAddress() const
				{
					FunctionPointerCaster<EventFunction, long long*> caster;
					caster.FunctionPointer = function;
					return (long long)caster.Pointer;
				}

				virtual long long GetObjectAddress() const
				{
					return (long long)object;
				}

				virtual void Call(void* _sender, EventArgs& _args)
				{
					(object->*function)(_sender, (ArgsType&)_args);
				}

			private:
				ClassType* object;
				EventFunction function;
			};
		}
	}
}

#endif