#ifndef _ENGINE_CORE_IO_ASSETLOADER_HPP
#define _ENGINE_CORE_IO_ASSETLOADER_HPP

#include "../../dllconfig.hpp"
#include "DictionaryEntry.hpp"
#include "../../Util/SharedPointer.hpp"
#include "../../Graphic/Materials/Material.hpp"
#include "../../Graphic/Meshes/Mesh.hpp"
#include "../../Graphic/Meshes/TransparentMesh.hpp"
#include "../../Scene/Animations/CurveData.hpp"

#include <unordered_map>
#include <string>

namespace Engine
{
	namespace Scene { class World; class SceneObject; }

	namespace Core
	{
		namespace IO
		{
			class _DLL_DECLSPEC AssetLoader
			{
			public:
				AssetLoader(Graphic::GraphicContext* _context, EngineManager* _manager);
				~AssetLoader();

				Scene::SceneObject* LoadAsset(unsigned short _id, Scene::World* _world);
				Scene::SceneObject* LoadAsset(std::string _name, Scene::World* _world);
				ShaderPtr LoadDeferredShader(unsigned short _id);
				ShaderPtr LoadForwardShader(unsigned short _id);
				ShaderPtr LoadNormalShader(unsigned short _id);
				ShaderPtr LoadOctreeShader(unsigned short _id);
				Util::SharedPointer<Graphic::Buffers::Texture2D> LoadTexture(unsigned short _id, bool _isGammaCorrect);
				Util::SharedPointer<Graphic::Materials::Material> LoadMaterial(unsigned short _id);
				Util::SharedPointer<Graphic::Materials::Material> LoadMaterial(std::string _name);
				Util::SharedPointer<Graphic::Meshes::Mesh> LoadMesh(unsigned short _id);
				Util::SharedPointer<Graphic::Meshes::TransparentMesh> LoadTransparentMesh(unsigned short _id);
				Scene::Animations::Animation* LoadAnimation(unsigned int _id);

			private:
				void LoadDictionaries();

				unsigned short LoadDictionary(FILE *_file, DictionaryEntry** _dictionary);
				DictionaryEntry ReadEntry(FILE *_file, DictionaryEntry* _dictionary);
				unsigned short LoadNamedDictionary(FILE *_file, DictionaryEntry** _dictionary, std::unordered_map<std::string, unsigned short>& _names, bool _showLog = false);
				DictionaryEntry ReadNamedEntry(FILE *_file, DictionaryEntry* _dictionary, std::unordered_map<std::string, unsigned short>& _names, bool _showLog = false);

				void ReadMeshData(unsigned int _id, float** _data, unsigned short& _dataCount, unsigned short& _vertexSize, unsigned int **_indices, unsigned short& _indexCount,
					std::vector<Graphic::Meshes::VertexStreamElement>& _mappings, Math::Vec4f& _boundingLocation, Math::Vec3f& _boundingSize);
				
				ShaderPtr LoadShader(unsigned short _id, unsigned int _mode, ShaderPtr** _dictionary);

				std::vector<Scene::SceneObject*> LoadNode(char** _buffer, Scene::World* _world);

				DictionaryEntry* assetDictionary;
				DictionaryEntry* meshDictionary;
				DictionaryEntry* materialDictionary;
				DictionaryEntry* textureDictionary;
				DictionaryEntry* shaderDictionary;
				DictionaryEntry* animationDictionary;
				std::unordered_map<std::string, unsigned short> assetNames;
				std::unordered_map<std::string, unsigned short> materialNames;

				ShaderPtr** deferredShaderCache;
				ShaderPtr** forwardShaderCache;
				ShaderPtr** normalShaderCache;
				ShaderPtr** octreeShaderCache;
				Util::SharedPointer<Graphic::Buffers::Texture2D>** textureCache;
				Util::SharedPointer<Graphic::Materials::Material>** materialCache;
				Util::SharedPointer<Graphic::Meshes::Mesh>** meshCache;
				Util::SharedPointer<Graphic::Meshes::TransparentMesh>** transparentMeshCache;
				Util::SharedPointer<Scene::Animations::CurveData>*** curveCache;

				unsigned int curveCacheCount;

				std::string unifiedVertexShaderCode, unifiedNormalShaderCode, unifiedDeferredShaderCode, unifiedForwardShaderCode;
				std::string unifiedOctreeVertexCode, unifiedOctreeGeometryCode, unifiedOctreeFragmentCode;

				FILE* shaderFile;
				FILE* dataFile;

				Graphic::GraphicContext* context;
				EngineManager* manager;
			};
		}
	}
}

#endif