#define DLL_EXPORT

#include "MouseInput.hpp"
#include "../../Util/Logger.hpp"
#include "../../Util/Convert.hpp"

using namespace Engine;
using namespace Engine::Core;
using namespace Engine::Core::IO;

MouseInput::MouseInput() : fixMousePosition(false), cursorVisibility(CursorVisibility::Visible), mouseWheel(0.0f)
{
	for (int i = 0; i < 3; i++)
		buttonState[i] = MouseButtonState::ButtonReleased;

	GetCursorPos(&mousePosition);
}
MouseInput::~MouseInput()
{ }

void MouseInput::MouseMessage(MouseButton _button, bool _isPressed)
{
	if (buttonState[_button] != MouseButtonState::ButtonDoubleClick)
	{
		if (_isPressed)
			buttonState[_button] = MouseButtonState::ButtonDown;
		else
			buttonState[_button] = MouseButtonState::ButtonUp;
	}
}

void MouseInput::MouseDoubleClickMessage(MouseButton _button)
{
	buttonState[_button] = MouseButtonState::ButtonDoubleClick;
}

void MouseInput::MouseWheelMessage(WPARAM _message)
{
	mouseWheel = (float)GET_WHEEL_DELTA_WPARAM(_message) / (float)WHEEL_DELTA;
}

void MouseInput::MouseWheelMessage(int _delta)
{
	mouseWheel = (float)_delta  / (float)WHEEL_DELTA;
}

void MouseInput::InitFrame()
{
	mouseWheel = 0.0f;

	for (int i = 0; i < 3; i++)
	{
		if (buttonState[i] == MouseButtonState::ButtonDown)
			buttonState[i] = MouseButtonState::ButtonPressed;
		else if (buttonState[i] == MouseButtonState::ButtonUp)
			buttonState[i] = MouseButtonState::ButtonReleased;
		else if (buttonState[i] == MouseButtonState::ButtonDoubleClick)
			buttonState[i] = MouseButtonState::ButtonUp;
	}

	POINT newMousePosition;
	GetCursorPos(&newMousePosition);

	relativeMousePosition = newMousePosition;
	relativeMousePosition.x -= mousePosition.x;
	relativeMousePosition.y -= mousePosition.y;

	if (fixMousePosition && (relativeMousePosition.x != 0 || relativeMousePosition.y != 0))
		SetCursorPos(mousePosition.x, mousePosition.y);
	else
		mousePosition = newMousePosition;

	//Change cursor visibility
	if (cursorVisibility == CursorVisibility::ChangeToHidden)
	{
		while (ShowCursor(false) > 0) { }
		cursorVisibility = CursorVisibility::Hidden;
	}
	else if (cursorVisibility == CursorVisibility::ChangeToVisible)
	{
		while (ShowCursor(true) <= 0) { }
		cursorVisibility = CursorVisibility::Visible;
	}
}

inline bool MouseInput::IsButtonDown(MouseButton _button) const
{
	return (buttonState[_button] == MouseButtonState::ButtonDown || buttonState[_button] == MouseButtonState::ButtonDoubleClick);
}
inline bool MouseInput::IsButtonUp(MouseButton _button) const
{
	return (buttonState[_button] == MouseButtonState::ButtonUp);
}
inline bool MouseInput::IsButtonPressed(MouseButton _button) const
{
	return (buttonState[_button] == MouseButtonState::ButtonDown) || (buttonState[_button] == MouseButtonState::ButtonPressed || buttonState[_button] == MouseButtonState::ButtonDoubleClick);
}
inline bool MouseInput::IsButtonDoubleClick(MouseButton _button) const
{
	return (buttonState[_button] == MouseButtonState::ButtonDoubleClick);
}

inline float MouseInput::MouseWheel() const
{
	return mouseWheel;
}

inline bool MouseInput::FixMousePosition() const
{
	return fixMousePosition;
}

inline void MouseInput::FixMousePosition(bool _fix)
{
	fixMousePosition = _fix;

	if (_fix && cursorVisibility != CursorVisibility::Hidden)
		cursorVisibility = CursorVisibility::ChangeToHidden;
	else if (!_fix  && cursorVisibility != CursorVisibility::Visible)
		cursorVisibility = CursorVisibility::ChangeToVisible;		
}

inline POINT MouseInput::MousePosition() const
{
	return mousePosition;
}

inline void MouseInput::MousePosition(POINT _value)
{
	mousePosition = _value;
}

inline POINT MouseInput::RelativeMousePosition() const
{
	return relativeMousePosition;
}