#ifndef _ENGINE_CORE_IO_MOUSEINPUT_HPP
#define _ENGINE_CORE_IO_MOUSEINPUT_HPP

#include "../../dllconfig.hpp"
#include <Windows.h>

namespace Engine
{
	namespace Core
	{
		namespace IO
		{
			enum MouseButton : unsigned char
			{
				Left = 0,
				Middle = 1,
				Right = 2
			};

			enum CursorVisibility : unsigned char
			{
				Visible, ChangeToVisible, Hidden, ChangeToHidden
			};

			enum MouseButtonState : unsigned char
			{
				ButtonDown, ButtonPressed, ButtonUp, ButtonReleased, ButtonDoubleClick
			};

			class _DLL_DECLSPEC MouseInput
			{
			public:
				MouseInput();
				~MouseInput();

				void MouseMessage(MouseButton _button, bool _isPressed);
				void MouseDoubleClickMessage(MouseButton _button);
				void MouseWheelMessage(WPARAM _message);
				void MouseWheelMessage(int _delta);

				/// <summary>
				/// Initializes all fields for the next frame.
				/// </summary>
				/// This method must be called from the main thread, otherwise some of the system calls would not work correctly
				void InitFrame();

				bool IsButtonDown(MouseButton _button) const;
				bool IsButtonUp(MouseButton _button) const;
				bool IsButtonPressed(MouseButton _button) const;
				bool IsButtonDoubleClick(MouseButton _button) const;
				float MouseWheel() const;

				POINT MousePosition() const;
				void MousePosition(POINT _value);
				POINT RelativeMousePosition() const;

				bool FixMousePosition() const;
				void FixMousePosition(bool _fix);

			private:
				CursorVisibility cursorVisibility;
				bool fixMousePosition;
				float mouseWheel;
				MouseButtonState buttonState[3];
				POINT mousePosition;
				POINT relativeMousePosition;
			};
		}
	}
}

#endif