#define DLL_EXPORT

#include "SystemConfigurationReader.hpp"

#include <Windows.h>

#include "../System/SystemConfiguration.hpp"
#include "../../Util/Convert.hpp"
#include "../../Util/Logger.hpp"

#ifndef NDEBUG
	#define SYSTEMCONFIGURATION_PATH "../../../Data/system_debug.cfg"
#else
	#define SYSTEMCONFIGURATION_PATH "../../../Data/system_release.cfg"
#endif

#define SYSTEMCONFIGURATION_VERSION "0.2"
#define COMMENTCHAR '%'
#define BUFFER_LENGTH 150

#define ALLOCCONSOLE AllocConsole();\
	freopen("CONOUT$", "a", stderr);\
	freopen("CONOUT$", "a", stdout);

using namespace Engine;
using namespace Engine::Core;
using namespace Engine::Math;
using namespace Engine::Core::System;
using namespace Engine::Core::IO;
using namespace Engine::Core::Collections;
using namespace Engine::Util;

void SystemConfigurationReader::Load()
{
	FILE *file = fopen(SYSTEMCONFIGURATION_PATH, "r");

	if (file == nullptr)
	{
		ALLOCCONSOLE;
		Logger::Fatal(std::string("Cannot open systemconfiguration: ") + SYSTEMCONFIGURATION_PATH);
		return;
	}

	char buffer[BUFFER_LENGTH];

	//Versionsnummer berprfen
	fgets(buffer, BUFFER_LENGTH, file);

	if (strncmp(SYSTEMCONFIGURATION_VERSION, buffer, strlen(SYSTEMCONFIGURATION_VERSION)) != 0)
	{
		ALLOCCONSOLE;
		Logger::Warning(std::string("SystemConfiguration: Version ") + std::string(buffer) + std::string("does not match Version ") + std::string(SYSTEMCONFIGURATION_VERSION));
		return;
	}

	//Load Values
	unsigned int line = 1;

	while (fgets(buffer, BUFFER_LENGTH, file) != nullptr)
	{
		line ++;

		if (strcmp("\n", buffer) != 0 && buffer[0] != COMMENTCHAR) //Ignore empty lines and comments
		{
			char* type = strtok(buffer, " ");
			char* rest = buffer + strlen(type) + 1;
			char* identifier = strtok(rest, "=");
			char* value = strtok(nullptr, "=");

			if (type == nullptr || identifier == nullptr || value == nullptr)
			{
				ALLOCCONSOLE;
				Logger::Fatal(std::string("SystemConfiguration: Syntax error in line ") + Convert::ToString(line));
				return;
			}
			else
			{
				void* data = nullptr;

				//Parse type
				if (strcmp(type, "int") == 0)
					data = new int(atoi(value));
				else if (strcmp(type, "uint") == 0)
					data = new unsigned int(strtoul(value, &rest, 10));
				else if (strcmp(type, "float") == 0)
					data = new float((float)atof(value));
				else if (strcmp(type, "bool") == 0)
				{
					if (strncmp(value, "true", 4) == 0)
						data = new bool(true);
					else
						data = new bool(false);
				}
				else if (strcmp(type, "string") == 0)
				{
					value[strlen(value) - 1] = '\0';
					data = new std::string(value);
				}
				else if (strcmp(type, "vec2") == 0)
				{
					float v1, v2;
					value[strlen(value) - 2] = '\0';
					v1 = (float)atof(strtok(value + 1, ","));
			        v2 = (float)atof(strtok(nullptr, ","));

					data = new Math::Vec2f(v1, v2);
				}
				else if (strcmp(type, "vec3") == 0)
				{
					float v1, v2, v3;
					value[strlen(value) - 2] = '\0';
					v1 = (float)atof(strtok(value + 1, ","));
			        v2 = (float)atof(strtok(nullptr, ","));
					v3 = (float)atof(strtok(nullptr, ","));

					data = new Math::Vec3f(v1, v2, v3);
				}

				if (data != nullptr)
					SystemConfiguration::configValues[HashString((const char*)identifier)] = data;
			}
		}
	}

	fclose(file);
}