#ifndef _ENGINE_CORE_MEMORY_ALLOCATOR_HPP
#define _ENGINE_CORE_MEMORY_ALLOCATOR_HPP

#include "../../dllconfig.hpp"

namespace Engine
{
	namespace Core
	{
		namespace Memory
		{			
			class _DLL_DECLSPEC Allocator
			{
			public:
				Allocator() { }
				virtual ~Allocator() { }

				template <typename T> T* Alloc()
				{
					return (T*)InternalAlloc(sizeof(T), 1);
				}

				template <typename T, unsigned int alignment> T* Alloc()
				{
					return (T*)InternalAlloc(sizeof(T), alignment);
				}

				void* Alloc(unsigned int _size, unsigned int _alignment)
				{
					return InternalAlloc(_size, _alignment);
				}

				template <typename T, unsigned int count> T* AllocArray()
				{
					return (T*)InternalAllocArray(sizeof(T), count, 1);
				}

				template <typename T, unsigned int count, unsigned int alignment> T* AllocArray()
				{
					return (T*)InternalAllocArray(sizeof(T), count, alignment);
				}

				virtual void Free(void* _data) = 0;
				virtual void FreeArray(void* _data) = 0;

				static Allocator& DefaultAllocator();

			private:
				static Allocator& defaultAllocator;

				virtual void* InternalAlloc(unsigned int _size, unsigned int _alignment) = 0;
				virtual void* InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment) = 0;
			};
		}
	}
}

#endif