#define DLL_EXPORT

#include "DoubleStackAllocator.hpp"
#include "../../Util/Logger.hpp"

using namespace Engine::Core::Memory;
using namespace Engine::Util;

DoubleStackAllocator::DoubleStackAllocator(unsigned int _size, Allocator& _allocator, unsigned int _alignment) : allocator(_allocator), size(_size)
{
	//Request memory
	data = (byte*)allocator.Alloc(_size, _alignment);

	//Initialize pointer
	leftFree = data;
	rightFree = data + _size;
}
DoubleStackAllocator::~DoubleStackAllocator()
{
	if (leftFree != data && rightFree != data + size)
		Logger::Fatal("Unable to free non empty DoubleStackAllocator");

	allocator.Free(data);
}

void DoubleStackAllocator::Free(void* _data)
{
	if (_data != nullptr)
	{
		byte* freeData = (byte*)_data;

		if (_data < leftFree)
		{
			unsigned int* sizePointer = (unsigned int*)_data - 1;

			if (freeData + *sizePointer != leftFree)
				Logger::Fatal("DoubleStackAllocator: Cannot free element not on top of stack");

			leftFree = (byte*)sizePointer;
		}
		else if (_data > rightFree)
		{
			if (freeData - sizeof(unsigned int) != rightFree)
				Logger::Fatal("DoubleStackAllocator: Cannot free element not on top of stack");

			unsigned int* sizePointer = (unsigned int*)_data - 1;
			rightFree += *sizePointer + sizeof(unsigned int);
		}
		else
		{
			Logger::Fatal("DoubleStackAllocator: Tried to free not allocated address");
		}
	}
}
void DoubleStackAllocator::FreeArray(void* _data)
{
	Free(_data);
}

void* DoubleStackAllocator::InternalAlloc(unsigned int _size, unsigned int _alignment)
{
	return InternalAlloc(_size, _alignment, AllocationType::PERSISTANT);
}
void* DoubleStackAllocator::InternalAlloc(unsigned int _size, unsigned int _alignment, AllocationType _allocType)
{
	if (_allocType == AllocationType::PERSISTANT)
	{
		unsigned int offsetToAlignment = (__int64)leftFree % _alignment;

		if (offsetToAlignment != 0)
			offsetToAlignment = _alignment - offsetToAlignment;

		if (leftFree + _size + offsetToAlignment + sizeof(unsigned int) > rightFree)
			Logger::Fatal("DoubleStackAllocator out of memory");

		unsigned int* returnPointer = (unsigned int*)leftFree;
		leftFree = leftFree + offsetToAlignment + _size + sizeof(unsigned int);
	
		*returnPointer = _size;
		return returnPointer + 1;
	}
	else if (_allocType == AllocationType::TEMPORARY)
	{
		unsigned int offsetToAlignment = (__int64)(rightFree - sizeof(unsigned int) - _size) % _alignment;

		if (leftFree + _size + offsetToAlignment + sizeof(unsigned int) > rightFree)
			Logger::Fatal("DoubleStackAllocator out of memory");

		byte* returnPointer = rightFree - _size - offsetToAlignment;
		rightFree = rightFree - offsetToAlignment - _size - sizeof(unsigned int);
	
		*(unsigned int*)rightFree = _size;
		return returnPointer;
	}

	return nullptr;
}

void* DoubleStackAllocator::InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment)
{
	return InternalAllocArray(_elementSize, _count, _alignment, AllocationType::PERSISTANT);
}
void* DoubleStackAllocator::InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment, AllocationType _allocType)
{
	return InternalAlloc(_elementSize * _count, _alignment, _allocType);
}