#ifndef _ENGINE_CORE_MEMORY_DOUBLESTACKALLOCATOR_HPP
#define _ENGINE_CORE_MEMORY_DOUBLESTACKALLOCATOR_HPP

#include "../../dllconfig.hpp"
#include "Allocator.hpp"

namespace Engine
{
	namespace Core
	{
		namespace Memory
		{
			enum AllocationType : short {
				TEMPORARY = -1,
				PERSISTANT = 1
			};

			class _DLL_DECLSPEC DoubleStackAllocator : public Allocator
			{
			public:
				DoubleStackAllocator(unsigned int _size, Allocator& _allocator = Allocator::DefaultAllocator(), unsigned int _alignment = 1);
				virtual ~DoubleStackAllocator();

				virtual void Free(void* _data);
				virtual void FreeArray(void* _data);

				using Allocator::Alloc;
				using Allocator::AllocArray;

				template <typename T, unsigned int alignment, AllocationType allocType> T* Alloc()
				{
					return (T*)InternalAlloc(sizeof(T), alignment, allocType);
				}

				void* Alloc(unsigned int _size, unsigned int _alignment, AllocationType _allocType)
				{
					return InternalAlloc(_size, _alignment, _allocType);
				}

				template <typename T, unsigned int count, unsigned int alignment, AllocationType allocType> T* AllocArray()
				{
					return (T*)InternalAllocArray(sizeof(T), count, alignment, allocType);
				}

			private:
				virtual void* InternalAlloc(unsigned int _size, unsigned int _alignment);
				void* InternalAlloc(unsigned int _size, unsigned int _alignment, AllocationType _allocType);
				virtual void* InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment);
				void* InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment, AllocationType _allocType);

				Allocator& allocator;
				byte* data;
				byte* leftFree;
				byte* rightFree;
				unsigned int size;
			};
		}
	}
}

#endif