#ifndef _ENGINE_CORE_MEMORY_POOLALLOCATOR_HPP
#define _ENGINE_CORE_MEMORY_POOLALLOCATOR_HPP

#include "Allocator.hpp"

namespace Engine
{
	namespace Core
	{
		namespace Memory
		{
			/// <summary>
			/// This allocator can handle a predefined number of fixed-sized elements.
			/// The smallest size for the elements is sizeof(unsigned int). If an element is free the index of the next free block is stored at the beginning of this block.
			/// When an array is allocated it occupies n+1 elements, because the first block holds the number of elements (as an unsigned int)
			/// </summary>
			class _DLL_DECLSPEC PoolAllocator : public Allocator
			{
			public:
				PoolAllocator(unsigned int _maxElements, unsigned int _elementSize, Allocator& _allocator = Allocator::DefaultAllocator(), unsigned int _alignment = 1);
				virtual ~PoolAllocator();

				virtual void Free(void* _data);

				virtual void FreeArray(void* _data);

			private:
				void* PointerFromId(unsigned int _id);
				unsigned int IdFromPointer(byte* _pointer);
				void* NextFreeFromPointer(void* _pointer);
				void* LastElement();

				virtual void* InternalAlloc(unsigned int _size, unsigned int _alignment);
				virtual void* InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment);
				
				Allocator& allocator;
				byte* data;
				unsigned int firstFree;
				unsigned int elementSize, maxElements;				
			};
		}
	}
}

#endif