#define DLL_EXPORT

#include "StackAllocator.hpp"

#include "../../Util/Logger.hpp"

using namespace Engine::Core::Memory;
using namespace Engine::Util;

StackAllocator::StackAllocator(unsigned int _size, Allocator& _allocator, unsigned int _alignment) : allocator(_allocator), size(_size), data(nullptr), freePointer(nullptr)
{
	//Request memory
	data = (byte*)allocator.Alloc(_size, _alignment);

	if (data == nullptr)
		Logger::Fatal("StackAllocator failed to allocate memory");

	freePointer = data;
}

StackAllocator::~StackAllocator()
{
	if (freePointer != data)
		Logger::Fatal("Unable to free non empty StackAllocator");

	allocator.Free(data);
}

void StackAllocator::Free(void* _data)
{
	if (_data != nullptr)
	{
		byte* freeData = (byte*)_data;
		unsigned int* sizePointer = (unsigned int*)_data - 1;

		if (freeData + *sizePointer != freePointer)
			Logger::Fatal("StackAllocator: Cannot free element not on top of stack");

		freePointer = (byte*)sizePointer;
	}
}
void StackAllocator::FreeArray(void* _data)
{
	Free(_data);
}

void* StackAllocator::InternalAlloc(unsigned int _size, unsigned int _alignment)
{
	unsigned int offsetToAlignment = (__int64)freePointer % _alignment;

	if (offsetToAlignment != 0)
		offsetToAlignment = _alignment - offsetToAlignment;

	if (freePointer + offsetToAlignment + _size + sizeof(unsigned int) > data + size)
		Logger::Fatal("StackAllocator out of memory");

	unsigned int* returnPointer = (unsigned int*)freePointer;
	freePointer = freePointer + offsetToAlignment + _size + sizeof(unsigned int);
	
	*returnPointer = _size;
	return returnPointer + 1;
}
void* StackAllocator::InternalAllocArray(unsigned int _elementSize, unsigned int _count, unsigned int _alignment)
{
	return InternalAlloc(_elementSize * _count, _alignment);
}