#ifndef _ENGINE_CORE_SYSTEM_SYSTEMCONFIGURATION_HPP
#define _ENGINE_CORE_SYSTEM_SYSTEMCONFIGURATION_HPP

#include "../../dllconfig.hpp"
#include "../IO/SystemConfigurationReader.hpp"
#include <string>
#include "../Collections/HashString.hpp"
#include <unordered_map>
#include "../../Util/Logger.hpp"

namespace Engine
{
	namespace Core
	{
		namespace System
		{
			class _DLL_DECLSPEC SystemConfiguration
			{
			public:
				template<typename T> static T& Get(Collections::HashString _name)
				{
					T* result = GetPointer<T>(_name);

					if (result == nullptr)
					{
						Util::Logger::Error("Unknown system configuration variable");
						system("PAUSE");
					}

					return *result;
				}				

			private:
				template<typename T> static T* GetPointer(Collections::HashString _name)
				{
					auto result = configValues.find(_name);

					if (result == configValues.end())
						return nullptr;

					return (T*)(result->second);
				}

				static std::unordered_map<Collections::HashString, void*> configValues;

				//This is needed for creating a static ctor
				class Initializer
				{
				public:
					Initializer();
					~Initializer();
				};

				friend class Initializer;
				friend class Engine::Core::IO::SystemConfigurationReader;
				static Initializer init;
			};
		}
	}
}

#endif