#define DLL_EXPORT

#include "CullingTask.hpp"
#include "../../../Scene/World.hpp"
#include "../../../Scene/SceneObject.hpp"
#include "../../../Scene/Cameras/Camera.hpp"
#include "../../../Core/Concurrency/BlockLocker.hpp"
#include "../SystemConfiguration.hpp"
#include "../../../Scene/Boundings/BoundingObject.hpp"
#include "../../../Scene/Cameras/Camera.hpp"
#include "../../../Scene/Components/BoundingComponent.hpp"
#include <algorithm>

using namespace Engine;
using namespace Engine::Core::System;
using namespace Engine::Core::Concurrency;
using namespace Engine::Core::System::Tasks;
using namespace Engine::Scene;
using namespace Engine::Scene::Cameras;
using namespace Engine::Scene::Boundings;

void CullingTask::Process(Scene::Cameras::Camera* _camera, const std::vector<Scene::Components::BoundingComponent*>& _boundings, unsigned int _threadId)
{
	size_t componentsPerThread = _boundings.size() / SystemConfiguration::Get<unsigned int>("WorkerThreadCount") + 1;
	size_t startIdx = _threadId * componentsPerThread;
	size_t endIdx = std::min<size_t>((_threadId + 1) * componentsPerThread, _boundings.size());

	auto startIterator = _boundings.begin() + startIdx;
	auto endIterator = _boundings.begin() + endIdx;

	for (auto it = startIterator; it != endIterator; ++it)
	{
		(*it)->Update();
		bool visible = (*it)->WorldObject()->IsVisible(_camera);
		_camera->Visibility((int)startIdx, visible);

		(*it)->VisibilityIndex((int)startIdx);

		if (visible)
			(*it)->AddVisible();

		startIdx++;
	}
}