#define DLL_EXPORT

#include "WorkerThread.hpp"
#include "SystemConfiguration.hpp"
#include "../../EngineManager.hpp"
#include "Tasks/PositionUpdateTask.hpp"
#include "../../Scene/World.hpp"
#include "Tasks\CullingTask.hpp"
#include "Tasks\UpdateTask.hpp"
#include "Tasks\PositionUpdateTask.hpp"
#include "../../Scene/Components/CameraComponent.hpp"

using namespace Engine;
using namespace Engine::Core::System;
using namespace Engine::Core::System::Tasks;
using namespace Engine::Core::Concurrency;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Boundings;

WorkerThread::WorkerThread(EngineManager* _manager) : manager(_manager), shouldStop(false)
{ }

WorkerThread::~WorkerThread()
{ }

void WorkerThread::Run()
{
	while (!shouldStop)
	{
		//Framestart abwarten, falls notwendig Event resetten
		manager->FrameStart().Wait();
		if (!shouldStop)
		{
			World* world = manager->World();

			//Alle Threads starten
			unsigned int startedThreads = ++manager->FrameStartCounter();
			unsigned int thisId = startedThreads - 1;
			if (startedThreads >= SystemConfiguration::Get<unsigned int>("WorkerThreadCount"))
				manager->FrameStart().Reset();

			PositionUpdateTask::Process(world->PositionComponents(), thisId);

			//Wenn letzter Thread: Kameras updaten
			unsigned int endedThreads = ++manager->LastUpdateThreadCounter();
			if (endedThreads == (unsigned int)SystemConfiguration::Get<unsigned int>("WorkerThreadCount"))
			{
				for (auto it = world->Cameras().begin(); it != world->Cameras().end(); ++it)
					(*it)->Update();
			}

			//Wenn alles fertig PositionUpdate finished signalisieren
			endedThreads = ++manager->FinishedPositionUpdateCounter();
			if (endedThreads == (unsigned int)(SystemConfiguration::Get<unsigned int>("WorkerThreadCount")))
			{
				manager->PositionUpdateFinished().Set();
			}

			//Warten bis alle fertig sind
			manager->PositionUpdateFinished().Wait();

			//CullingJobs durchfhren
			for (auto it = world->Cameras().begin(); it != world->Cameras().end(); ++it)
			{
				CullingTask::Process((*it)->Camera(), world->BoundingObjects(), thisId);
			}

			//Wenn letzter Job -> Event setzen
			endedThreads = ++manager->FinishedCullingCounter();
			if (endedThreads >= SystemConfiguration::Get<unsigned int>("WorkerThreadCount"))
			{
				manager->CullingFinished().Set();

				//Das hier gehrt noch sauberer gemacht
				manager->World()->OrderTransparentObjects();
			}

			manager->CullingFinished().Wait();

			//UpdateJobs durchfhren
			UpdateTask::Process(world->UpdateComponents(), thisId);

			//Wenn letzter Job -> Event setzen
			endedThreads = ++manager->FinishedUpdateCounter();
			if (endedThreads == SystemConfiguration::Get<unsigned int>("WorkerThreadCount"))
				manager->UpdateFinished().Set();
		}
	}
}

void WorkerThread::Stop()
{
	shouldStop = true;
}