#ifndef _ENGINE_ENGINEMANAGER_HPP
#define _ENGINE_ENGINEMANAGER_HPP

#include "dllconfig.hpp"
#include "Prototypes.hpp"

#include "Util\Clock.hpp"
#include "Core\Concurrency\Atomic.hpp"
#include "Core\Concurrency\Event.hpp"
#include "Util\StopWatch.hpp"

#include <sstream>
#include <vector>

namespace Engine
{
	class _DLL_DECLSPEC EngineManager
	{
	public:
		EngineManager(Scene::World* _world, Core::Control::RenderControl* _renderControl);
		~EngineManager();

		void RenderFrame();
		void WaitForFrame();

		Scene::World* World() const;
		Core::Control::RenderControl* RenderControl() const;

		Util::Clock& Clock();
		unsigned int FrameId() const;

		void StartWorker();
		void WaitWorker();

		Engine::Core::Concurrency::Atomic<unsigned int>& FrameStartCounter();
		Engine::Core::Concurrency::Atomic<unsigned int>& FinishedPositionUpdateCounter();
		Engine::Core::Concurrency::Atomic<unsigned int>& FinishedCullingCounter();
		Engine::Core::Concurrency::Atomic<unsigned int>& FinishedUpdateCounter();
		Engine::Core::Concurrency::Atomic<unsigned int>& LastUpdateThreadCounter();

		Engine::Core::Concurrency::Event& FrameStart();
		Engine::Core::Concurrency::Event& PositionUpdateFinished();
		Engine::Core::Concurrency::Event& CullingFinished();
		Engine::Core::Concurrency::Event& UpdateFinished();

		Graphic::FontRenderer* DebugFontRenderer() const;

		bool IsDebugGUIFrame() const;

	private:

		Util::Clock clock;
		unsigned int frameId;

		//Events for Workerthread synchronisation
		Engine::Core::Concurrency::Atomic<unsigned int> frameStartCounter;
		Engine::Core::Concurrency::Atomic<unsigned int> finishedPositionUpdateCounter;
		Engine::Core::Concurrency::Atomic<unsigned int> finishedCullingCounter;
		Engine::Core::Concurrency::Atomic<unsigned int> finishedUpdateCounter;
		Engine::Core::Concurrency::Atomic<unsigned int> lastUpdateThreadCounter;
		Engine::Core::Concurrency::Event frameStart;
		Engine::Core::Concurrency::Event positionUpdateFinished;
		Engine::Core::Concurrency::Event cullingFinished;
		Engine::Core::Concurrency::Event updateFinished;

		Core::Control::RenderControl* renderControl;
		Scene::World* world;
		Engine::Core::System::WorkerThread** workers;

		//Debugoutput
		Graphic::FontRenderer* debugFontRenderer;
		double nextGuiFrame;
		std::stringstream cullingCountString;
		std::stringstream screenCameraTime;
		Util::StopWatch frameTimeSW;
		Util::StopWatch updateSW;

		void RenderControl_SizeChanged(void* _sender, Core::EventHandling::EventArgs& _args);
	};
}

#endif