#define DLL_EXPORT

#include "Cone.hpp"

#define _USE_MATH_DEFINES
#include <math.h>
#include "../../Math/Math.hpp"
#include "../GraphicContext.hpp"

#define CONE_SUBDIVISION 8

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::BasicBodies;

Cone::Cone(GraphicContext* _context) : context(_context)
{
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glGenVertexArrays(1, &vao);

	Math::Vec3f data[2 + CONE_SUBDIVISION];

	unsigned int indices[2 * CONE_SUBDIVISION * 3];

	//float innerRadius = (_height * sinf(_angle)) / sinf((float)M_PI_2 - _angle);
	//float r = 1.0823922f * innerRadius;
	//float r = 2.0823922f * innerRadius;
	float anglePerPart = (2.0f * (float)M_PI) / (float)CONE_SUBDIVISION;

	data[0] = Math::Vec3f(0.0f, 0.0f, 0.0f); //Appex
	data[1] = Math::Vec3f(0.0f, 0.0f, -1.0f); //Center of Circle

	for (unsigned int i = 0; i < CONE_SUBDIVISION; ++i)
	{
		float cAngle = anglePerPart * i;
		data[i+2] = Math::Vec3f(cosf(cAngle), sin(cAngle), -1.0f);
	}

	for (unsigned int i = 0; i < CONE_SUBDIVISION; ++i)
	{
		indices[i*6] = 0;
		indices[i*6 + 1] = i + 2;
		indices[i*6 + 2] = ((i+1) % CONE_SUBDIVISION) + 2;
		indices[i*6 + 3] = i + 2;
		indices[i*6 + 4] = 1;
		indices[i*6 + 5] = ((i+1) % CONE_SUBDIVISION) + 2;
	}

	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, (2 + CONE_SUBDIVISION) * 3 * sizeof(float), data, GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, 6 * CONE_SUBDIVISION * sizeof(unsigned int), indices, GL_STATIC_DRAW);

	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

Cone::~Cone()
{
	glDeleteVertexArrays(1, &vao);
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
}

void Cone::Draw(GLenum _primitiveType) const
{
	glBindVertexArray(vao);
	context->DrawElements(_primitiveType, 6 * CONE_SUBDIVISION, GL_UNSIGNED_INT, 0);
}

Math::Mat4f Cone::CreateMatrix(float _angle, float _height)
{
	float innerRadius = (_height * sinf(_angle)) / sinf((float)M_PI_2 - _angle);
	float r = 1.0823922f * innerRadius;
	return Math::Mat4f::Scale(Math::Vec3f(r, r, _height));
}