#define DLL_EXPORT

#include "IcoSphere.hpp"
#include "../../Math/Math.hpp"
#include "../GraphicContext.hpp"

using namespace Engine;
using namespace Engine::Math;
using namespace Engine::Graphic;
using namespace Engine::Graphic::BasicBodies;

IcoSphere::IcoSphere(GraphicContext* _context) : context(_context)
{
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glGenVertexArrays(1, &vao);

	float t = (1.0f + sqrtf(5.0f)) / 2.0f;
	Math::Vec3f data[12] = {
		Math::Vec3f(-1.0f, t, 0.0f).Normalize(),
		Math::Vec3f(1.0f, t, 0.0f).Normalize(),
		Math::Vec3f(-1.0f, -t, 0.0f).Normalize(),
		Math::Vec3f(1.0f, -t, 0.0f).Normalize(),

		Math::Vec3f(0.0f, -1.0f, t).Normalize(),
		Math::Vec3f(0.0f, 1.0f, t).Normalize(),
		Math::Vec3f(0.0f, -1.0f, -t).Normalize(),
		Math::Vec3f(0.0f, 1.0f, -t).Normalize(),

		Math::Vec3f(t, 0.0f, -1.0f).Normalize(),
		Math::Vec3f(t, 0.0f, 1.0f).Normalize(),
		Math::Vec3f(-t, 0.0f, -1.0f).Normalize(),
		Math::Vec3f(-t, 0.0f, 1.0f).Normalize()
	};

	unsigned int indices[60] = {
		0, 11, 5, 0, 5, 1, 0, 1, 7, 0, 7, 10, 0, 10, 11,
		1, 5, 9, 5, 11, 4, 11, 10, 2, 10, 7, 6, 7, 1, 8,
		3, 9, 4, 3, 4, 2, 3, 2, 6, 3, 6, 8, 3, 8, 9,
		4, 9, 5, 2, 4, 11, 6, 2, 10, 8, 6, 7, 9, 8, 1
	};

	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, 12 * 3 * sizeof(float), data, GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, 60 * sizeof(unsigned int), indices, GL_STATIC_DRAW);

	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

IcoSphere::~IcoSphere()
{
	glDeleteVertexArrays(1, &vao);
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
}

void IcoSphere::Draw(GLenum _primitiveType) const
{
	glBindVertexArray(vao);
	context->DrawElements(_primitiveType, 60, GL_UNSIGNED_INT, 0);
}

Math::Mat4f IcoSphere::CreateMatrix(float _radiusInscribedSphere)
{
	/*
	Explanation:

	r_u = 0.9510565163 * a
	=> (r_u == 1)
	a = 1.051462224

	r_i(r_u == 1) = 0.7557613141 * a
	r_i(r_u == 1) = 0.794654472

	r_i = 0.794654472 * factor
	factor = r_i / 0.794654472
	*/
	float factor = _radiusInscribedSphere / 0.794654472f;
	return Mat4f::Scale(Vec3f(factor, factor, factor));
}