#define DLL_EXPORT

#include "LineCone.hpp"

#define _USE_MATH_DEFINES
#include <math.h>
#include "../../Math/Math.hpp"
#include "../GraphicContext.hpp"
#include "../Shaders/Shader.hpp"

#define CONE_SUBDIVISION 40
#define CONE_LINES 6

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::BasicBodies;

LineCone::LineCone(GraphicContext* _context) : context(_context)
{
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glGenVertexArrays(1, &vao);

	Math::Vec3f data[CONE_SUBDIVISION + 1];
	data[0] = Math::Vec3f(0.0f, 0.0f, 0.0f); //Appex

	float anglePerPart = (2.0f * (float)M_PI) / (float)CONE_SUBDIVISION;
	for (unsigned int i = 0; i < CONE_SUBDIVISION; ++i)
	{
		float cAngle = anglePerPart * i;
		data[i+1] = Math::Vec3f(cosf(cAngle), sinf(cAngle), -1.0f);
	}

	unsigned int indices[(CONE_SUBDIVISION * 2 + CONE_LINES * 2)];
	for (unsigned int i = 0; i < CONE_SUBDIVISION; ++i)
	{
		indices[i*2] = i + 1;
		indices[i*2 + 1] = (i + 1) % CONE_SUBDIVISION + 1;
	}
	for (unsigned int i = 0; i < CONE_LINES; ++i)
	{
		indices[CONE_SUBDIVISION * 2 + i * 2] = 0;
		indices[CONE_SUBDIVISION * 2 + i * 2 + 1] = 1 + CONE_SUBDIVISION / CONE_LINES * i;
	}

	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, (1 + CONE_SUBDIVISION) * 3 * sizeof(float), data, GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, (CONE_SUBDIVISION * 2 + CONE_LINES * 2) * sizeof(unsigned int), indices, GL_STATIC_DRAW);

	glEnableVertexAttribArray(SHADER_POSITION);
	glVertexAttribPointer(SHADER_POSITION, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

LineCone::~LineCone()
{
	glDeleteVertexArrays(1, &vao);
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
}

void LineCone::Draw() const
{
	glBindVertexArray(vao);
	context->DrawElements(GL_LINES, 4 * CONE_SUBDIVISION, GL_UNSIGNED_INT, 0);	

	/*glBindBuffer(GL_ARRAY_BUFFER, vbo);
	Math::Vec3f* data = (Math::Vec3f*)glMapBufferRange(GL_ARRAY_BUFFER, 0, (1 + CONE_SUBDIVISION) * 3 * sizeof(float), GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_RANGE_BIT);

	//Create Data
	float innerRadius = (_height * sinf(_angle)) / sinf((float)M_PI_2 - _angle);
	float r = 1.0823922f * innerRadius;
	float anglePerPart = (2.0f * (float)M_PI) / (float)CONE_SUBDIVISION;

	data[0] = Math::Vec3f(0.0f, 0.0f, 0.0f); //Appex

	for (unsigned int i = 0; i < CONE_SUBDIVISION; ++i)
	{
		float cAngle = anglePerPart * i;
		data[i+1] = Math::Vec3f(cosf(cAngle) * r, sin(cAngle) * r, -_height);
	}

	glUnmapBuffer(GL_ARRAY_BUFFER);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);

	glEnableVertexAttribArray(LineShader::HANDLE_POSITION);
	glVertexAttribPointer(LineShader::HANDLE_POSITION, 3, GL_FLOAT, GL_FALSE, 0, 0);*/	
}

Math::Mat4f LineCone::CreateMatrix(float _angle, float _height)
{
	float innerRadius = (_height * sinf(_angle)) / sinf((float)M_PI_2 - _angle);
	float r = 1.0823922f * innerRadius;
	return Math::Mat4f::Scale(Math::Vec3f(r, r, _height));
}