#define DLL_EXPORT

#include "LineCube.hpp"

#define _USE_MATH_DEFINES
#include <math.h>
#include "../../Math/Math.hpp"
#include "../GraphicContext.hpp"
#include "../Shaders/Shader.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::BasicBodies;

LineCube::LineCube(GraphicContext* _context) : context(_context)
{
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glGenVertexArrays(1, &vao);

	Math::Vec3f data[8] = 
	{
		Math::Vec3f(0.0f, 0.0f, 0.0f),
		Math::Vec3f(1.0f, 0.0f, 0.0f),
		Math::Vec3f(1.0f, 1.0f, 0.0f),
		Math::Vec3f(0.0f, 1.0f, 0.0f),

		Math::Vec3f(0.0f, 0.0f, 1.0f),
		Math::Vec3f(1.0f, 0.0f, 1.0f),
		Math::Vec3f(1.0f, 1.0f, 1.0f),
		Math::Vec3f(0.0f, 1.0f, 1.0f)
	};
	unsigned int indices[24] =
	{
		0, 1, 1, 2, 2, 3, 3, 0,
		4, 5, 5, 6, 6, 7, 7, 4,
		0, 4, 1, 5, 2, 6, 3, 7
	};	

	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, 8 * 3 * sizeof(float), data, GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, 24 * sizeof(unsigned int), indices, GL_STATIC_DRAW);

	glEnableVertexAttribArray(SHADER_POSITION);
	glVertexAttribPointer(SHADER_POSITION, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
}

LineCube::~LineCube()
{
	glDeleteVertexArrays(1, &vao);
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
}

void LineCube::Draw() const
{
	glBindVertexArray(vao);
	context->DrawElements(GL_LINES, 24, GL_UNSIGNED_INT, 0);
}

void LineCube::DrawInstanced(GLsizei _instanceCount) const
{
	glBindVertexArray(vao);
	context->DrawElementsInstanced(GL_LINES, 24, GL_UNSIGNED_INT, 0, _instanceCount);
}