#define DLL_EXPORT

#include "LineSphere.hpp"

#define _USE_MATH_DEFINES
#include <math.h>
#include "../../Math/Math.hpp"
#include "../GraphicContext.hpp"
#include "../Shaders/Shader.hpp"

#define SPHERE_SUBDIVISION 40

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::BasicBodies;

LineSphere::LineSphere(GraphicContext* _context) : context(_context)
{
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glGenVertexArrays(1, &vao);

	Math::Vec3f data[SPHERE_SUBDIVISION * 3];
	unsigned int indices[SPHERE_SUBDIVISION * 6];

	for (int i = 0; i < SPHERE_SUBDIVISION; i++)
	{
		float w = (2.0f * (float)M_PI) / (float)SPHERE_SUBDIVISION * (float)i;
		data[i] = Math::Vec3f(cosf(w), sinf(w), 0.0f);
		data[SPHERE_SUBDIVISION + i] = Math::Vec3f(cosf(w), 0.0f, sinf(w));
		data[2 * SPHERE_SUBDIVISION + i] = Math::Vec3f(0.0f, cosf(w), sinf(w));

		indices[2*i] = i;
		indices[2*i+1] = (i+1) % SPHERE_SUBDIVISION;
		indices[2 * SPHERE_SUBDIVISION + 2*i] = SPHERE_SUBDIVISION + i;
		indices[2 * SPHERE_SUBDIVISION + 2*i + 1] = SPHERE_SUBDIVISION + (i+1) % SPHERE_SUBDIVISION;
		indices[4 * SPHERE_SUBDIVISION + 2*i] = 2 * SPHERE_SUBDIVISION + i;
		indices[4 * SPHERE_SUBDIVISION + 2*i + 1] = 2 * SPHERE_SUBDIVISION + (i+1) % SPHERE_SUBDIVISION;
	}

	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, SPHERE_SUBDIVISION * 9 * sizeof(float), data, GL_STATIC_DRAW);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, SPHERE_SUBDIVISION * 6 * sizeof(unsigned int), indices, GL_STATIC_DRAW);

	glEnableVertexAttribArray(SHADER_POSITION);
	glVertexAttribPointer(SHADER_POSITION, 3, GL_FLOAT, GL_FALSE, 0, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
}

LineSphere::~LineSphere()
{
	glDeleteVertexArrays(1, &vao);
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
}

void LineSphere::Draw() const
{
	glBindVertexArray(vao);
	context->DrawElements(GL_LINES, SPHERE_SUBDIVISION * 6, GL_UNSIGNED_INT, 0);
}