#define DLL_EXPORT

#include "Buffer.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;

Buffer::Buffer() : handle(-1), size(0)
{
	glGenBuffers(1, &handle);
}

Buffer::Buffer(GLenum _target, GLsizei _size, const GLvoid* _data, GLenum _usage) : handle(-1), size(_size)
{
	glGenBuffers(1, &handle);
	glBindBuffer(_target, handle);
	glBufferData(_target, _size, _data, _usage);
	glBindBuffer(_target, 0);

}

Buffer::~Buffer()
{
	glDeleteBuffers(1, &handle);
}

void Buffer::Copy(const Buffer& _source, const Buffer& _target, GLintptr _readOffset, GLintptr _writeOffset, GLsizeiptr _size)
{
	glBindBuffer(GL_COPY_READ_BUFFER, _source.handle);
	glBindBuffer(GL_COPY_WRITE_BUFFER, _target.handle);
	glCopyBufferSubData(GL_COPY_READ_BUFFER, GL_COPY_WRITE_BUFFER, _readOffset, _writeOffset, _size);
}

void Buffer::Bind(GLenum _target) const
{
	glBindBuffer(_target, handle);
}

void Buffer::Unbind(GLenum _target) const
{
	glBindBuffer(_target, 0);
}

GLuint Buffer::Handle() const
{
	return handle;
}
GLsizei Buffer::Size() const
{
	return size;
}

void* Buffer::MapRead(GLenum _target) const
{
	glBindBuffer(_target, handle);
	void* ptr = glMapBufferRange(_target, 0, size, GL_MAP_READ_BIT);
	return ptr;
}

void* Buffer::MapWrite(GLenum _target) const
{
	glBindBuffer(_target, handle);
	void* ptr = glMapBufferRange(_target, 0, size, GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_BUFFER_BIT | GL_MAP_UNSYNCHRONIZED_BIT);
	return ptr;
}

void Buffer::Unmap(GLenum _target) const
{
	glUnmapBuffer(_target);
	glBindBuffer(_target, 0);
}