#define DLL_EXPORT

#include "ShaderStorageBuffer.hpp"
#include "../../Util/Logger.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;
using namespace Engine::Util;

ShaderStorageBuffer::ShaderStorageBuffer()
{
	glGenBuffers(1, &handle);
}

ShaderStorageBuffer::~ShaderStorageBuffer()
{
	glDeleteBuffers(1, &handle);
}

void ShaderStorageBuffer::Bind(GLuint _location) const
{
	glBindBufferBase(GL_SHADER_STORAGE_BUFFER, _location, handle);
}

void ShaderStorageBuffer::BufferData(GLvoid* _data, unsigned int _size, GLenum _usage) const
{
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, handle);
	glBufferData(GL_SHADER_STORAGE_BUFFER, _size, _data, _usage);
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);
}

void ShaderStorageBuffer::BufferSubData(GLvoid* _data, unsigned int _offset, unsigned int _size) const
{
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, handle);
	glBufferSubData(GL_SHADER_STORAGE_BUFFER, _offset, _size, _data);
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);
}

GLvoid* ShaderStorageBuffer::MapMemory(GLenum _access) const
{
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, handle);
	GLvoid* ptr = glMapBuffer(GL_SHADER_STORAGE_BUFFER, GL_READ_ONLY);
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);

	if (ptr == nullptr)
	{
		Logger::Error("Unable to map TextureBuffer to Memory");
		system("PAUSE");
	}

	return ptr;
}

void ShaderStorageBuffer::UnMapMemory() const
{
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, handle);
	glUnmapBuffer(GL_SHADER_STORAGE_BUFFER);
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);
}

void ShaderStorageBuffer::Clear(const void* _data, GLenum _internalFormat, GLenum _format, GLenum _type) const
{
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, handle);
	glClearBufferData(GL_SHADER_STORAGE_BUFFER, _internalFormat, _format, _type, _data);
	glBindBuffer(GL_SHADER_STORAGE_BUFFER, 0);
}