#define DLL_EXPORT

#include "Texture2D.hpp"

#include "../../Util/Logger.hpp"
#include "../../Util/Convert.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;
using namespace Engine::Util;

Texture2D::Texture2D(GLenum _internalFormat, GLenum _format, bool _generateMipmaps) : format(_format), internalFormat(_internalFormat), generateMipmaps(_generateMipmaps)
{
	glGenTextures(1, &handle);

	glBindTexture(GL_TEXTURE_2D, handle);

	if (_generateMipmaps)
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	}
	else
	{
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	}
}

Texture2D::~Texture2D()
{
	glDeleteTextures(1, &handle);
}

void Texture2D::Bind(GLint _textureUnit) const
{
	glActiveTexture(GL_TEXTURE0 + _textureUnit);
	glBindTexture(GL_TEXTURE_2D, handle);
}

void Texture2D::BindImage(GLint _textureUnit) const
{
	glBindImageTexture(_textureUnit, handle, 0, GL_FALSE, 0, GL_READ_WRITE, internalFormat);
}

void Texture2D::BufferImage(unsigned int _width, unsigned int _height, GLenum _type, GLvoid* _data)
{
	glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, _width, _height, 0, format, _type, _data);

	if (generateMipmaps)
		glGenerateMipmap(GL_TEXTURE_2D);
}

void Texture2D::BufferSubImage(unsigned int _left, unsigned int _top, unsigned int _width, unsigned int _height, GLenum _type, GLvoid* _data)
{
	glTexSubImage2D(GL_TEXTURE_2D, 0, _left, _top, _width, _height, format, _type, _data);
	
	if (generateMipmaps)
		glGenerateMipmap(GL_TEXTURE_2D);
}

inline GLuint Texture2D::Handle() const
{
	return handle;
}

void Texture2D::SetFilter(GLint _minFilter, GLint _magFilter) const
{
	glBindTexture(GL_TEXTURE_2D, handle);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, _minFilter);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, _magFilter);
}

void Texture2D::SetWrap(GLint _wrapX, GLint _wrapY, GLint _wrapZ) const
{
	glBindTexture(GL_TEXTURE_2D, handle);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, _wrapX);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, _wrapY);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, _wrapZ);
}

void Texture2D::GenerateMipmaps() const
{
	glBindTexture(GL_TEXTURE_2D, handle);
	glGenerateMipmap(GL_TEXTURE_2D);
}

void Texture2D::SetCompareFunction(GLenum _compareMode, GLenum _compareFunc) const
{
	glBindTexture(GL_TEXTURE_2D, handle);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, _compareMode);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, _compareFunc);
}