#define DLL_EXPORT

#include "Texture3D.hpp"
#include "../../Util/Logger.hpp"
#include "../../Util/Convert.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;
using namespace Engine::Util;

Texture3D::Texture3D(GLenum _internalFormat, GLenum _format, bool _generateMipmaps) : format(_format), internalFormat(_internalFormat), generateMipmaps(_generateMipmaps)
{
	glGenTextures(1, &handle);
}

Texture3D::~Texture3D()
{
	glDeleteTextures(1, &handle);
}

void Texture3D::Bind(GLint _textureUnit) const
{
	glActiveTexture(GL_TEXTURE0 + _textureUnit);
	glBindTexture(GL_TEXTURE_3D, handle);
}

void Texture3D::BindImage(GLint _textureUnit) const
{
	glBindImageTexture(_textureUnit, handle, 0, GL_TRUE, 0, GL_READ_WRITE, internalFormat);
}

void Texture3D::BufferImage(unsigned int _width, unsigned int _height, unsigned int _depth, GLenum _type, GLvoid* _data)
{
	glTexImage3D(GL_TEXTURE_3D, 0, internalFormat, _width, _height, _depth, 0, format, _type, _data);

	if (generateMipmaps)
		glGenerateMipmap(GL_TEXTURE_3D);
}

void Texture3D::BufferSubImage(unsigned int _xoffset, unsigned int _yoffset, unsigned int _zoffset, unsigned int _width, unsigned int _height, unsigned int _depth, GLenum _type, GLvoid* _data)
{
	glTexSubImage3D(GL_TEXTURE_3D, 0, _xoffset, _yoffset, _zoffset, _width, _height, _depth, format, _type, _data);
	
	if (generateMipmaps)
		glGenerateMipmap(GL_TEXTURE_3D);
}

void Texture3D::SetWrap(GLint _wrapX, GLint _wrapY, GLint _wrapZ) const
{
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_R, _wrapX);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_S, _wrapY);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_WRAP_T, _wrapZ);
}

void Texture3D::SetFilter(GLint _minFilter, GLint _magFilter) const
{
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MIN_FILTER, _minFilter);
	glTexParameteri(GL_TEXTURE_3D, GL_TEXTURE_MAG_FILTER, _magFilter);
}

void Texture3D::GenerateMipmaps() const
{
	glGenerateMipmap(GL_TEXTURE_3D);
}

inline GLuint Texture3D::Handle() const
{
	return handle;
}