#define DLL_EXPORT

#include "TextureBuffer.hpp"
#include "../../Util/Logger.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;
using namespace Engine::Util;

TextureBuffer::TextureBuffer(GLenum _format) : format(_format)
{
	glGenBuffers(1, &handleBuffer);
	glGenTextures(1, &handleTexture);
}

TextureBuffer::~TextureBuffer()
{
	glDeleteBuffers(1, &handleBuffer);
	glDeleteTextures(1, &handleTexture);
}

void TextureBuffer::Bind(GLuint _location, GLenum _access) const
{
	glBindImageTexture(_location, handleTexture, 0, GL_FALSE, 0, _access, format);
}

void TextureBuffer::BufferData(GLvoid* _data, unsigned int _size, GLenum _usage) const
{
	glBindBuffer(GL_TEXTURE_BUFFER, handleBuffer);
	glBufferData(GL_TEXTURE_BUFFER, _size, _data, _usage);
	glBindTexture(GL_TEXTURE_BUFFER, handleTexture);
	glTexBuffer(GL_TEXTURE_BUFFER, format, handleBuffer);
	glBindBuffer(GL_TEXTURE_BUFFER, 0);
	glBindTexture(GL_TEXTURE_BUFFER, 0);
}

void TextureBuffer::BufferSubData(GLvoid* _data, unsigned int _offset, unsigned int _size) const
{
	glBindBuffer(GL_TEXTURE_BUFFER, handleBuffer);
	glBufferSubData(GL_TEXTURE_BUFFER, _offset, _size, _data);
	glBindTexture(GL_TEXTURE_BUFFER, handleTexture);
	glTexBuffer(GL_TEXTURE_BUFFER, format, handleBuffer);
	glBindBuffer(GL_TEXTURE_BUFFER, 0);
	glBindTexture(GL_TEXTURE_BUFFER, 0);
}

GLvoid* TextureBuffer::MapMemory(GLenum _access) const
{
	glBindBuffer(GL_TEXTURE_BUFFER, handleBuffer);
	GLvoid* ptr = glMapBuffer(GL_TEXTURE_BUFFER, GL_READ_ONLY);
	glBindBuffer(GL_TEXTURE_BUFFER, 0);

	if (ptr == nullptr)
	{
		Logger::Error("Unable to map TextureBuffer to Memory");
		system("PAUSE");
	}

	return ptr;
}

void TextureBuffer::UnMapMemory() const
{
	glBindBuffer(GL_TEXTURE_BUFFER, handleBuffer);
	glUnmapBuffer(GL_TEXTURE_BUFFER);
	glBindBuffer(GL_TEXTURE_BUFFER, 0);
}

void TextureBuffer::Clear(const void* _data, GLenum _format, GLenum _type) const
{
	glBindBuffer(GL_TEXTURE_BUFFER, handleBuffer);
	glClearBufferData(GL_TEXTURE_BUFFER, format, _format, _type, _data);
}