#define DLL_EXPORT

#include "DirectionalLightEffect.hpp"
#include "../../Scene/Cameras/RenderCamera.hpp"
#include "../Shaders/Shader.hpp"
#include "../BasicBodies/Plane.hpp"
#include "../GraphicContext.hpp"
#include "../../Scene/Lights/DirectionalLight.hpp"
#include "../../Core/System/SystemConfiguration.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::Effects;
using namespace Engine::Core;
using namespace Engine::Core::System;

ShaderPtr* DirectionalLightEffect::shaderPtr = nullptr;

DirectionalLightEffect::DirectionalLightEffect(const Scene::Cameras::Camera& _camera, GraphicContext* _context) : 
	camera(_camera), context(_context)
{
	if (shaderPtr == nullptr || shaderPtr->GetCounter() == nullptr)
	{
		if (shaderPtr != nullptr)
			delete shaderPtr;

		Shader* shader = new Shader(GL_TRIANGLES, _context);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("DirectionalLightVertexShader"), GL_VERTEX_SHADER);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("DirectionalLightFragmentShader"), GL_FRAGMENT_SHADER);
		shader->BindAttribute("position", SHADER_POSITION);
		shader->Link();

		shaderPtr = new ShaderPtr(shader);
		--(*shaderPtr->GetCounter());
	}

	shader = *shaderPtr;
}

DirectionalLightEffect::~DirectionalLightEffect()
{ }

void DirectionalLightEffect::Render(const Scene::Lights::DirectionalLight* _light)
{
	context->Activate();
	context->CullFace().EnableBackFaceCulling();

	shader->Activate();
	shader->UniformVariable<GLint>("normalTexture") = 0;
	shader->UniformVariable<GLint>("depthTexture") = 1;
	shader->UniformVariable<Math::Mat4f>("inverseCameraMatrix") = camera.InverseCameraMatrix();
	shader->UniformVariable<Math::Vec4f>("cameraPosition") = camera.Position();
	shader->UniformVariable<Math::Vec3f>("lightDirection") = _light->WorldDirection();
	shader->UniformVariable<Math::Vec3f>("lightIntensity") = _light->Intensity();
	shader->ChangeValues();

	context->ScreenPlane()->Draw(shader->PrimitiveType());
}