#define DLL_EXPORT

#include "PointLightEffect.hpp"
#include "../../Scene/Cameras/RenderCamera.hpp"
#include "../Shaders/Shader.hpp"
#include "../BasicBodies/IcoSphere.hpp"
#include "../../Scene/Lights/PointLight.hpp"
#include "../GraphicContext.hpp"
#include "../../Core/System/SystemConfiguration.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::Effects;
using namespace Engine::Core;
using namespace Engine::Core::System;

ShaderPtr* PointLightEffect::shaderPtr = nullptr;

PointLightEffect::PointLightEffect(const Scene::Cameras::Camera& _camera, GraphicContext* _context) : 
	camera(_camera), context(_context)
{
	if (shaderPtr == nullptr || shaderPtr->GetCounter() == nullptr)
	{
		if (shaderPtr != nullptr)
			delete shaderPtr;

		Shader* shader = new Shader(GL_TRIANGLES, _context);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("PointLightVertexShader"), GL_VERTEX_SHADER);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("PointLightFragmentShader"), GL_FRAGMENT_SHADER);
		shader->BindAttribute("position", SHADER_POSITION);
		shader->Link();

		shaderPtr = new ShaderPtr(shader);
		--(*shaderPtr->GetCounter());
	}

	shader = *shaderPtr;
}

PointLightEffect::~PointLightEffect()
{ }

void PointLightEffect::Render(const Scene::Lights::PointLight* _light)
{
	context->CullFace().EnableFrontFaceCulling();
	shader->Activate();
	
	shader->UniformVariable<GLint>("normalTexture") = 0;
	shader->UniformVariable<GLint>("depthTexture") = 1;
	shader->UniformVariable<GLfloat>("radius") = _light->Boundings().Radius();
	shader->UniformVariable<Math::Mat4f>("inverseCameraMatrix") = camera.InverseCameraMatrix();
	shader->UniformVariable<Math::Vec4f>("cameraPosition") = camera.Position();
	shader->UniformVariable<Math::Vec3f>("lightPosition") = _light->WorldPosition();
	shader->UniformVariable<Math::Vec3f>("lightIntensity") = _light->Intensity();
	shader->UniformVariable<Math::Vec3f>("lightAttenuation") = _light->Attenuation();
	shader->UniformVariable<Math::Mat4f>("cameraMatrix") = camera.CameraMatrix();
	shader->UniformVariable<Math::Mat4f>("modelMatrix") = _light->LightMatrix();
	
	shader->ChangeValues();

	context->UnitSphere()->Draw(shader->PrimitiveType());
}