#define DLL_EXPORT

#include "SpotLightEffect.hpp"
#include "../../Scene/Cameras/RenderCamera.hpp"
#include "../Shaders/Shader.hpp"
#include "../BasicBodies/Cone.hpp"
#include "../../Scene/Lights/SpotLight.hpp"
#include "../GraphicContext.hpp"
#include "../../Core/System/SystemConfiguration.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Graphic::Effects;
using namespace Engine::Core;
using namespace Engine::Core::System;

ShaderPtr* SpotLightEffect::shaderPtr = nullptr;

SpotLightEffect::SpotLightEffect(const Scene::Cameras::Camera& _camera, GraphicContext* _context) : 
	camera(_camera), context(_context)
{
	if (shaderPtr == nullptr || shaderPtr->GetCounter() == nullptr)
	{
		if (shaderPtr != nullptr)
			delete shaderPtr;

		Shader* shader = new Shader(GL_TRIANGLES, _context);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("SpotLightVertexShader"), GL_VERTEX_SHADER);
		shader->AttachShaderFile(SystemConfiguration::Get<std::string>("SpotLightFragmentShader"), GL_FRAGMENT_SHADER);
		shader->BindAttribute("position", SHADER_POSITION);
		shader->Link();

		shaderPtr = new ShaderPtr(shader);
		--(*shaderPtr->GetCounter());
	}

	shader = *shaderPtr;
}

SpotLightEffect::~SpotLightEffect()
{ }

void SpotLightEffect::Render(const Scene::Lights::SpotLight* _light)
{
	context->CullFace().EnableFrontFaceCulling();

	shader->Activate();	
	shader->UniformVariable<GLint>("normalTexture") = 0;
	shader->UniformVariable<GLint>("depthTexture") = 1;
	shader->UniformVariable<Math::Mat4f>("inverseCameraMatrix") = camera.InverseCameraMatrix();
	shader->UniformVariable<Math::Vec4f>("cameraPosition") = camera.Position();
	shader->UniformVariable<Math::Vec3f>("lightPosition") = _light->WorldPosition().GetVec3f();
	shader->UniformVariable<Math::Vec3f>("lightDirection") = _light->WorldDirection();
	shader->UniformVariable<Math::Vec3f>("lightIntensity") = _light->Intensity();
	shader->UniformVariable<Math::Vec3f>("lightAttenuation") = _light->Attenuation();
	shader->UniformVariable<GLfloat>("lightAngle") = _light->Angle();
	shader->UniformVariable<GLfloat>("lightAngularAttenuation") = _light->AngularAttenuation();
	shader->UniformVariable<Math::Mat4f>("cameraMatrix") = camera.CameraMatrix();
	shader->UniformVariable<Math::Mat4f>("modelMatrix") = _light->LightMatrix();
	shader->ChangeValues();

	context->UnitCone()->Draw(shader->PrimitiveType());
}