#ifndef _ENGINE_GRAPHIC_GI_GENERATION_OCTREEGENERATIONSTAGE_HPP
#define _ENGINE_GRAPHIC_GI_GENERATION_OCTREEGENERATIONSTAGE_HPP

#include "../../../Prototypes.hpp"
#include "IGenerationStage.hpp"

#include "../../Shaders/Shader.hpp"
#include "../../Buffers/AtomicCounterBuffer.hpp"
#include "../../Buffers/TextureBuffer.hpp"
#include "../../Buffers/Texture2D.hpp"
#include "../../FrameBuffers/FrameBuffer.hpp"
#include "../../Buffers/ShaderStorageBuffer.hpp"
#include "../../Buffers/IndirectDrawBuffer.hpp"
#include <gl\glew.h>
#include <string>
#include "../../../Util/GLStopWatch.hpp"
#include "../../../Util/StopWatch.hpp"

#define OCTREE_LEAFLEVEL 10

namespace Engine
{
	namespace Graphic
	{
		namespace GI
		{
			namespace Generation
			{
				class _DLL_DECLSPEC OctreeGenerationStage : public IGenerationStage
				{
				public:
					OctreeGenerationStage(Scene::Components::VoxelOctreeComponent* _component, EngineManager* _manager, GraphicContext* _context, unsigned int _entryCount);
					virtual ~OctreeGenerationStage();

					virtual void Process();
					void Resize(unsigned int _entryCount);

					#ifdef _ENGINE_OCTREEE_TIMERS
					Util::GLStopWatch& TotalWatch();
					Util::GLStopWatch& InitWatch();
					Util::GLStopWatch& VoxelizeWatch();
					Util::GLStopWatch* OctreeBuildWatches();
					#endif

					Buffers::Texture2D lightDepth;

				private:
					void WriteToFile(const std::string _path) const;

					EngineManager* manager;
					GraphicContext* context;
					Scene::Components::VoxelOctreeComponent* component;

					Buffers::ShaderStorageBuffer deferredBuffer;

					Buffers::AtomicCounterBuffer linkCounter;
					Buffers::IndirectDrawBuffer deferredCounter;
					int useCounter;

					GLuint octreeVAO;
					Shaders::Shader octreeShader;
					Shaders::Shader ptrShader;

					Shaders::Shader injectionShader;
					FrameBuffers::FrameBuffer lightBuffer;
					
					FrameBuffers::FrameBuffer emptyBuffer;
					Buffers::Texture2D emptyTexture;
					

					#ifdef _ENGINE_OCTREEE_TIMERS
					Util::GLStopWatch totalWatch;
					Util::GLStopWatch initWatch;
					Util::GLStopWatch voxelizeWatch;
					Util::GLStopWatch octreeBuildWatches[OCTREE_LEAFLEVEL];
					#endif
				};
			}
		}
	}
}

#endif