#define DLL_EXPORT

#include "AmbientOcclusionIlluminationStage.hpp"

#include "../OctreeData.hpp"
#include "../Preparation/OctreePreparationStage.hpp"
#include "../../GraphicContext.hpp"
#include "../../BasicBodies/Plane.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::GI;
using namespace Engine::Graphic::GI::Illumination;
using namespace Engine::Graphic::GI::Generation;
using namespace Engine::Graphic::GI::Preparation;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Math;
using namespace Engine::Util;

AmbientOcclusionIlluminationStage::AmbientOcclusionIlluminationStage(VoxelOctreeComponent* _component, GraphicContext* _context) : component(_component), context(_context),
	ambientOcclusionShader(GL_TRIANGLES, _context)
#ifdef _ENGINE_OCTREEE_TIMERS
	, totalWatch()
#endif
{
	ambientOcclusionShader.AttachShaderFile("../../../Data/OctreeShader/octree_ambient_occlusion.vert", GL_VERTEX_SHADER);
	ambientOcclusionShader.AttachShaderFile("../../../Data/OctreeShader/octree_ambient_occlusion.frag", GL_FRAGMENT_SHADER);
	ambientOcclusionShader.Link();
}

AmbientOcclusionIlluminationStage::~AmbientOcclusionIlluminationStage()
{ }

void AmbientOcclusionIlluminationStage::Process(const Mat4f& _inverseCameraMatrix, const Math::Vec3f& _cameraPosition, const FrameBuffers::FrameBuffer& _illuminationBuffer)
{
	#ifdef _ENGINE_OCTREEE_TIMERS
	totalWatch.Start();
	#endif

	OctreeData* data = (OctreeData*)component->Data();
	data->LinkBuffer().Bind(1);
	data->LockBuffer().Bind(2, GL_READ_ONLY);
	data->BrickTexture().Bind(0);

	ambientOcclusionShader.Activate();
	ambientOcclusionShader.UniformVariable<Mat4f>("inverseCameraMatrix") = _inverseCameraMatrix;
	ambientOcclusionShader.UniformArrays<Math::Mat4f>("cameraMatrixArray")[2] = component->MatrixZ();
	ambientOcclusionShader.UniformVariable<GLint>("steps") = 10;
	ambientOcclusionShader.ChangeValues();

	context->ScreenPlane()->Draw(GL_TRIANGLES);

	ambientOcclusionShader.Deactivate();

	#ifdef _ENGINE_OCTREEE_TIMERS
	totalWatch.Stop();
	#endif
}

#ifdef _ENGINE_OCTREEE_TIMERS
GLStopWatch& AmbientOcclusionIlluminationStage::TotalWatch()
{
	return totalWatch;
}
#endif