#define DLL_EXPORT

#include "OctreeData.hpp"

#include <cassert>
#include <cmath>
#include <Windows.h>

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::GI;
using namespace Engine::Graphic::Buffers;

OctreeData::OctreeData() : linkBuffer(), lockBuffer(GL_R32UI), brickTexture(GL_RGBA32F, GL_RGBA, false)
{
	brickTexture.Bind(0);
	brickTexture.SetFilter(GL_LINEAR, GL_LINEAR);
	brickTexture.SetWrap(GL_CLAMP, GL_CLAMP, GL_CLAMP);
}

OctreeData::~OctreeData()
{ }

void OctreeData::Resize(unsigned int _entryCount)
{
	linkBuffer.BufferData(nullptr, _entryCount * 8 * 4, GL_STATIC_DRAW);
	lockBuffer.BufferData(nullptr, _entryCount * sizeof(GLuint), GL_STATIC_DRAW);

	unsigned int x,y,z;

	z = (int)ceil(_entryCount / (262144));

	if (_entryCount > 262144)
	{
		y = 512;
		x = 512;
	}
	else if (_entryCount > 512)
	{
		y = (int)ceil((_entryCount - (262144 * z)) / 512);
		x = 512;
	}
	else
	{
		y = 1;
		x = _entryCount - z * 262144 - y * 512;
	}

	float* clearValues = new float[x * 3 * y * 3 * z * 3 * 4];
	ZeroMemory(clearValues, x*y*z * 3 * 3 * 3 * 4 * sizeof(float));

	brickTexture.Bind(5);
	brickTexture.BufferImage(x * 3, y * 3, z * 3, GL_FLOAT, clearValues);

	delete[] clearValues;
}

const TextureBuffer& OctreeData::LockBuffer() const
{
	return lockBuffer;
}

const ShaderStorageBuffer& OctreeData::LinkBuffer() const
{
	return linkBuffer;
}

const IndirectDrawBuffer& OctreeData::LevelBuffers(unsigned int _level) const
{
	assert(_level < 10);
	return levelBuffers[_level];
}

const Buffers::Texture3D& OctreeData::BrickTexture() const
{
	return brickTexture;
}