#ifndef _ENGINE_GRAPHIC_GRAPHICCONTEXT_HPP
#define _ENGINE_GRAPHIC_GRAPHICCONTEXT_HPP

#include "../dllconfig.hpp"
#include "../Prototypes.hpp"

#include <Windows.h>
#include <gl\glew.h>
#include <gl\wglew.h>
#include "DelayedChange\IDelayedValueObserver.hpp"
#include "GraphicStates\BlendingState.hpp"
#include "GraphicStates\FaceCullingState.hpp"
#include "GraphicStates\DepthTestState.hpp"
#include "GraphicStates\LineWidthState.hpp"
#include "GraphicStates\UnpackAlignmentState.hpp"
#include "GraphicStates\PolygonOffsetState.hpp"
#include "GraphicStates\StencilTestState.hpp"
#include "GraphicStates\RasterizerState.hpp"
#include "FrameBuffers\FrameBuffer.hpp"
#include "../Math/Math.hpp"


namespace Engine
{
	namespace Graphic
	{
		class _DLL_DECLSPEC GraphicContext : public DelayedChange::IDelayedValueObserver
		{
		public:
			GraphicContext(HWND _hwnd, unsigned int _width, unsigned int _height);
			~GraphicContext();

			void SwapBuffer();
			void Activate();
			void DrawArrays(GLenum _mode, GLint _first, GLsizei _count);
			void DrawElements(GLenum _mode, GLsizei _count, GLenum _type, const GLvoid* _indices);
			void DrawElementsInstanced(GLenum _mode, GLsizei _count, GLenum _type, const GLvoid* _indices, GLsizei _primcount);
			void DrawArraysInstanced(GLenum _mode,  GLint _first,  GLsizei _count,  GLsizei _primcount);
			void DrawArraysIndirect(GLenum _mode, const void *_indirect);
			void Clear(GLbitfield _mask);

			GraphicStates::BlendingState& Blend();
			GraphicStates::FaceCullingState& CullFace();
			GraphicStates::DepthTestState& DepthTest();
			GraphicStates::LineWidthState& LineWidth();
			GraphicStates::UnpackAlignmentState& UnpackAlignement();
			GraphicStates::PolygonOffsetState& PolygonOffset();
			GraphicStates::StencilTestState& StencilTest();
			GraphicStates::RasterizerState& Rasterizer();

			BasicBodies::Plane* ScreenPlane() const;
			BasicBodies::IcoSphere* UnitSphere() const;
			BasicBodies::Cone* UnitCone() const;
			BasicBodies::LineCone* LineCone() const;
			BasicBodies::LineSphere* LineSphere() const;
			BasicBodies::LineCube* LineCube() const;

			/*Shaders::LightEffectShader* DirectionalLightShader() const;
			Shaders::LightEffectShader* PointLightShader() const;
			Shaders::LightEffectShader* SpotLightShader() const;
			Shaders::PostprocessingShader* GaussShader() const;
			Shaders::PostprocessingShader* AdaptiveLumShader() const;
			Shaders::PostprocessingShader* TonemapShader() const;
			Shaders::PostprocessingShader* FxaaShader() const;*/

			Shaders::Shader* DefaultFontShader() const;
			Shaders::Shader* DebugLineShader() const;

			const FrameBuffers::FrameBuffer& BackBuffer() const;

			bool IsCurrentShader(const Shaders::Shader* _shader) const;
			void BindShader(const Shaders::Shader* _shader);

			Graphic::IconRenderer* IconRenderer() const;
			Buffers::Texture2D* IconPointLight() const;
			Buffers::Texture2D* IconSpotLight() const;

			void Resize(unsigned int _width, unsigned int _height);

			const Math::Mat4f& GuiProjectionMatrix() const;

		private:
			static void APIENTRY DebugCallback(GLenum source, GLenum type, GLuint id, GLenum severity, GLsizei length, const GLchar* message, GLvoid* userParam);

			GraphicStates::BlendingState blend;
			GraphicStates::FaceCullingState cullFace;
			GraphicStates::DepthTestState depthTest;
			GraphicStates::LineWidthState lineWidth;
			GraphicStates::UnpackAlignmentState unpackAlignement;
			GraphicStates::PolygonOffsetState polygonOffset;
			GraphicStates::StencilTestState stencilTest;
			GraphicStates::RasterizerState rasterizer;

			BasicBodies::Plane* plane;
			BasicBodies::IcoSphere* unitSphere;
			BasicBodies::Cone* unitCone;
			BasicBodies::LineCone* lineCone;
			BasicBodies::LineSphere* lineSphere;
			BasicBodies::LineCube* lineCube;

			FrameBuffers::FrameBuffer backBuffer;

			Shaders::Shader* fontShader;
			Shaders::Shader* lineShader;

			/*Shaders::LightEffectShader* directionalLightShader;
			Shaders::LightEffectShader* pointLightShader;
			Shaders::LightEffectShader* spotLightShader;
			Shaders::PostprocessingShader* gaussShader;
			Shaders::PostprocessingShader* adaptiveLumShader;
			Shaders::PostprocessingShader* tonemapShader;
			Shaders::PostprocessingShader* fxaaShader;*/

			Graphic::IconRenderer* iconRenderer;
			Buffers::Texture2D* iconPointLight;
			Buffers::Texture2D* iconSpotLight;

			Math::Mat4f guiProjectionMatrix;

			GLuint currentShader;

			HDC hdc, oldHDC;
			HGLRC hrc, oldHRC;
			HWND hwnd;
		};
	}
}

#endif