#define DLL_EXPORT

#include "StencilTestState.hpp"
#include "../GraphicContext.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::DelayedChange;
using namespace Engine::Graphic::GraphicStates;

StencilTestState::StencilTestState(GraphicContext* _context) : IDelayedValue(_context), state(GL_FALSE), stencilFail(GL_KEEP), depthFail(GL_KEEP), pass(GL_KEEP), func(GL_ALWAYS), refValue(0), mask(0xFFFFFFFF)
{ }
StencilTestState::~StencilTestState()
{ }

void StencilTestState::Disable()
{
	if (state == GL_TRUE)
	{
		state = GL_FALSE;
		OnNotifyValueChanged();
	}
}
void StencilTestState::Enable()
{
	if (state == GL_FALSE)
	{
		state = GL_TRUE;
		OnNotifyValueChanged();
	}
}
void StencilTestState::Operation(GLenum _stencilFail, GLenum _depthFail, GLenum _pass)
{
	if (stencilFail != _stencilFail)
	{
		stencilFail = _stencilFail;
		OnNotifyValueChanged();
	}
	if (depthFail != _depthFail)
	{
		depthFail = _depthFail;
		OnNotifyValueChanged();
	}
	if (pass != _pass)
	{
		pass = _pass;
		OnNotifyValueChanged();
	}
}
void StencilTestState::Function(GLenum _func, GLint _refValue, GLuint _mask)
{
	if (func != _func)
	{
		func = _func;
		OnNotifyValueChanged();
	}
	if (refValue != _refValue)
	{
		refValue = _refValue;
		OnNotifyValueChanged();
	}
	if (mask != _mask)
	{
		mask = _mask;
		OnNotifyValueChanged();
	}
}

void StencilTestState::DoChangeValue()
{
	if (state == GL_TRUE)
	{
		glEnable(GL_STENCIL_TEST);
		glStencilOp(stencilFail, depthFail, pass);
		glStencilFunc(func, refValue, mask);
	}
	else
		glDisable(GL_STENCIL_TEST);
}