#define DLL_EXPORT

#include "IconRenderer.hpp"
#include "GraphicContext.hpp"
#include "../Core/System/SystemConfiguration.hpp"
#include "../Util/Logger.hpp"
#include "Buffers/Texture2D.hpp"

using namespace Engine;
using namespace Engine::Core::System;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Buffers;
using namespace Engine::Graphic::Shaders;
using namespace Engine::Math;
using namespace Engine::Util;

IconRenderer::IconRenderer(GraphicContext* _context, unsigned int _screenWidth, unsigned int _screenHeight) : width(_screenWidth), height(_screenHeight), context(_context),
	iconShader(GL_POINTS, _context)
{
	iconShader.AttachShaderFile(SystemConfiguration::Get<std::string>("IconVertexShader"), GL_VERTEX_SHADER);
	iconShader.AttachShaderFile(SystemConfiguration::Get<std::string>("IconGeometryShader"), GL_GEOMETRY_SHADER);
	iconShader.AttachShaderFile(SystemConfiguration::Get<std::string>("IconFragmentShader"), GL_FRAGMENT_SHADER);
	iconShader.Link();

	//Buffer anlegen
	 glGenBuffers(1, &vbo);
	 glBindBuffer(GL_ARRAY_BUFFER, vbo);
	 glBufferData(GL_ARRAY_BUFFER, 3 * sizeof(float), nullptr, GL_STREAM_DRAW);

	 glGenVertexArrays(1, &vao);
}
IconRenderer::~IconRenderer()
{
	glDeleteBuffers(1, &vbo);
	glDeleteVertexArrays(1, &vao);
}

void IconRenderer::Render(Texture2D* _texture, const Math::Vec3f& _position, const Math::Mat4f& _cameraMatrix, const Math::Vec2f& _size)
{
	Vec2f screenSize(_size.X() / width, _size.Y() / height);

	context->DepthTest().TestAndWrite();
	context->CullFace().Disable();
	context->Blend().Enable(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	iconShader.Activate();
	iconShader.UniformVariable<Math::Mat4f>("cameraMatrix") = _cameraMatrix;
	iconShader.UniformVariable<Math::Vec2f>("size") = screenSize;
	_texture->Bind(0);
	iconShader.UniformVariable<GLint>("iconTexture") = 0;
	iconShader.ChangeValues();

	glBindVertexArray(vao);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);

	float* data = (float*)glMapBufferRange(GL_ARRAY_BUFFER, 0, 3 * sizeof(float), GL_MAP_WRITE_BIT | GL_MAP_INVALIDATE_RANGE_BIT);
	memcpy(data, &_position, 3*sizeof(float));

	if (!glUnmapBuffer(GL_ARRAY_BUFFER))
		Logger::Fatal("Failed to unmap ARRAY_BUFFER");

	glEnableVertexAttribArray(SHADER_POSITION);
	glVertexAttribPointer(SHADER_POSITION, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (GLvoid*)0);
	context->DrawArrays(iconShader.PrimitiveType(), 0, 1);

	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindVertexArray(0);
}

void IconRenderer::Resize(unsigned int _width, unsigned int _height)
{
	width = _width;
	height = _height;
}