#ifndef _ENGINE_GRAPHIC_MATERIALS_MATERIAL_HPP
#define _ENGINE_GRAPHIC_MATERIALS_MATERIAL_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"

#include "../../Math/Math.hpp"
#include "IMaterialPart.hpp"
#include "../../Graphic/Shaders/Shader.hpp"

namespace Engine
{
	namespace Graphic
	{
		namespace Materials
		{
			enum ShadeMode { Normals, Deferred, Forward, Octree };

			class _DLL_DECLSPEC Material
			{
			public:
				Material(ShaderPtr _shader, ShaderPtr _forwardShader,
					ShaderPtr _normalShader, ShaderPtr _octreeShader);
				~Material();

				void StartActivate(ShadeMode _mode) const;
				void FinishActivate(ShadeMode _mode) const;
				void Deactivate() const;

				std::vector<IMaterialPart*>& Parts();
				ShaderPtr Shader() const;
				void Shader(ShaderPtr _shader);
				ShaderPtr ForwardShader() const;
				void ForwardShader(ShaderPtr _shader);
				ShaderPtr NormalShader() const;
				void NormalShader(ShaderPtr _shader);
				ShaderPtr OctreeShader() const;
				void OctreeShader(ShaderPtr _shader);

				bool IsShadeless() const;
				void IsShadeless(bool _value);

				bool IsTransparent() const;
				void IsTransparent(bool _value);

			private:
				std::vector<IMaterialPart*> parts;
				ShaderPtr shader;
				ShaderPtr forwardShader;
				ShaderPtr normalShader;
				ShaderPtr octreeShader;

				bool isShadeless, isTransparent;
			};
		}
	}
}

#endif