#ifndef _ENGINE_GRAPHIC_MESHES_BSPTRIANGLE_HPP
#define _ENGINE_GRAPHIC_MESHES_BSPTRIANGLE_HPP

#include "../../dllconfig.hpp"
#include "../../Math/Math.hpp"
#include <vector>
#include "VertexStreamElement.hpp"
#include "../../Scene/Boundings/SplittingResult.hpp"

namespace Engine
{
	namespace Graphic
	{
		namespace Meshes
		{
			class _DLL_DECLSPEC BSPTriangle
			{
			public:
				BSPTriangle(unsigned int _i1, unsigned int _i2, unsigned int _i3, const Math::Vec4f& _p1, const Math::Vec4f& _p2, const Math::Vec4f& _p3, const Math::Vec4f& _plane);
				~BSPTriangle();

				const unsigned int* Indices() const;
				const Math::Vec4f* Positions() const;
				const Math::Vec4f& Plane() const;

				Scene::Boundings::SplittingResult IntersectPlane(const Math::Vec4f& _plane) const;
				Scene::Boundings::SplittingResult IntersectPlane(const Math::Vec4f& _plane, float& _d1, float& _d2, float& _d3) const;

				void Split(float _d1, float _d2, float _d3, std::vector<BSPTriangle>& _front, std::vector<BSPTriangle>& _back, std::vector<float*>& _tdata, const std::vector<VertexStreamElement>& _mapping, int _floatPerVertex);

			private:
				static float* Interpolate(float _d1, float _d2, float* _tdata1, float* _tdata2, const std::vector<VertexStreamElement>& _mapping, int _floatPerVertex);

				unsigned int indices[3];
				Math::Vec4f positions[3];
				Math::Vec4f plane;
			};
		}
	}
}

#endif