#define DLL_EXPORT

#include "Mesh.hpp"

using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Meshes;
using namespace Engine::Scene::Boundings;

Mesh::Mesh(float* _vertexData, unsigned int _vertexCount, unsigned int _vertexSize, unsigned int* _indices, unsigned int _indexCount, const std::vector<VertexStreamElement>& _mapping,
		   const Math::Vec4f& _boundingLocation, const Math::Vec3f& _boundingSize, GraphicContext* _context) :
	vbo(0), vao(0), indexVbo(0), count(_indexCount), boundingBox(_boundingLocation, _boundingSize), context(_context)
{
	glBindVertexArray(0); //If someone did not unbind the last vao, the vao's elementbuffer would be set to null (and you search hours to find it)

	//Daten buffern
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, _vertexCount * _vertexSize, _vertexData, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, _indexCount * sizeof(unsigned int), _indices, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	//Pointer zusammenstellen
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	
	for (auto it = _mapping.begin(); it != _mapping.end(); ++it)
	{
		glEnableVertexAttribArray(it->AttribHandle());
		glVertexAttribPointer(it->AttribHandle(), it->Size(), GL_FLOAT, GL_FALSE, _vertexSize, (GLvoid*)it->Offset());
	}

	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	
	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}

Mesh::~Mesh()
{
	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
	glDeleteVertexArrays(1, &vao);
}

void Mesh::Draw(GLenum _primitiveType)
{
	glBindVertexArray(vao);

	context->DrawElements(_primitiveType, count, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0); //Knnte man ev. aus Performaceberlegungen weglassen
}

const Box& Mesh::BoundingBox() const
{
	return boundingBox;
}