#define DLL_EXPORT

#include "TransparentMesh.hpp"
#include "../Shaders/Shader.hpp"
#include "../../Util/Logger.hpp"
#include "../../Util/Convert.hpp"
#include "../../Util/StopWatch.hpp"
#include "../../Core/System/SystemConfiguration.hpp"

using namespace Engine;
using namespace Engine::Util;
using namespace Engine::Math;
using namespace Engine::Core::System;
using namespace Engine::Scene::Boundings;
using namespace Engine::Graphic;
using namespace Engine::Graphic::Meshes;
using namespace Engine::Graphic::Shaders;

//#define SPLITWEIGHT 3.0f
//#define BALANCEWEIGHT 0.5f
//#define NUM_TEST_VERTICES 25

//#define BSP_USE_AXIS_PLANES

TransparentMesh::TransparentMesh(float* _vertexData, unsigned int _vertexCount, unsigned int _vertexSize, unsigned int* _indices, unsigned int _indexCount, const std::vector<VertexStreamElement>& _mapping,
				 const Math::Vec4f& _boundingLocation, const Math::Vec3f& _boundingSize, GraphicContext* _context) : context(_context),
	boundingBox(_boundingLocation, _boundingSize), bspRoot(nullptr), count(_indexCount), vao(0), vbo(0), numberOfTestsNeeded(0), numberOfEmptyNodes(0), numberOfLeafNodes(0)
{
	glBindVertexArray(0); //If someone did not unbind the last vao, the vao's elementbuffer would be set to null (and you search hours to find it)

	//
	//Begin Setup BSP Tree
	//

	//Get List of vertices	
	/*int floatPerVertex = _vertexSize / sizeof(float);
	int offset = 0;
	for (auto it = _mapping.begin(); it != _mapping.end(); ++it)
		if (it->AttribHandle() == SHADER_POSITION)
			offset = it->Offset() / sizeof(float);

	std::vector<float*> vertices;
	std::vector<BSPTriangle> triangles;

	//VertexData
	for (unsigned int i = 0; i < _vertexCount; i++)
	{
		float* v = new float[floatPerVertex];
		memcpy(v, _vertexData + i * floatPerVertex, floatPerVertex * sizeof(float));
		vertices.push_back(v);
	}

	//IndexData
	for (unsigned int i = 0; i < _indexCount; i+=3)
	{
		unsigned int i1, i2, i3;
		i1 = _indices[i];
		i2 = _indices[i + 1];
		i3 = _indices[i + 2];

		Vec3f p1(vertices[i1] + offset);
		Vec3f p2(vertices[i2] + offset);
		Vec3f p3(vertices[i3] + offset);

		Vec3f n = Vec3f::Normalize(Vec3f::Cross(p2 - p1, p3 - p1));
		float d = Vec3f::Dot(n, p1);

		BSPTriangle t(i1, i2, i3, Vec4f(p1, 1.0f), Vec4f(p2, 1.0f), Vec4f(p3, 1.0f), Math::Vec4f(n, -d));
		triangles.push_back(t);
	}

	//Build tree
	StopWatch sw("BspTree:Build");
	sw.Start();
	bspRoot = BuildBSPTree(vertices, triangles, _mapping, floatPerVertex, count);
	sw.Stop();
	sw.Display(true);
	Logger::Debug(std::string("Number of test nodes: ") + Convert::ToString(numberOfTestsNeeded));
	Logger::Debug(std::string("Number of empty nodes: ") + Convert::ToString(numberOfEmptyNodes));
	Logger::Debug(std::string("Number of leaf nodes: ") + Convert::ToString(numberOfLeafNodes));
	Logger::Debug(std::string("TriangleCount: ") + Convert::ToString(count / 3));

	//
	//End Setup BSP Tree
	//

	//Daten buffern
	glGenBuffers(1, &vbo);
	glGenBuffers(1, &indexVbo);

	float* vertexData = new float[floatPerVertex * vertices.size()];
	float* vertexDataPointer = vertexData;
	for (auto it = vertices.begin(); it != vertices.end(); ++it)
	{
		memcpy(vertexDataPointer, (*it), _vertexSize);
		vertexDataPointer += floatPerVertex;
	}

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, vertices.size() * _vertexSize, vertexData, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	delete vertexData;

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, count * sizeof(unsigned int), 0, GL_STREAM_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	

	//Pointer zusammenstellen
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	
	for (auto it = _mapping.begin(); it != _mapping.end(); ++it)
	{
		glEnableVertexAttribArray(it->AttribHandle());
		glVertexAttribPointer(it->AttribHandle(), it->Size(), GL_FLOAT, GL_FALSE, _vertexSize, (GLvoid*)it->Offset());
	}
	
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);

	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);*/
}

TransparentMesh::~TransparentMesh()
{
	/*if (bspRoot != nullptr)
		delete bspRoot;

	glDeleteBuffers(1, &vbo);
	glDeleteBuffers(1, &indexVbo);
	glDeleteVertexArrays(1, &vao);*/
}

BSPNode* TransparentMesh::BuildBSPTree(std::vector<float*>& _data, std::vector<BSPTriangle> _triangles, const std::vector<VertexStreamElement>& _mapping, int _floatPerVertex, GLsizei& _count)
{
	if (_triangles.size() == 0)
	{
		return nullptr;
	}
	if (_triangles.size() == 1)
	{
		numberOfLeafNodes ++;
		return new BSPNode(_triangles[0].Indices()[0], _triangles[0].Indices()[1], _triangles[0].Indices()[2], _triangles[0].Plane());
	}
	else
	{
		Vec4f plane;
		int bestIndex = FindBestPlane(_triangles, plane);

		std::vector<BSPTriangle> front;
		std::vector<BSPTriangle> back;
		int i = 0;

		for (auto it = _triangles.begin(); it != _triangles.end(); ++it)
		{
			if (i != bestIndex)
			{
				float d1, d2, d3;
				SplittingResult side = it->IntersectPlane(plane, d1, d2, d3);

				if (side == SplittingResult::Front)
					front.push_back(*it);
				else if (side == SplittingResult::Back)
					back.push_back(*it);
				else
				{
					//Split
					it->Split(d1, d2, d3, front, back, _data, _mapping, _floatPerVertex);
					_count += 6; //1 removed, 3 added
				}
			}

			i++;
		}

		BSPNode* node;

		if (bestIndex != -1)
			node = new BSPNode(_triangles[bestIndex].Indices()[0], _triangles[bestIndex].Indices()[1], _triangles[bestIndex].Indices()[2], plane);
		else
		{
			node = new BSPNode(0, 0, 0, plane);
			numberOfEmptyNodes++;
		}

		node->Front(BuildBSPTree(_data, front, _mapping, _floatPerVertex, _count));
		node->Back(BuildBSPTree(_data, back, _mapping, _floatPerVertex, _count));

		numberOfTestsNeeded++;

		return node;
	}
}

float TransparentMesh::PlaneWeight(std::vector<BSPTriangle>& _triangles, const Vec4f& _plane, int _index)
{
	int countFront = 0;
	int countBack = 0;
	int countSplit = 0;
	int i = 0;

	for (auto it = _triangles.begin(); it != _triangles.end(); ++it)
	{
		if (i != _index)
		{
			SplittingResult side = it->IntersectPlane(_plane);

			if (side == SplittingResult::Front)
				countFront++;
			else if (side == SplittingResult::Back)
				countBack++;
			else
				countSplit++;
		}

		i++;
	}

	//Calculate weight
	float splitRatio = ((float)countSplit / (float)_triangles.size());
	float balanceRatio = fabs((float)((countFront + countSplit) - (countBack + countSplit)) / (float)(countFront + countBack + 2 * countSplit));

	if (splitRatio > 0.1)
		return 1000.0f;

	//Exclude cases where everything is on one side of the plane
	if (countFront == 0 || countBack == 0)
		return 1000.0f;

	float weight = (splitRatio * SystemConfiguration::Get<float>("BspSplitWeight") + balanceRatio * SystemConfiguration::Get<float>("BspBalanceWeight")) /
		(SystemConfiguration::Get<float>("BspSplitWeight") + SystemConfiguration::Get<float>("BspBalanceWeight"));

	return weight;
}

int TransparentMesh::FindBestPlane(std::vector<BSPTriangle>& _triangles, Math::Vec4f& _plane)
{
	float bestWeight = 100000.0f;
	float newWeight = 100000.0f;
	int best = -1;

	if (SystemConfiguration::Get<bool>("BspUseAxisPlanes"))
	{
		//Test with axis aligned split plane in the center
		Vec4f center(0.0f, 0.0f, 0.0f, 0.0f);
		for (auto it = _triangles.begin(); it != _triangles.end(); ++it)
		{
			center += it->Positions()[0];
			center += it->Positions()[1];
			center += it->Positions()[2];
		}
		center /= (float)(_triangles.size() * 3);

		//X-Plane
		Vec4f plane(1.0f, 0.0f, 0.0f, -center.X());
		newWeight = PlaneWeight(_triangles, plane, -1);
		best = -1;
		bestWeight = newWeight;
		_plane = plane;

		//Y-Plane
		plane = Vec4f(0.0f, 1.0f, 0.0f, -center.Y());
		newWeight = PlaneWeight(_triangles, plane, -1);
		if (newWeight < bestWeight)
		{
			bestWeight = newWeight;
			_plane = plane;
		}

		//Z-Plane
		plane = Vec4f(0.0f, 0.0f, 1.0f, -center.Z());
		newWeight = PlaneWeight(_triangles, plane, -1);
		if (newWeight < bestWeight)
		{
			bestWeight = newWeight;
			_plane = plane;
		}
	}
	
	//Test with triangle spanned splitplane
	if (_triangles.size() < SystemConfiguration::Get<unsigned int>("BspTestVertexCount")) //Test all Vertices
	{
		for (int i = 1; i < _triangles.size(); i++)
		{
			newWeight = PlaneWeight(_triangles, _triangles[i].Plane(), i);
			if (newWeight <= bestWeight)
			{
				bestWeight = newWeight;
				best = i;
				_plane = _triangles[i].Plane();
			}
		}
	}
	else //Test random vertices
	{
		for (unsigned int i = 1; i < SystemConfiguration::Get<unsigned int>("BspTestVertexCount"); i++)
		{
			int idx = rand() % _triangles.size();
			newWeight = PlaneWeight(_triangles, _triangles[idx].Plane(), idx);
			if (newWeight <= bestWeight)
			{
				bestWeight = newWeight;
				best = idx;
				_plane = _triangles[idx].Plane();
			}
		}
	}

	return best;
}

void TransparentMesh::Draw(GLenum _primitiveType, unsigned int* _indices)
{
	/*glBindVertexArray(vao);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexVbo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, count * sizeof(unsigned int), _indices, GL_STREAM_DRAW);	

	context->DrawElements(_primitiveType, count, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);*/
}

void TransparentMesh::BuildIndices(unsigned int** _indexArray, const Math::Vec4f& _cameraPosition)
{
	//BuildIndices(_indexArray, _cameraPosition, bspRoot);
}

void TransparentMesh::BuildIndices(unsigned int** _indexArray, const Math::Vec4f& _cameraPosition, const BSPNode* _node)
{
	/*float d = Vec4f::Dot(_node->Plane(), _cameraPosition);

	if (d > 0.0f && _node->Back() != nullptr) //Camera in Front -> Back - This - Front
		BuildIndices(_indexArray, _cameraPosition, _node->Back());
	else if (d <= 0.0f && _node->Front() != nullptr) //Camera Back -> Front - This - Back
		BuildIndices(_indexArray, _cameraPosition, _node->Front());

	if (_node->Indices()[0] != 0 || _node->Indices()[1] != 0 || _node->Indices()[2] != 0)
	{
		memcpy(*_indexArray, _node->Indices(), 3 * sizeof(unsigned int));
		(*_indexArray) += 3;
	}

	if (d > 0.0f && _node->Front() != nullptr)
		BuildIndices(_indexArray, _cameraPosition, _node->Front());
	else if (d <= 0.0f && _node->Back() != nullptr)
		BuildIndices(_indexArray, _cameraPosition, _node->Back());*/
}

const Scene::Boundings::Box& TransparentMesh::BoundingBox() const
{
	return boundingBox;
}

const GLsizei TransparentMesh::Count() const
{
	return count;
}
