#ifndef _ENGINE_GRAPHIC_MESHES_TRANSPARENTMESH_HPP
#define _ENGINE_GRAPHIC_MESHES_TRANSPARENTMESH_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"
#include "../GraphicContext.hpp"
#include "VertexStreamElement.hpp"
#include "../../Scene/Boundings/Box.hpp"
#include "BSPNode.hpp"
#include "BSPTriangle.hpp"

#include <gl\glew.h>
#include <vector>

namespace Engine
{
	namespace Graphic
	{
		namespace Meshes
		{
			class _DLL_DECLSPEC TransparentMesh
			{
			public:
				TransparentMesh(float* _vertexData, unsigned int _vertexCount, unsigned int _vertexSize, unsigned int* _indices, unsigned int _indexCount, const std::vector<VertexStreamElement>& _mapping,
					 const Math::Vec4f& _boundingLocation, const Math::Vec3f& _boundingSize, GraphicContext* _context);
				~TransparentMesh();

				void Draw(GLenum _primitiveType, unsigned int* _indices);
				void BuildIndices(unsigned int** _indexArray, const Math::Vec4f& _cameraPosition);

				const GLsizei Count() const;

				const Scene::Boundings::Box& BoundingBox() const;

			private:
				BSPNode* BuildBSPTree(std::vector<float*>& _data, std::vector<BSPTriangle> _triangles, const std::vector<VertexStreamElement>& _mapping, int _floatPerVertex, GLsizei& _count);
				float PlaneWeight(std::vector<BSPTriangle>& _triangles, const Math::Vec4f& _plane, int _index);
				int FindBestPlane(std::vector<BSPTriangle>& _triangles, Math::Vec4f& _plane);
				void BuildIndices(unsigned int** _indexArray, const Math::Vec4f& _cameraPosition, const BSPNode* _node);

				GLuint vbo, indexVbo, vao;
				GLsizei count;

				unsigned int numberOfTestsNeeded, numberOfEmptyNodes, numberOfLeafNodes;

				BSPNode* bspRoot;

				GraphicContext* context;
				Scene::Boundings::Box boundingBox;
			};
		}
	}
}

#endif