#ifndef _ENGINE_GRAPHIC_SHADER_SHADER_HPP
#define _ENGINE_GRAPHIC_SHADER_SHADER_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"

#include "../../Util/SharedPointer.hpp"
#include "Uniform.hpp"
#include "UniformArray.hpp"

#include <gl\glew.h>
#include <string>
#include <unordered_map>
#include "../../Core/Collections/UnorderedMapExtensions.hpp"
#include "../DelayedChange/IDelayedValueObserver.hpp"

#define SHADER_POSITION 0 

#define SHADER_FONT_CHARINDEX 1

#define SHADER_MATERIAL_NORMAL 1
#define SHADER_MATERIAL_TEXCOORD_DIFFUSE 2
#define SHADER_MATERIAL_TANGENT_DIFFUSE 3

namespace Engine
{
	namespace Graphic
	{
		namespace Shaders
		{
			class _DLL_DECLSPEC Shader : public DelayedChange::IDelayedValueObserver
			{
			public:
				Shader(GLenum _primitiveType, GraphicContext* _context);
				~Shader();

				GLuint Handle() const;
				GLenum PrimitiveType() const;

				void Activate() const;
				void Deactivate() const;

				bool AttachShaderFile(const std::string _shaderPath, GLenum _shaderType, std::string _appendCode = "", std::string* _error = nullptr);
				bool AttachShader(const char* _shaderCode, GLenum _shaderType, std::string* _error = nullptr);
				bool Link(std::string* _error = nullptr);

				void BindOutput(std::string _name, GLuint _index) const;
				void BindAttribute(std::string _name, GLuint _index) const;

				static std::string LoadShaderFile(const std::string _shaderPath);

				template<typename T> inline Uniform<T>& UniformVariable(Core::Collections::HashString _name)
				{
					auto it = uniformVariables.find(_name);

					if (it != uniformVariables.end())
						return *((Uniform<T>*)it->second);
					else
					{
						#ifndef _EDITOR_BUILD
						Util::Logger::Warning("Trying to set non existing uniform variable");
						#endif
						return Uniform<T>::Default();
					}
				}

				template<typename T> inline UniformArray<T>& UniformArrays(Core::Collections::HashString _name)
				{
					auto it = uniformArrays.find(_name);

					if (it != uniformArrays.end())
						return *((UniformArray<T>*)it->second);
					else
					{
						#ifndef _EDITOR_BUILD
						Util::Logger::Warning("Trying to set non existing uniform array");
						#endif
						return UniformArray<T>::Default();
					}
				}

				GLint GetTextureUnit();
				void ResetTextureUnit(GLint _startUnit);


			private:
				GLenum primitiveType;
				GLuint handle;
				GraphicContext* context;
				GLint maxTextureUnit;

				std::unordered_map<Core::Collections::HashString, IUniform*> uniformVariables;
				std::unordered_map<Core::Collections::HashString, IUniformArray*> uniformArrays;

				const std::string OpenGLShaderPrefix() const;

				void QueryActiveUniforms();
				IUniform* BuildUniform(GLenum _type, const char* _name);
				IUniformArray* BuildUniformArray(GLenum _type, const char* _name, GLsizei _size);
				static void ParseFileCode(std::string& _code, std::string _path);
			};
		}
	}
}

typedef Engine::Util::SharedPointer<Engine::Graphic::Shaders::Shader> ShaderPtr;

#endif