#ifndef _ENGINE_GRAPHIC_SHADERS_UNIFORMARRAY_HPP
#define _ENGINE_GRAPHIC_SHADERS_UNIFORMARRAY_HPP

#include "../../dllconfig.hpp"
#include "IUniformArray.hpp"
#include "Uniform.hpp"
#include <cassert>

namespace Engine
{
	namespace Graphic
	{
		namespace Shaders
		{
			template<typename T>
			class _DLL_DECLSPEC UniformArray : public IUniformArray
			{
			public:
				UniformArray(size_t _count) : IUniformArray(_count), uniforms(nullptr)
				{
					if (_count > 0)
						uniforms = new Uniform<T>*[_count];
				}

				~UniformArray()
				{
					if (uniforms != nullptr)
						delete uniforms;
				}

				void SetUniform(size_t _index, IUniform* _uniform)
				{
					assert(_index < Count());

					uniforms[_index] = (Uniform<T>*)_uniform;
				}

				Uniform<T>& operator [] (size_t _index)
				{
					if (_index < Count())
						return *(uniforms[_index]);
					else
						return Uniform<T>::Default();
				}

				static UniformArray<T>& Default()
				{
					return defaultArray;
				}

			private:
				static UniformArray<T> defaultArray;

				UniformArray() : IUniformArray(0), uniforms(nullptr)
				{ }

				Uniform<T>** uniforms;
			};

			template<typename T> UniformArray<T> UniformArray<T>::defaultArray;
		}
	}
}

#endif