#define DLL_EXPORT

#include "Mat4f.hpp"
#include "Mat3f.hpp"
#include <math.h>

using namespace Engine::Math::CPPonly;

Mat4f::Mat4f()
{
	for (int i = 0; i < 16; i++)
		values[i] = 0.0f;

	values[0] = 1.0f;
	values[5] = 1.0f;
	values[10] = 1.0f;
	values[15] = 1.0f;
}

Mat4f::Mat4f(float _diag)
{
	for (int i = 0; i < 16; i++)
		values[i] = 0.0f;

	values[0] = _diag;
	values[5] = _diag;
	values[10] = _diag;
	values[15] = _diag;
}

Mat4f::Mat4f(float _m11, float _m12, float _m13, float _m14, float _m21, float _m22, float _m23, float _m24, float _m31, float _m32, float _m33, float _m34, float _m41, float _m42, float _m43, float _m44)
{
	values[0] = _m11;
	values[1] = _m12;
	values[2] = _m13;
	values[3] = _m14;
	values[4] = _m21;
	values[5] = _m22;
	values[6] = _m23;
	values[7] = _m24;
	values[8] = _m31;
	values[9] = _m32;
	values[10] = _m33;
	values[11] = _m34;
	values[12] = _m41;
	values[13] = _m42;	
	values[14] = _m43;
	values[15] = _m44;
}

Mat4f::Mat4f(const float* _values)
{
	memcpy(values, _values, 16*sizeof(float));
}

Mat4f::Mat4f(const Mat4f& _toCopy)
{
	memcpy(values, _toCopy.values, 16*sizeof(float));
}

Mat4f::Mat4f(Mat3f& _toCopy)
{
	values[0] = _toCopy.GetGlMatrix()[0];
	values[1] = _toCopy.GetGlMatrix()[1];
	values[2] = _toCopy.GetGlMatrix()[2];
	values[3] = 0.0f;

	values[4] = _toCopy.GetGlMatrix()[3];
	values[5] = _toCopy.GetGlMatrix()[4];
	values[6] = _toCopy.GetGlMatrix()[5];
	values[7] = 0.0f;

	values[8] = _toCopy.GetGlMatrix()[6];
	values[9] = _toCopy.GetGlMatrix()[7];
	values[10] = _toCopy.GetGlMatrix()[8];
	values[11] = 0.0f;

	values[12] = 0.0f;
	values[13] = 0.0f;
	values[14] = 0.0f;
	values[15] = 1.0f;
}

Mat4f::~Mat4f() { }

inline float Mat4f::M11() const
{
	return values[0];
}
inline Mat4f& Mat4f::M11(float _value)
{
	values[0] = _value;
	return *this;
}
inline float Mat4f::M12() const
{
	return values[1];
}
inline Mat4f& Mat4f::M12(float _value)
{
	values[1] = _value;
	return *this;
}
inline float Mat4f::M13() const
{
	return values[2];
}
inline Mat4f& Mat4f::M13(float _value)
{
	values[2] = _value;
	return *this;
}
inline float Mat4f::M14() const
{
	return values[3];
}
inline Mat4f& Mat4f::M14(float _value)
{
	values[3] = _value;
	return *this;
}

inline float Mat4f::M21() const
{
	return values[4];
}
inline Mat4f& Mat4f::M21(float _value)
{
	values[4] = _value;
	return *this;
}
inline float Mat4f::M22() const
{
	return values[5];
}
inline Mat4f& Mat4f::M22(float _value)
{
	values[5] = _value;
	return *this;
}
inline float Mat4f::M23() const
{
	return values[6];
}
inline Mat4f& Mat4f::M23(float _value)
{
	values[6] = _value;
	return *this;
}
inline float Mat4f::M24() const
{
	return values[7];
}
inline Mat4f& Mat4f::M24(float _value)
{
	values[7] = _value;
	return *this;
}

inline float Mat4f::M31() const
{
	return values[8];
}
inline Mat4f& Mat4f::M31(float _value)
{
	values[8] = _value;
	return *this;
}
inline float Mat4f::M32() const
{
	return values[9];
}
inline Mat4f& Mat4f::M32(float _value)
{
	values[9] = _value;
	return *this;
}
inline float Mat4f::M33() const
{
	return values[10];
}
inline Mat4f& Mat4f::M33(float _value)
{
	values[10] = _value;
	return *this;
}
inline float Mat4f::M34() const
{
	return values[11];
}
inline Mat4f& Mat4f::M34(float _value)
{
	values[11] = _value;
	return *this;
}

inline float Mat4f::M41() const
{
	return values[12];
}
inline Mat4f& Mat4f::M41(float _value)
{
	values[12] = _value;
	return *this;
}
inline float Mat4f::M42() const
{
	return values[13];
}
inline Mat4f& Mat4f::M42(float _value)
{
	values[13] = _value;
	return *this;
}
inline float Mat4f::M43() const
{
	return values[14];
}
inline Mat4f& Mat4f::M43(float _value)
{
	values[14] = _value;
	return *this;
}
inline float Mat4f::M44() const
{
	return values[15];
}
inline Mat4f& Mat4f::M44(float _value)
{
	values[15] = _value;
	return *this;
}

inline float* Mat4f::GetGlMatrix() const
{
	return (float*)(&values[0]);
}

inline bool Mat4f::IsColumnMajor()
{
	return false;
}

Mat4f& Mat4f::Transpose()
{
	float z = M12();
	M12(M21());
	M21(z);

	z = M13();
	M13(M31());
	M31(z);

	z = M14();
	M14(M41());
	M41(z);

	z = M23();
	M23(M32());
	M32(z);

	z = M24();
	M24(M42());
	M42(z);

	z = M34();
	M34(M43());
	M43(z);

	return *this;
}
Mat4f Mat4f::Transpose(const Mat4f& _matrix)
{
	return Mat4f(_matrix.M11(), _matrix.M21(), _matrix.M31(), _matrix.M41(),
		_matrix.M12(), _matrix.M22(), _matrix.M32(), _matrix.M42(),
		_matrix.M13(), _matrix.M23(), _matrix.M33(), _matrix.M43(),
		_matrix.M14(), _matrix.M24(), _matrix.M34(), _matrix.M44());
}

Mat4f& Mat4f::Invert()
{
	Mat4f m = Mat4f::Invert(*this);
	for (int i = 0; i < 16; i++)
		values[i] = m.values[i];
	return *this;
}
Mat4f Mat4f::Invert(const Mat4f& _matrix)
{
	return _matrix.Adj() / _matrix.Det();
}
float Mat4f::Det() const
{
	return Mat3f::Det(Mat3f(values[1], values[5], values[9], values[2], values[6], values[10], values[3], values[7], values[11])) * -values[12] +
		   Mat3f::Det(Mat3f(values[0], values[4], values[8], values[2], values[6], values[10], values[3], values[7], values[11])) * values[13] +
		   Mat3f::Det(Mat3f(values[0], values[4], values[8], values[1], values[5], values[9], values[3], values[7], values[11])) * -values[14] +
		   Mat3f::Det(Mat3f(values[0], values[4], values[8], values[1], values[5], values[9], values[2], values[6], values[10])) * values[15];
}
float Mat4f::Det(const Mat4f& _matrix)
{
	return _matrix.Det();
}
Mat4f Mat4f::Adj() const
{
	return Mat4f(
		M23()*M34()*M42() - M24()*M33()*M42() + M24()*M32()*M43() - M22()*M34()*M43() - M23()*M32()*M44() + M22()*M33()*M44(),
		M14()*M33()*M42() - M13()*M34()*M42() - M14()*M32()*M43() + M12()*M34()*M43() + M13()*M32()*M44() - M12()*M33()*M44(),
		M13()*M24()*M42() - M14()*M23()*M42() + M14()*M22()*M43() - M12()*M24()*M43() - M13()*M22()*M44() + M12()*M23()*M44(),
		M14()*M23()*M32() - M13()*M24()*M32() - M14()*M22()*M33() + M12()*M24()*M33() + M13()*M22()*M34() - M12()*M23()*M34(),
		M24()*M33()*M41() - M23()*M34()*M41() - M24()*M31()*M43() + M21()*M34()*M43() + M23()*M31()*M44() - M21()*M33()*M44(),
		M13()*M34()*M41() - M14()*M33()*M41() + M14()*M31()*M43() - M11()*M34()*M43() - M13()*M31()*M44() + M11()*M33()*M44(),
		M14()*M23()*M41() - M13()*M24()*M41() - M14()*M21()*M43() + M11()*M24()*M43() + M13()*M21()*M44() - M11()*M23()*M44(),
		M13()*M24()*M31() - M14()*M23()*M31() + M14()*M21()*M33() - M11()*M24()*M33() - M13()*M21()*M34() + M11()*M23()*M34(),
		M22()*M34()*M41() - M24()*M32()*M41() + M24()*M31()*M42() - M21()*M34()*M42() - M22()*M31()*M44() + M21()*M32()*M44(),
		M14()*M32()*M41() - M12()*M34()*M41() - M14()*M31()*M42() + M11()*M34()*M42() + M12()*M31()*M44() - M11()*M32()*M44(),
		M12()*M24()*M41() - M14()*M22()*M41() + M14()*M21()*M42() - M11()*M24()*M42() - M12()*M21()*M44() + M11()*M22()*M44(),
		M14()*M22()*M31() - M12()*M24()*M31() - M14()*M21()*M32() + M11()*M24()*M32() + M12()*M21()*M34() - M11()*M22()*M34(),
		M23()*M32()*M41() - M22()*M33()*M41() - M23()*M31()*M42() + M21()*M33()*M42() + M22()*M31()*M43() - M21()*M32()*M43(),
		M12()*M33()*M41() - M13()*M32()*M41() + M13()*M31()*M42() - M11()*M33()*M42() - M12()*M31()*M43() + M11()*M32()*M43(),
		M13()*M22()*M41() - M12()*M23()*M41() - M13()*M21()*M42() + M11()*M23()*M42() + M12()*M21()*M43() - M11()*M22()*M43(),
		M12()*M23()*M31() - M13()*M22()*M31() + M13()*M21()*M32() - M11()*M23()*M32() - M12()*M21()*M33() + M11()*M22()*M33()
		);
}
Mat4f Mat4f::Adj(const Mat4f& _matrix)
{
	return _matrix.Adj();
}

Mat4f Mat4f::GetRotation(const Mat4f& _matrix)
{
	Vec3f col1(_matrix.values[0], _matrix.values[4], _matrix.values[8]);
	Vec3f col2(_matrix.values[1], _matrix.values[5], _matrix.values[9]);
	Vec3f col3(_matrix.values[2], _matrix.values[6], _matrix.values[10]);

	col1.Normalize();
	col2.Normalize();
	col3.Normalize();

	return Mat4f(col1.X(), col2.X(), col3.X(), 0.0f, col1.Y(), col2.Y(), col3.Y(), 0.0f, col1.Z(), col2.Z(), col3.Z(), 0.0f, 0.0f, 0.0f, 0.0f, 1.0);

	/*Mat4f mat(_matrix);

	//Remove scaling
	__m128 row1 = _mm_load_ps(mat.values);
	__m128 row2 = _mm_load_ps(mat.values + 4);
	__m128 row3 = _mm_load_ps(mat.values + 8);

	__m128 row1s = _mm_mul_ps(row1, row1);
	__m128 row2s = _mm_mul_ps(row2, row2);
	__m128 row3s = _mm_mul_ps(row3, row3);

	row1s = _mm_add_ps(_mm_add_ps(row1s, row2s), row3s); //dot product over columns
	row2s = _mm_sqrt_ps(row1s); //Scalings

	row1 = _mm_div_ps(row1, row2s);
	row2 = _mm_div_ps(row2, row2s);
	row3 = _mm_div_ps(row3, row2s);

	_mm_store_ps(mat.values, row1);
	_mm_store_ps(mat.values + 4, row2);
	_mm_store_ps(mat.values + 8, row3);

	//Remote translation
	mat.M14(0.0f);
	mat.M24(0.0f);
	mat.M34(0.0f);

	return mat;*/
}

Mat4f Mat4f::RemoveScaling(const Mat4f& _matrix)
{
	Vec3f col1(_matrix.values[0], _matrix.values[4], _matrix.values[8]);
	Vec3f col2(_matrix.values[1], _matrix.values[5], _matrix.values[9]);
	Vec3f col3(_matrix.values[2], _matrix.values[6], _matrix.values[10]);

	col1.Normalize();
	col2.Normalize();
	col3.Normalize();

	return Mat4f(col1.X(), col2.X(), col3.X(), _matrix.M14(), col1.Y(), col2.Y(), col3.Y(), _matrix.M24(), col1.Z(), col2.Z(), col3.Z(), _matrix.M34(), 0.0f, 0.0f, 0.0f, 1.0);



	/*//Remove scaling
	__m128 row1 = _mm_load_ps(mat.values);
	__m128 row2 = _mm_load_ps(mat.values + 4);
	__m128 row3 = _mm_load_ps(mat.values + 8);

	__m128 row1s = _mm_mul_ps(row1, row1);
	__m128 row2s = _mm_mul_ps(row2, row2);
	__m128 row3s = _mm_mul_ps(row3, row3);

	row1s = _mm_add_ps(_mm_add_ps(row1s, row2s), row3s); //dot product over columns
	row2s = _mm_sqrt_ps(row1s); //Scalings

	row1 = _mm_div_ps(row1, row2s);
	row2 = _mm_div_ps(row2, row2s);
	row3 = _mm_div_ps(row3, row2s);

	_mm_store_ps(mat.values, row1);
	_mm_store_ps(mat.values + 4, row2);
	_mm_store_ps(mat.values + 8, row3);

	//Remote translation
	mat.M14(_matrix.M14());
	mat.M24(_matrix.M24());
	mat.M34(_matrix.M34());

	return mat;*/
}

Mat4f Mat4f::RotateX(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat4f(1.0f, 0.0f, 0.0f, 0.0f, 0.0f, cosa, -sina, 0.0f, 0.0f, sina, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateX(Mat4f& _matrix, float _angle) {

	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M12();
	_matrix.M12(z1 * cosa + _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa - z1 * sina);

	z1 = _matrix.M22();
	_matrix.M22(z1 * cosa + _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa - z1 * sina);

	z1 = _matrix.M32();
	_matrix.M32(z1 * cosa + _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa - z1 * sina);

	z1 = _matrix.M42();
	_matrix.M42(z1 * cosa + _matrix.M43() * sina);
	_matrix.M43(_matrix.M43() * cosa - z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateY(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat4f(cosa, 0.0f, sina, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, -sina, 0.0f, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateY(Mat4f& _matrix, float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M11();
	_matrix.M11(z1 * cosa - _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa + z1 * sina);

	z1 = _matrix.M21();
	_matrix.M21(z1 * cosa - _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa + z1 * sina);

	z1 = _matrix.M31();
	_matrix.M31(z1 * cosa - _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa + z1 * sina);

	z1 = _matrix.M41();
	_matrix.M41(z1 * cosa - _matrix.M43() * sina);
	_matrix.M43(_matrix.M43() * cosa + z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateZ(float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);

	return Mat4f(cosa, -sina, 0.0f, 0.0f, sina, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateZ(Mat4f& _matrix, float _angle)
{ 
	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M11();
	_matrix.M11(z1 * cosa + _matrix.M12() * sina);
	_matrix.M12(_matrix.M12() * cosa - z1 * sina);

	z1 = _matrix.M21();
	_matrix.M21(z1 * cosa + _matrix.M22() * sina);
	_matrix.M22(_matrix.M22() * cosa - z1 * sina);

	z1 = _matrix.M31();
	_matrix.M31(z1 * cosa + _matrix.M32() * sina);
	_matrix.M32(_matrix.M32() * cosa - z1 * sina);

	z1 = _matrix.M41();
	_matrix.M41(z1 * cosa + _matrix.M42() * sina);
	_matrix.M41(_matrix.M42() * cosa - z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateAxis(const Vec3f& _axis, float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);
	float icos = 1.0f - cosa;

	return Mat4f(cosa + _axis.X() * _axis.X() * icos,
		_axis.X() * _axis.Y() * icos - _axis.Z() * sina,
		_axis.X() * _axis.Z() * icos + _axis.Y() * sina,
		0.0f,
		_axis.X() * _axis.Y() * icos + _axis.Z() * sina,
		cosa + _axis.Y() * _axis.Y() * icos,
		_axis.Y() * _axis.Z() * icos - _axis.X() * sina,
		0.0f,
		_axis.X() * _axis.Z() * icos - _axis.Y() * sina,
		_axis.Y() * _axis.Z() * icos + _axis.X() * sina,
		cosa + _axis.Z() * _axis.Z() * icos,
		0.0f,
		0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateAxis(Mat4f& _matrix, const Vec3f& _axis, float _angle)
{	
	_matrix *= Mat4f::RotateAxis(_axis, _angle);
	return _matrix;
}
Mat4f Mat4f::Translate(const Vec3f& _direction)
{
	return Mat4f(1.0f, 0.0f, 0.0f, _direction.X(), 0.0f, 1.0f, 0.0f, _direction.Y(), 0.0f, 0.0f, 1.0f, _direction.Z(), 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::Translate(Mat4f& _matrix, const Vec3f& _direction)
{
	_matrix.M14(_matrix.M14() + _matrix.M11() * _direction.X() + _matrix.M12() * _direction.Y() + _matrix.M13() * _direction.Z());
	_matrix.M24(_matrix.M24() + _matrix.M21() * _direction.X() + _matrix.M22() * _direction.Y() + _matrix.M23() * _direction.Z());
	_matrix.M34(_matrix.M34() + _matrix.M31() * _direction.X() + _matrix.M32() * _direction.Y() + _matrix.M33() * _direction.Z());
	_matrix.M44(_matrix.M44() + _matrix.M41() * _direction.X() + _matrix.M42() * _direction.Y() + _matrix.M43() * _direction.Z());

	return _matrix;
}
Mat4f Mat4f::Scale(const Vec3f& _scales)
{
	return Mat4f(_scales.X(), 0.0f, 0.0f, 0.0f, 0.0f, _scales.Y(), 0.0f, 0.0f, 0.0f, 0.0f, _scales.Z(), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::Scale(Mat4f& _matrix, const Vec3f& _scale)
{
	_matrix.M11(_matrix.M11() * _scale.X());
	_matrix.M21(_matrix.M21() * _scale.X());
	_matrix.M31(_matrix.M31() * _scale.X());
	_matrix.M41(_matrix.M41() * _scale.X());

	_matrix.M12(_matrix.M12() * _scale.Y());
	_matrix.M22(_matrix.M22() * _scale.Y());
	_matrix.M32(_matrix.M32() * _scale.Y());
	_matrix.M42(_matrix.M42() * _scale.Y());

	_matrix.M13(_matrix.M13() * _scale.Z());
	_matrix.M23(_matrix.M23() * _scale.Z());
	_matrix.M33(_matrix.M33() * _scale.Z());
	_matrix.M43(_matrix.M43() * _scale.Z());

	return _matrix;
}
Mat4f Mat4f::Persp(float _fov, float _aspectRation, float _nearPlane, float _farPlane)
{
	float top = tan(_fov * 0.5f) * _nearPlane;
	float bottom = -top;
	float left = _aspectRation * bottom;
	float right = _aspectRation * top;

	return Mat4f(
		(2.0f * _nearPlane) / (right - left), 0.0f, (right + left) / (right - left), 0.0f,
		0.0f, (2.0f * _nearPlane) / (top - bottom), (top + bottom) / (top - bottom), 0.0f,
		0.0f, 0.0f, - (_farPlane + _nearPlane) / (_farPlane - _nearPlane), -2.0f * _farPlane * _nearPlane / (_farPlane - _nearPlane),
		0.0f, 0.0f, -1.0f, 0.0);

	/*float h = 1.0f / tan(_fov * 0.5f);
	float negDepth = _nearPlane - _farPlane;

	return Mat4f(
		h / _aspectRation, 0.0f, 0.0f, 0.0f,
		0.0f, h, 0.0f, 0.0f,
		0.0f, 0.0f, _farPlane / negDepth, 2.0f * (_nearPlane * _farPlane) / negDepth,
		0.0f, 0.0f, -1.0f, 0.0f
		);*/
}
Mat4f Mat4f::Ortho(float _left, float _right, float _bottom, float _top, float _nearPlane, float _farPlane)
{
	return Mat4f(
		2.0f / (_right - _left), 0.0f, 0.0f, -(_right + _left) / (_right - _left),
		0.0f, 2.0f / (_top - _bottom), 0.0f,  -(_top + _bottom) / (_top - _bottom),
		0.0f, 0.0f, -2.0f / (_farPlane - _nearPlane), -(_farPlane + _nearPlane) / (_farPlane - _nearPlane),
		0.0f, 0.0f, 0.0f, 1.0f);
}

Mat4f Mat4f::operator * (const Mat4f& _matrix) const
{
	Mat4f tmp(
		M11() * _matrix.M11() + M12() * _matrix.M21() + M13() * _matrix.M31() + M14() * _matrix.M41(),
		M11() * _matrix.M12() + M12() * _matrix.M22() + M13() * _matrix.M32() + M14() * _matrix.M42(),
		M11() * _matrix.M13() + M12() * _matrix.M23() + M13() * _matrix.M33() + M14() * _matrix.M43(),
		M11() * _matrix.M14() + M12() * _matrix.M24() + M13() * _matrix.M34() + M14() * _matrix.M44(),

		M21() * _matrix.M11() + M22() * _matrix.M21() + M23() * _matrix.M31() + M24() * _matrix.M41(),
		M21() * _matrix.M12() + M22() * _matrix.M22() + M23() * _matrix.M32() + M24() * _matrix.M42(),
		M21() * _matrix.M13() + M22() * _matrix.M23() + M23() * _matrix.M33() + M24() * _matrix.M43(),
		M21() * _matrix.M14() + M22() * _matrix.M24() + M23() * _matrix.M34() + M24() * _matrix.M44(),

		M31() * _matrix.M11() + M32() * _matrix.M21() + M33() * _matrix.M31() + M34() * _matrix.M41(),
		M31() * _matrix.M12() + M32() * _matrix.M22() + M33() * _matrix.M32() + M34() * _matrix.M42(),
		M31() * _matrix.M13() + M32() * _matrix.M23() + M33() * _matrix.M33() + M34() * _matrix.M43(),
		M31() * _matrix.M14() + M32() * _matrix.M24() + M33() * _matrix.M34() + M34() * _matrix.M44(),

		M41() * _matrix.M11() + M42() * _matrix.M21() + M43() * _matrix.M31() + M44() * _matrix.M41(),
		M41() * _matrix.M12() + M42() * _matrix.M22() + M43() * _matrix.M32() + M44() * _matrix.M42(),
		M41() * _matrix.M13() + M42() * _matrix.M23() + M43() * _matrix.M33() + M44() * _matrix.M43(),
		M41() * _matrix.M14() + M42() * _matrix.M24() + M43() * _matrix.M34() + M44() * _matrix.M44()
		);

	return tmp;
}
Mat4f Mat4f::operator * (const float _scalar) const
{
	Mat4f m(values);
	m *= _scalar;
	return m;
}
Vec4f Mat4f::operator * (const Vec4f& _vector) const
{
	return Vec4f(
		M11() * _vector.X() + M12() * _vector.Y() + M13() * _vector.Z() + M14() * _vector.W(),
		M21() * _vector.X() + M22() * _vector.Y() + M23() * _vector.Z() + M24() * _vector.W(),
		M31() * _vector.X() + M32() * _vector.Y() + M33() * _vector.Z() + M34() * _vector.W(),
		M41() * _vector.X() + M42() * _vector.Y() + M43() * _vector.Z() + M44() * _vector.W()
		);
}
Vec3f Mat4f::operator * (const Vec3f& _vector) const
{
	return Vec3f(
		M11() * _vector.X() + M12() * _vector.Y() + M13() * _vector.Z(),
		M21() * _vector.X() + M22() * _vector.Y() + M23() * _vector.Z(),
		M31() * _vector.X() + M32() * _vector.Y() + M33() * _vector.Z()
		);
}
Mat4f Mat4f::operator / (const float _scalar) const
{
	float rezi = 1.0f / _scalar;
	Mat4f m(values);
	m *= rezi;
	return m;
}
Mat4f& Mat4f::operator *= (const Mat4f& _matrix)
{
	Mat4f z = *this;

	M11(z.M11() * _matrix.M11() + z.M12() * _matrix.M21() + z.M13() * _matrix.M31() + z.M14() * _matrix.M41());
	M12(z.M11() * _matrix.M12() + z.M12() * _matrix.M22() + z.M13() * _matrix.M32() + z.M14() * _matrix.M42());
	M13(z.M11() * _matrix.M13() + z.M12() * _matrix.M23() + z.M13() * _matrix.M33() + z.M14() * _matrix.M43());
	M14(z.M11() * _matrix.M14() + z.M12() * _matrix.M24() + z.M13() * _matrix.M34() + z.M14() * _matrix.M44());

	M21(z.M21() * _matrix.M11() + z.M22() * _matrix.M21() + z.M23() * _matrix.M31() + z.M24() * _matrix.M41());
	M22(z.M21() * _matrix.M12() + z.M22() * _matrix.M22() + z.M23() * _matrix.M32() + z.M24() * _matrix.M42());
	M23(z.M21() * _matrix.M13() + z.M22() * _matrix.M23() + z.M23() * _matrix.M33() + z.M24() * _matrix.M43());
	M24(z.M21() * _matrix.M14() + z.M22() * _matrix.M24() + z.M23() * _matrix.M34() + z.M24() * _matrix.M44());

	M31(z.M31() * _matrix.M11() + z.M32() * _matrix.M21() + z.M33() * _matrix.M31() + z.M34() * _matrix.M41());
	M32(z.M31() * _matrix.M12() + z.M32() * _matrix.M22() + z.M33() * _matrix.M32() + z.M34() * _matrix.M42());
	M33(z.M31() * _matrix.M13() + z.M32() * _matrix.M23() + z.M33() * _matrix.M33() + z.M34() * _matrix.M43());
	M34(z.M31() * _matrix.M14() + z.M32() * _matrix.M24() + z.M33() * _matrix.M34() + z.M34() * _matrix.M44());

	M41(z.M41() * _matrix.M11() + z.M42() * _matrix.M21() + z.M43() * _matrix.M31() + z.M44() * _matrix.M41());
	M42(z.M41() * _matrix.M12() + z.M42() * _matrix.M22() + z.M43() * _matrix.M32() + z.M44() * _matrix.M42());
	M43(z.M41() * _matrix.M13() + z.M42() * _matrix.M23() + z.M43() * _matrix.M33() + z.M44() * _matrix.M43());
	M44(z.M41() * _matrix.M14() + z.M42() * _matrix.M24() + z.M43() * _matrix.M34() + z.M44() * _matrix.M44());

	return *this;
}
Mat4f& Mat4f::operator *= (const float _scalar)
{
	for (int i = 0; i < 16; i++)
		values[i] *= _scalar;
	
	return *this;
}
Mat4f& Mat4f::operator /= (const float _scalar)
{
	float rezi = 1.0f / _scalar;

	for (int i = 0; i < 16; i++)
		values[i] *= rezi;
	
	return *this;
}
Mat4f& Mat4f::operator = (const Mat4f& _matrix)
{
	if (this != &_matrix)
	{
		for (int i = 0; i < 16; i++)
			values[i] = _matrix.values[i];
	}

	return *this;
}
bool Mat4f::operator == (const Mat4f& _matrix) const
{
	for (int i = 0; i < 16; i++)
		if (values[i] != _matrix.values[i])
			return false;

	return true;
}
bool Mat4f::operator != (const Mat4f& _matrix) const
{
	for (int i = 0; i < 16; i++)
		if (values[i] != _matrix.values[i])
			return true;

	return false;
}

inline float Mat4f::M(int _column, int _row) const
{
	return values[_row * 4 + _column];
}
inline void Mat4f::M(int _column, int _row, float _value)
{
	values[_row * 4 + _column] = _value;
}

std::ostream& operator <<(std::ostream &os, const Mat4f &obj)
{
	os << "[" << obj.M11() << ", " << obj.M12() << ", " << obj.M13() << ", " << obj.M14() << "; " << obj.M21() << ", " <<  obj.M22() << ", " <<  obj.M23() << ", " << obj.M24() << "; " <<  obj.M31() << ", " <<  obj.M32() << ", " <<  obj.M33() << ", " << obj.M34() << "; " << obj.M41() << ", " << obj.M42() << ", " << obj.M43() << ", " << obj.M44() << "]";
	return os;
}