#ifndef _ENGINE_MATH_CPPONLY_MAT4F_HPP
#define _ENGINE_MATH_CPPONLY_MAT4F_HPP

#include "../../dllconfig.hpp"
#include "Mat3f.hpp"
#include "Vec3f.hpp"
#include "Vec4f.hpp"

#include <iostream>

namespace Engine
{
	namespace Math
	{
		namespace CPPonly
		{
			/// <summary>
			/// Defines a 4x4 matrix and provides functions for working with them
			/// </summary>
			class _DLL_DECLSPEC Mat4f
			{
			public:
				/// <summary>
				/// Initializes a new Mat4f object and initializes it with a identity matrix
				/// </summary>
				Mat4f();
				/// <summary>
				/// Initializes a new Mat4f object and sets the diagonal to a value
				/// </summary>
				/// <param name="_diag">Value of the diagonal elements</param>
				Mat4f(float _diag);
				/// <summary>
				/// Initializes a new Mat4f object
				/// </summary>
				/// <param name="_m11">Value for the first column in the first row</param>
				/// <param name="_m12">Value for the second column in the first row</param>
				/// <param name="_m13">Value for the third column in the first row</param>
				/// <param name="_m14">Value for the fourth column in the first row</param>
				/// <param name="_m21">Value for the first column in the second row</param>
				/// <param name="_m22">Value for the second column in the second row</param>
				/// <param name="_m23">Value for the third column in the second row</param>
				/// <param name="_m24">Value for the fourth column in the second row</param>
				/// <param name="_m31">Value for the first column in the third row</param>
				/// <param name="_m32">Value for the second column in the third row</param>
				/// <param name="_m33">Value for the third column in the third row</param>
				/// <param name="_m34">Value for the fourth column in the third row</param>
				/// <param name="_m41">Value for the first column in the fourth row</param>
				/// <param name="_m42">Value for the second column in the fourth row</param>
				/// <param name="_m43">Value for the third column in the fourth row</param>
				/// <param name="_m44">Value for the fourth column in the fourth row</param>
				Mat4f(float _m11, float _m12, float _m13, float _m14, float _m21, float _m22, float _m23, float _m24, float _m31, float _m32, float _m33, float _m34, float _m41, float _m42, float _m43, float _m44);
				/// <summary>
				/// Initializes a new Mat4f object
				/// </summary>
				/// <param name="_m11">Values for the matrix</param>
				Mat4f(const float* _values);
				/// <summary>
				/// Initializes a new Mat4f object
				/// <summary>
				/// <param name="_toCopy">Matrix to copy the values from</param>
				Mat4f(const Mat4f& _toCopy);
				/// <summary>
				/// Initializes a new Mat4f object
				/// <summary>
				/// <param name="_toCopy">Matrix to copy the values from</param>
				Mat4f(Mat3f& _toCopy);
				/// <summary>
				/// Deletes all the data of a Mat4f object
				/// </summary>
				~Mat4f();

				/// <summary>
				/// Returns the value of the first column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M11() const;
				/// <summary>
				/// Sets the value of the first column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M11(float _value);
				/// <summary>
				/// Returns the value of the second column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M12() const;
				/// <summary>
				/// Sets the value of the second column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M12(float _value);
				/// <summary>
				/// Returns the value of the third column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M13() const;
				/// <summary>
				/// Sets the value of the third column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M13(float _value);
				/// <summary>
				/// Returns the value of the forth column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M14() const;
				/// <summary>
				/// Sets the value of the fourth column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M14(float _value);

				/// <summary>
				/// Returns the value of the first column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M21() const;
				/// <summary>
				/// Sets the value of the first column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M21(float _value);
				/// <summary>
				/// Returns the value of the second column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M22() const;
				/// <summary>
				/// Sets the value of the second column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M22(float _value);
				/// <summary>
				/// Returns the value of the third column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M23() const;
				/// <summary>
				/// Sets the value of the third column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M23(float _value);
				/// <summary>
				/// Returns the value of the fourth column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M24() const;
				/// <summary>
				/// Sets the value of the fourth column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M24(float _value);

				/// <summary>
				/// Returns the value of the first column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M31() const;
				/// <summary>
				/// Sets the value of the first column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M31(float _value);
				/// <summary>
				/// Returns the value of the second column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M32() const;
				/// <summary>
				/// Sets the value of the second column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M32(float _value);
				/// <summary>
				/// Returns the value of the thrid column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M33() const;
				/// <summary>
				/// Sets the value of the third column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M33(float _value);
				/// <summary>
				/// Returns the value of the fourth column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M34() const;
				/// <summary>
				/// Sets the value of the fourth column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M34(float _value);

				/// <summary>
				/// Returns the value of the first column in the fourth row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M41() const;
				/// <summary>
				/// Sets the value of the first column in the fourth row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M41(float _value);
				/// <summary>
				/// Returns the value of the second column in the fourth row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M42() const;
				/// <summary>
				/// Sets the value of the second column in the fourth row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M42(float _value);
				/// <summary>
				/// Returns the value of the third column in the fourth row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M43() const;
				/// <summary>
				/// Sets the value of the third column in the fourth row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M43(float _value);
				/// <summary>
				/// Returns the value of the fourth column in the fourth row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M44() const;
				/// <summary>
				/// Sets the value of the fourth column in the fourth row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>A reference to the matrix</returns>
				Mat4f& M44(float _value);

				/// <summary>
				/// Returns a pointer to an OpenGL array
				/// </summary>
				/// <returns>Pointer to OpenGL array</returns>
				float* GetGlMatrix() const;
				/// <summary>
				/// Returns if the matrix is stored in column order
				/// </summary>
				/// <returns>True if stored in column order, else when not</returns>
				static bool IsColumnMajor();

				/// <summary>
				/// Transposes the matrix
				/// </summary>
				/// <returns>A reference to the transposed matrix</returns>
				Mat4f& Transpose();
				/// <summary>
				/// Transposes a Mat4f object
				/// </summary>
				/// <param name="_matrix">Matrix to transpose</param>
				/// <returns>The transposed matrix</returns>
				static Mat4f Transpose(const Mat4f& _matrix);

				/// <summary>
				/// Creates a rotation matrix around the x-axis
				/// </summary>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A rotation matrix</returns>
				static Mat4f RotateX(float _angle);
				/// <summary>
				/// Rotates a matrix around the x-axis
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat4f& RotateX(Mat4f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix around the y-axis
				/// </summary>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A rotation matrix</returns>
				static Mat4f RotateY(float _angle);
				/// <summary>
				/// Rotates a matrix around the y-axis
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat4f& RotateY(Mat4f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix around the y-axis
				/// </summary>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A rotation matrix</returns>
				static Mat4f RotateZ(float _angle);
				/// <summary>
				/// Rotates a matrix around the z-axis
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat4f& RotateZ(Mat4f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix around a defined matrix
				/// </summary>
				/// <param name="_axis">The axis to rotate around</param>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A rotation matrix</returns>
				static Mat4f RotateAxis(const Vec3f& _axis, float _angle);
				/// <summary>
				/// Rotates a matrix around a defined axis
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_axis">The axis to rotate around</param>
				/// <param name="_angle">The angle in radian to rotate</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat4f& RotateAxis(Mat4f& _matrix, const Vec3f& _axis, float _angle);
				/// <summary>
				/// Creates a translation matrix
				/// </summary>
				/// <param name="_direction">The direction to translate</param>
				/// <returns>The translation matrix</returns>
				static Mat4f Translate(const Vec3f& _direction);
				/// <summary>
				/// Translates a matrix
				/// </summary>
				/// <param name="_direction">The direction to translate</param>
				/// <returns>A reference to the translated matrix</returns>
				static Mat4f& Translate(Mat4f& _matrix, const Vec3f& _direction);
				/// <summary>
				/// Creates a scaling matrix
				/// </summary>
				/// <param name="_scales">The scaling factors</param>
				/// <returns>The scaling matrix</returns>
				static Mat4f Scale(const Vec3f& _scales);
				/// <summary>
				/// Scales a matrix
				/// </summary>
				/// <param name="_matrix">The matrix to scale</param>
				/// <param name="_scale">The scaling values</param>
				/// <returns>A reference to the scaled matrix</returns>
				static Mat4f& Scale(Mat4f& _matrix, const Vec3f& _scale);
				/// <summary>
				/// Creates a perspective projection matrix
				/// </summary>
				/// <param name="_fov">The opening angle of the camera in radian</param>
				/// <param name="_aspectRation">The ration between width and height</param>
				/// <param name="_nearPlane">The distance to the near plane</param>
				/// <param name="_farPlane">The distance to the far plane</param>
				/// <returns>The projection matrix</returns>
				static Mat4f Persp(float _fov, float _aspectRation, float _nearPlane, float _farPlane);
				/// <summary>
				/// Creates a orthographic projection matrix
				/// </summary>
				/// <param name="_left">Value of the left side</param>
				/// <param name="_right">Value of the right side</param>
				/// <param name="_bottom">The value of the bottom</param>
				/// <param name="_top">The value of the top</param>
				/// <param name="_nearPlane">The distance to the near plane</param>
				/// <param name="_farPlane">The distance to the far plane</param>
				/// <returns>The projection matrix</returns>
				static Mat4f Ortho(float _left, float _right, float _bottom, float _top, float _nearPlane, float _farPlane);


				Mat4f& Invert();
				static Mat4f Invert(const Mat4f& _matrix);
				float Det() const;
				static float Det(const Mat4f& _matrix);
				Mat4f Adj() const;
				static Mat4f Adj(const Mat4f& _matrix);

				static Mat4f GetRotation(const Mat4f& _matrix);
				static Mat4f RemoveScaling(const Mat4f& _matrix);

				/// <summary>
				/// Defines the multiplication of two matrices
				/// </summary>
				/// <param name="_matrix>The matrix to multiply with</param>
				/// <returns>The rotated matrix</returns>
				Mat4f operator * (const Mat4f& _matrix) const;

				Mat4f operator * (const float _scalar) const;
				/// <summary>
				/// Defines the multiplication of a vector and a matrix
				/// </summary>
				/// <param name="_vector>The vector</param>
				/// <returns>The multiplied vector</param>
				Vec4f operator * (const Vec4f& _vector) const;
				/// <summary>
				/// Defines the multiplication of a vector and a matrix
				/// </summary>
				/// <param name="_vector>The vector</param>
				/// <returns>The multiplied vector</param>
				Vec3f operator * (const Vec3f& _vector) const;

				Mat4f operator / (const float _scalar) const;

				/// <summary>
				/// Defines the multiplication of two matrices
				/// </summary>
				/// <param name="_matrix">To matrix to multiply with</param>
				/// <returns>A reference to the multiplied matrix</returns>
				Mat4f& operator *= (const Mat4f& _matrix);

				Mat4f& operator *= (const float _scalar);

				Mat4f& operator /= (const float _scalar);
				/// <summary>
				/// Defines the assignment operator to get a deep copy
				/// </summary>
				/// <param name="_matrix">Source to copy from</param>
				/// <returns>Copied Mat3 object</returns>
				Mat4f& operator = (const Mat4f& _matrix);
				/// <summary>
				/// Defines the equal operator for two matrices
				/// </summary>
				/// <param name="_matrix">The matrix to compare</param>
				/// <returns>True if the two matrices are equal</returns>
				bool operator == (const Mat4f& _matrix) const;
				/// <summary>
				/// Defines the unequal operator for two matrices
				/// </summary>
				/// <param name="_matrix">The matrix to compare</param>
				/// <returns>True if the two matrices are unequal</returns>
				bool operator != (const Mat4f& _matrix) const;

			private:
				float M(int _row, int _column) const;
				void M(int _row, int _column, float _value);

				float values[16];
			};
		}
	}
}

_DLL_DECLSPEC std::ostream& operator <<(std::ostream &os, const Engine::Math::CPPonly::Mat4f &obj);

#endif