#define DLL_EXPORT

#include "Vec2f.hpp"
#include <algorithm>

using namespace Engine::Math::CPPonly;

Vec2f::Vec2f()
{
	values[0] = 0.0f;
	values[1] = 0.0f;
}

Vec2f::Vec2f(float _x, float _y)
{
	values[0] = _x;
	values[1] = _y;
}

Vec2f::Vec2f(float* _values)
{
	values[0] = _values[0];
	values[1] = _values[1];
}

Vec2f::Vec2f(const Vec2f& _refToVec)
{	
	memcpy(&values, &(_refToVec.values), sizeof(float) * 2);
}

Vec2f::~Vec2f()
{
}

float Vec2f::X() const
{
	return this->values[0];
}

Vec2f& Vec2f::X(float _x)
{
	values[0] = _x;
	return *this;
}

float Vec2f::Y() const
{
	return this->values[1];
}

Vec2f& Vec2f::Y(float _y)
{
	values[1] = _y;
	return *this;
}

float* Vec2f::GetGlVector() const
{
	return (float*)(&values[0]);
}

Vec2f& Vec2f::Normalize()
{
	float l = this->Length();

	this->values[0] /= l;
	this->values[1] /= l;

	return *this;
}

Vec2f Vec2f::Normalize(const Vec2f &_vector)
{
	float l = _vector.Length();

	return Vec2f(_vector.X() / l, _vector.Y() / l);
}

float Vec2f::Length() const
{
	return sqrt(X() * X() + Y() * Y());
}

float Vec2f::Length(const Vec2f &_vector)
{
	return sqrt(_vector.X() * _vector.X() + _vector.Y() * _vector.Y());
}

Vec2f Vec2f::ComponentMax(const Vec2f& _v1, const Vec2f& _v2)
{
	return Vec2f(std::max(_v1.X(), _v2.X()), std::max(_v1.Y(), _v2.Y()));
}

Vec2f Vec2f::ComponentMin(const Vec2f& _v1, const Vec2f& _v2)
{
	return Vec2f(std::min(_v1.X(), _v2.X()), std::min(_v1.Y(), _v2.Y()));
}

Vec2f Vec2f::Interpolate(const Vec2f& _v1, const Vec2f& _v2, float _a1, float _a2)
{
	float fac = _a1 / (_a1 + _a2);
	Vec2f diff = _v2 - _v1;

	return Vec2f(_v1.X() + fac * diff.X(), _v1.Y() + fac * diff.Y());
}

Vec2f Vec2f::operator+(const Vec2f& _toAdd) const
{
	return Vec2f(X() + _toAdd.X(), Y() + _toAdd.Y());
}

Vec2f& Vec2f::operator+=(const Vec2f& _toAdd)
{
	X(X() + _toAdd.X());
	Y(Y() + _toAdd.Y());
	return *this;
}

Vec2f Vec2f::operator-(const Vec2f& _toSubtract) const
{
	return Vec2f(X() - _toSubtract.X(), Y() - _toSubtract.Y());
}

Vec2f& Vec2f::operator-=(const Vec2f& _toSubtract)
{
	X(X() - _toSubtract.X());
	Y(Y() - _toSubtract.Y());
	return *this;
}

Vec2f Vec2f::operator*(float _scalar) const
{
	return Vec2f(X() * _scalar, Y() * _scalar);
}

Vec2f& Vec2f::operator*=(float _scalar)
{
	X(X() * _scalar);
	Y(Y() * _scalar);
	return *this;
}

Vec2f Vec2f::operator/ (float _scalar) const
{
	float rezi = 1.0f / _scalar;
	return (*this) * rezi;
}

Vec2f& Vec2f::operator /= (float _scalar)
{
	(*this) *= (1.0f / _scalar);
	return *this;
}

bool Vec2f::operator==(const Vec2f& _toCompare) const
{
	if (X() == _toCompare.X() && Y() == _toCompare.Y())
		return true;
	else
		return false;
}

bool Vec2f::operator!=(const Vec2f& _toCompare) const
{
	if (X() == _toCompare.X() && Y() == _toCompare.Y())
		return false;
	else
		return true;
}

Vec2f& Vec2f::operator=(const Vec2f& _copy)
{
	if (this != &_copy)
	{
		values[0] = _copy.X();
		values[1] = _copy.Y();
	}

	return *this;
}

std::ostream& operator <<(std::ostream &os, const Vec2f &obj)
{
	os << "[" << obj.X() << ", " << obj.Y() << "]";
	return os;
}