#ifndef _ENGINE_MATH_CPPONLY_VEC2F_HPP
#define _ENGINE_MATH_CPPONLY_VEC2F_HPP

#include "../../dllconfig.hpp"

#include <iostream>

namespace Engine
{
	namespace Math
	{
		namespace CPPonly
		{
			/// <summary>
			/// Descripes a two-dimensional vector and provies functions for working with them
			/// </summary>
			class _DLL_DECLSPEC Vec2f
			{

			public:
				/// <summary>
				/// Initializes a new instance of the Vec2 class
				/// </summary>
				Vec2f();
				/// <summary>
				/// Initializes an new instance of the Vec2 class
				/// </summary>
				/// <param name="_x">x-coordinate</param>
				/// <param name="_y">y-coordinate</param>
				Vec2f(float _x, float _y);

				Vec2f(float* _values);

				/// <summary>
				/// Initializes an new instance of the Vec2 class
				/// </summary>
				/// <param name="_refToVec">Vector to copy</param>
				Vec2f(const Vec2f& _refToVec);
				/// <summary>
				/// Releases the data from a Vec2 class
				/// </summary>
				~Vec2f();

				/// <summary>
				/// Sets or retrieves the value of the x-coordinate
				/// </summary>
				inline float X() const;
				/// <summary>
				/// Sets or retrieves the value of the x-coordinate
				/// </summary>
				/// <param name="_x">New value for the x-coordinate</param>
				/// <returns>A reference to the Vec2 class</returns>
				inline Vec2f& X(float _x);
				/// <summary>
				/// Sets or retrieves the value of the y-coordinate
				/// </summary>
				inline float Y() const;
				/// <summary>
				/// Sets or retrieves the value of the y-coordinate
				/// </summary>
				/// <param name="_x">New value for the x-coordinate</param>
				/// <returns>A reference to the Vec2 class</returns>
				inline Vec2f& Y(float _y);
				/// <summary>
				/// Returns a pointer to a OpenGL vector
				/// </summary>
				/// <returns>Pointer to OpenGL vector</returns>
				inline float* GetGlVector() const;

				/// <summary>
				/// Normalizes the vector
				/// </summary>
				/// <returns>Normalized vector</returns>
				inline Vec2f& Normalize();
				/// <summary>
				/// Normalizes a Vec2 class
				/// </summary>
				/// <param name="_vector">A Vec2 to normalize</param>
				/// <returns>The normalized vector</returns>
				static Vec2f Normalize(const Vec2f &_vector);

				/// <summary>
				/// Returns the lenght of the vector
				/// </summary>
				inline float Length() const;
				/// <summary>
				/// Returns the lenght of a Vec2
				/// </summary>
				/// <param name="_vector">The vector</param>
				/// <returns>The lenght of the vector</returns>
				static float Length(const Vec2f &_vector);

				static Vec2f ComponentMax(const Vec2f& _v1, const Vec2f& _v2);
				static Vec2f ComponentMin(const Vec2f& _v1, const Vec2f& _v2);

				static Vec2f Interpolate(const Vec2f& _v1, const Vec2f& _v2, float _a1, float _a2);

				/// <summary>
				/// Defines a pointwise addition for Vec2 classes
				/// </summary>
				/// <param name="_toAdd">Vector to add</param>
				/// <returns>The added Vector</returns>
				inline Vec2f operator + (const Vec2f& _toAdd) const;
				/// <summary>
				/// Subtracts to Vec2 classes pointwise
				/// </summary>
				/// <param name="_toSubtract">Vector to subtract</param>
				/// <returns>The Subtracted Vector</returns>
				inline Vec2f operator - (const Vec2f& _toSubtract) const;
				/// <summary>
				/// Multiplies the vector with a scalar
				/// </summary>
				/// <param name="_scalar">Scalar to multiply with</param>
				/// <returns>The multiplied vector</returns>
				inline Vec2f operator * (float _scalar) const;

				inline Vec2f operator / (float _scalar) const;
				/// <summary>
				/// Defines a pointwise addition for Vec2 classes
				/// </summary>
				/// <param name="_toAdd">Vector to add</param>
				/// <returns>The added Vector</returns>
				inline Vec2f& operator += (const Vec2f& _toAdd);
				/// <summary>
				/// Subtracts to Vec2 classes pointwise
				/// </summary>
				/// <param name="_toSubtract">Vector to subtract</param>
				/// <returns>The Subtracted Vector</returns>
				inline Vec2f& operator -= (const Vec2f& _toSubtract);
				/// <summary>
				/// Multiplies the vector with a scalar
				/// </summary>
				/// <param name="_scalar">Scalar to multiply with</param>
				/// <returns>Multiplied vector</returns>
				inline Vec2f& operator *= (float _scalar);

				inline Vec2f& operator /= (float _scalar);
				/// <summary>
				/// Defines the equal-operator for two Vec2 objects
				/// </summary>
				/// <param name="_toCheck">Vec2 class to check with</param>
				/// <returns>true if vectors are equal, false otherwise</returns>
				bool operator == (const Vec2f& _toCompare) const;
				/// <summary>
				/// Defines the unequal-operator for two Vec2 objects
				/// </summary>
				/// <param name="_toCheck">Vec2 class to check with</param>
				/// <returns>true if vectors are unequal, false otherwise</returns>
				bool operator != (const Vec2f& _toCompare) const;
				/// <summary>
				/// Defines the assignment operator to get a deep copy
				/// </summary>
				/// <param name="_copy">Source to copy from</param>
				/// <returns>Copied Vec2 object</returns>
				Vec2f& operator = (const Vec2f& _copy);

			private:
				float values[2];
			};
		}
	}
}

_DLL_DECLSPEC std::ostream& operator <<(std::ostream &os, const Engine::Math::CPPonly::Vec2f &obj);

#endif