#ifndef _ENGINE_MATH_CPPONLY_VEC3F_HPP
#define _ENGINE_MATH_CPPONLY_VEC3F_HPP

#include "../../dllconfig.hpp"

#include <iostream>

namespace Engine 
{
	namespace Math 
	{
		namespace CPPonly
		{
			class Vec4f;

			/// <summary>
			/// Descripes a three-dimensional vector and provides functions for working with them
			/// </summary>
			class _DLL_DECLSPEC Vec3f 
			{
			public:
				/// <summary>
				/// Initializes a new instance of the Vec3f class
				/// </summary>
				Vec3f(void);
				/// <summary>
				/// Initializes a new instance of the Vec3f class
				/// </summary>
				/// <param name="_x">x-coordinate</param>
				/// <param name="_y">y-coordinate</param>
				/// <param name="_z">z-coordinate</param>
				Vec3f(float _x, float _y, float _z);
				/// <summary>
				/// Initializes a new instance of the Vec3f class
				/// </summary>
				/// <param name="_retToVec">Vector to copy</param>
				Vec3f(const Vec3f &_refToVec);
				/// <summary>
				/// Initializes a new instance of the Vec3f class
				/// </summary>
				/// <param name="_values">Values for the Vec3f</param>
				Vec3f(float* _values);
				/// <summary>
				/// Initializes a new instance of the Vec3f class, uses the X, Y and Z index of the specified Vec4f for the eqivalent indeces in the new Vec3
				/// </summary>
				/// <param name="_values">Values for the Vec3f</param>
				Vec3f(const Vec4f &_vec4);
				/// <summary>
				/// Releases the data from a Vec3f class
				/// </summary>
				~Vec3f(void);

				/// <summary>
				/// Sets or retrieves the value of the x-coordinate
				/// </summary>
				inline float X() const;
				/// <summary>
				/// Sets or retrieves the value of the x-coordinate
				/// </summary>
				/// <param name="_x">New value for the x-coordinate</param>
				/// <returns>A reference to the Vec3f class</returns>
				inline Vec3f& X(float _x);
				///<summary>
				///Sets or retrieves the value of the y-coordinate
				/// </summary>
				inline float Y() const;
				///<summary>
				///Sets or retrieves the value of the y-coordinate
				/// </summary>
				/// <param name="_x">New value for the y-coordinate</param>
				/// <returns>A reference to the Vec3f class</returns>
				inline Vec3f& Y(float _y);
				///<summary>
				///Sets or retrieves the value of the z-coordinate
				/// </summary>
				inline float Z() const;
				///<summary>
				///Sets or retrieves the value of the z-coordinate
				/// </summary>
				/// <param name="_x">New value for the z-coordinate</param>
				/// <returns>A reference to the Vec3f class</returns>
				inline Vec3f& Z(float _z);
				/// <summary>
				/// Returns a pointer to a OpenGL vector
				/// </summary>
				/// <returns>Pointer to OpenGL vector</returns>
				inline float* GetGlVector() const;

				inline Vec4f GetVec4f() const;

				/// <summary>
				/// Normalizes the vector
				/// </summary>
				/// <returns>Normalized vector</returns>
				inline Vec3f& Normalize();
				/// <summary>
				/// Normalizes a Vec3f class
				/// </summary>
				/// <param name="_vector">A Vec3f to normalize</param>
				/// <returns>The normalized vector</returns>
				static Vec3f Normalize(const Vec3f &_vector);

				/// <summary>
				/// Returns the lenght of the vector
				/// </summary>
				float inline Length() const;
				/// <summary>
				/// Returns the lenght of a Vec3f
				/// </summary>
				/// <param name="_vector">The vector</param>
				/// <returns>The lenght of the vector</returns>
				static float Length(Vec3f &_vector);

				/// <summary>
				/// Calculates the dot product of two Vec3f objects
				/// </summary>
				/// <param name="_left">Left vector</param>
				/// <param name="_right">Right vector</param>
				/// <returns>The dot product of the two vectors</returns>
				static float Dot(const Vec3f &_left, const Vec3f &_right);
				/// <summary>
				/// Calculates the cross product of two Vec3f objects
				/// </summary>
				/// <param name="_left">Left vector</param>
				/// <param name="_right">Right vector</param>
				/// <returns>The cross product of the two vectors</returns>
				static Vec3f Cross(const Vec3f &_left, const Vec3f &_right);
				/// <summary>
				/// Returns the componentwise maximum vector
				/// </summary>
				/// <param name="_vector">Vector to compare</param>
				/// <returns>The maximum vector</returns>
				static Vec3f ComponentMax(const Vec3f& _v1, const Vec3f& _v2);
				/// <summary>
				/// Returns the componentwise minimum vector
				/// </summary>
				/// <param name="_vector">Vector to compare</param>
				/// <returns>The minimum vector</returns>
				static Vec3f ComponentMin(const Vec3f& _v1, const Vec3f& _v2);

				static Vec3f Interpolate(const Vec3f& _v1, const Vec3f& _v2, float _a1, float _a2);

				/// <summary>
				/// Defines a pointwise addition for Vec3f classes
				/// </summary>
				/// <param name="_toAdd">Vector to add</param>
				/// <returns>The added Vector</returns>
				inline Vec3f operator + (const Vec3f &_toAdd) const;

				inline Vec3f operator + (const Vec4f &_toAdd) const;

				/// <summary>
				/// Subtracts to Vec3f classes pointwise
				/// </summary>
				/// <param name="_toSubtract">Vector to subtract</param>
				/// <returns>The Subtracted Vector</returns>
				inline Vec3f operator - (const Vec3f &_toSubtract) const;

				inline Vec3f operator - (const Vec4f &_toSubtract) const;

				/// <summary>
				/// Multiplies the vector with a scalar
				/// </summary>
				/// <param name="_scalar">Scalar to multiply with</param>
				/// <returns>The multiplied vector</returns>
				inline Vec3f operator * (float _scalar) const;

				inline Vec3f operator / (float _scalar) const;

				/// <summary>
				/// Defines a pointwise addition for Vec3f classes
				/// </summary>
				/// <param name="_toAdd">Vector to add</param>
				/// <returns>The added Vector</returns>
				inline Vec3f& operator += (const Vec3f &_toAdd);

				inline Vec3f& operator += (const Vec4f &_toAdd);

				/// <summary>
				/// Subtracts to Vec3f classes pointwise
				/// </summary>
				/// <param name="_toSubtract">Vector to subtract</param>
				/// <returns>The Subtracted Vector</returns>
				inline Vec3f& operator -= (const Vec3f &_toSubtract);

				inline Vec3f& operator -= (const Vec4f &_toSubtract);

				/// <summary>
				/// Multiplies the vector with a scalar
				/// </summary>
				/// <param name="_scalar">Scalar to multiply with</param>
				/// <returns>Multiplied vector</returns>
				inline Vec3f& operator *= (float _scalar);

				inline Vec3f& operator /= (float _scalar);

				/// <summary>
				/// Defines the equal-operator for two Vec3f objects
				/// </summary>
				/// <param name="_toCompare">Vec3f class to check with</param>
				/// <returns>true if vectors are equal, false otherwise</returns>
				bool operator == (const Vec3f &_toCompare) const;
				/// <summary>
				/// Defines the unequal-operator for two Vec3f objects
				/// </summary>
				/// <param name="_toCompare">Vec3f class to check with</param>
				/// <returns>true if vectors are unequal, false otherwise</returns>
				bool operator != (const Vec3f &_toCompare) const;

				/// <summary>
				/// Defines the assignment operator to get a deep copy
				/// </summary>
				/// <param name="_copy">Source to copy from</param>
				/// <returns>Copied Vec3f object</returns>
				Vec3f& operator = (const Vec3f &_copy);

			private:
				float values[3];
			};
		}
	}
}

_DLL_DECLSPEC std::ostream& operator <<(std::ostream &os, const Engine::Math::CPPonly::Vec3f &obj);

#endif

