#define DLL_EXPORT

#include "Vec4f.hpp"

#include <math.h>
#include <algorithm>

using namespace Engine::Math::CPPonly;

Vec4f::Vec4f()
{
	values[0]=0;
	values[1]=0;
	values[2]=0;
	values[3]=1;
}

Vec4f::Vec4f(float* _values)
{
	for (int i = 0; i < 4; i++)
		values[i] = _values[i];
}

Vec4f::Vec4f(float _x, float _y, float _z)
{
	values[0]=_x;
	values[1]=_y;
	values[2]=_z;
	values[3]=1;
}

Vec4f::Vec4f(float _x, float _y, float _z, float _w)
{
	values[0]=_x;
	values[1]=_y;
	values[2]=_z;
	values[3]=_w;
}

Vec4f::Vec4f(const Vec4f &_refToVec)
{
	values[0]=_refToVec.values[0];
	values[1]=_refToVec.values[1];
	values[2]=_refToVec.values[2];
	values[3]=_refToVec.values[3];
}

Vec4f::Vec4f(const Vec3f &_refToVec, float yW)
{
	values[0]=_refToVec.X();
	values[1]=_refToVec.Y();
	values[2]=_refToVec.Z();
	values[3]=yW;
}

Vec4f& Vec4f::operator = (const Vec4f &_rhs)
{
	values[0]=_rhs.values[0];
	values[1]=_rhs.values[1];
	values[2]=_rhs.values[2];
	values[3]=_rhs.values[3];
	return *this;
}

Vec4f::~Vec4f()
{
}


float Vec4f::X() const
{
	return values[0];
}

Vec4f& Vec4f::X(float _x)
{
	values[0]=_x;
	return *this;
}

float Vec4f::Y() const
{
	return values[1];
}

Vec4f& Vec4f::Y(float _y)
{
	values[1]=_y;
	return *this;
}

float Vec4f::Z() const
{
	return values[2];
}

Vec4f& Vec4f::Z(float _z)
{
	values[2]=_z;
	return *this;
}

float Vec4f::W() const
{
	return values[3];
}

Vec4f& Vec4f::W(float _w)
{
	values[3]=_w;
	return *this;
}

float* Vec4f::GetGlVector() const
{
	return (float*)(&(values[0]));
}

Vec3f Vec4f::GetVec3f() const
{
	return Vec3f(X(), Y(), Z());
}


Vec4f& Vec4f::Homogenize()
{
	if(values[3]!=1.0f){
		values[0]/=values[3];
		values[1]/=values[3];
		values[2]/=values[3];
		values[3]=1;
	}
	return *this;
}

Vec4f Vec4f::Homogenize(const Vec4f& _vector)
{
	if(_vector.values[3] != 1.0f)
	{
		float rezi = 1.0f / _vector.values[3];
		return Vec4f(_vector.values[0] * rezi, _vector.values[1] * rezi, _vector.values[2] * rezi, 1.0f);
	}
	
	return _vector;
}

Vec4f& Vec4f::Normalize3()
{
	float length=sqrt(values[0]*values[0] + values[1]*values[1] + values[2]*values[2]);
	values[0]/=length;
	values[1]/=length;
	values[2]/=length;
	return *this;
}

Vec4f Vec4f::Normalize3(const Vec4f &_vector)
{
	return Vec4f(_vector).Normalize3();
}

Vec4f& Vec4f::Normalize()
{
	float length=sqrt(values[0]*values[0] + values[1]*values[1] + values[2]*values[2] + values[3]*values[3]);
	values[0]/=length;
	values[1]/=length;
	values[2]/=length;
	values[3]/=length;
	return *this;
}

Vec4f Vec4f::Normalize(const Vec4f &_vector)
{
	return Vec4f(_vector).Normalize();
}
			
float Vec4f::Length3() const
{
	return sqrt(values[0]*values[0] + values[1]*values[1] + values[2]*values[2]);
}

float Vec4f::Length3(const Vec4f &_vector){
	return _vector.Length3();
}

float Vec4f::Length() const
{
	return sqrt(values[0]*values[0] + values[1]*values[1] + values[2]*values[2] + values[3]*values[3]);
}

float Vec4f::Length(const Vec4f &_vector)
{
	return _vector.Length();
}

float Vec4f::Dot3(const Vec4f &_left, const Vec4f &_right)
{
	return _left.values[0]*_right.values[0] + 
		   _left.values[1]*_right.values[1] + 
		   _left.values[2]*_right.values[2];
}

float Vec4f::Dot(const Vec4f &_left, const Vec4f &_right)
{
	return _left.values[0]*_right.values[0] + 
		   _left.values[1]*_right.values[1] + 
		   _left.values[2]*_right.values[2] +
		   _left.values[3]*_right.values[3];
}

Vec4f Vec4f::Cross3(const Vec4f &_left, const Vec4f &_right)
{
	return Vec4f(
		_left.Y() * _right.Z() - _left.Z() * _right.Y(),
		_left.Z() * _right.X() - _left.X() * _right.Z(),
		_left.X() * _right.Y() - _left.Y() * _right.X()
		);
}

Vec4f Vec4f::ComponentMax(const Vec4f& _v1, const Vec4f& _v2)
{
	return Vec4f(std::max(_v1.X(), _v2.X()), std::max(_v1.Y(), _v2.Y()), std::max(_v1.Z(), _v2.Z()), std::max(_v1.W(), _v2.W()));
}

Vec4f Vec4f::ComponentMin(const Vec4f& _v1, const Vec4f& _v2)
{
	return Vec4f(std::min(_v1.X(), _v2.X()), std::min(_v1.Y(), _v2.Y()), std::min(_v1.Z(), _v2.Z()), std::min(_v1.W(), _v2.W()));
}

Vec4f Vec4f::Interpolate(const Vec4f& _v1, const Vec4f& _v2, float _a1, float _a2)
{
	float fac = _a1 / (_a1 + _a2);
	Vec4f diff = _v2 - _v1;

	return Vec4f(_v1.X() + fac * diff.X(), _v1.Y() + fac * diff.Y(), _v1.Z() + fac * diff.Z(), _v1.W() + fac * diff.W());
}

Vec4f Vec4f::operator + (const Vec4f &_rhs) const
{
	return Vec4f(values[0]+_rhs.values[0], values[1]+_rhs.values[1], values[2]+_rhs.values[2], values[3]+_rhs.values[3]);
}

Vec3f Vec4f::operator + (const Vec3f &_rhs) const
{
	return Vec3f(values[0]+_rhs.GetGlVector()[0], values[1]+_rhs.GetGlVector()[1], values[2]+_rhs.GetGlVector()[2]);
}
			
Vec4f Vec4f::operator - (const Vec4f &_rhs) const
{
	return Vec4f(values[0]-_rhs.values[0], values[1]-_rhs.values[1], values[2]-_rhs.values[2], values[3]-_rhs.values[3]);
}

Vec3f Vec4f::operator - (const Vec3f &_rhs) const
{
	return Vec3f(values[0]-_rhs.GetGlVector()[0], values[1]-_rhs.GetGlVector()[1], values[2]-_rhs.GetGlVector()[2]);
}

Vec4f Vec4f::operator * (float _scalar) const		
{
	return Vec4f(values[0]*_scalar, values[1]*_scalar, values[2]*_scalar, values[3]*_scalar);
}

Vec4f Vec4f::operator / (float _scalar) const		
{
	float rezi = 1.0f / _scalar;
	return Vec4f(values[0]*rezi, values[1]*rezi, values[2]*rezi, values[3]*rezi);
}

Vec4f& Vec4f::operator += (const Vec3f &_rhs)
{
	values[0]+=_rhs.GetGlVector()[0];
	values[1]+=_rhs.GetGlVector()[1];
	values[2]+=_rhs.GetGlVector()[2];
	return *this;
}

Vec4f& Vec4f::operator += (const Vec4f &_rhs)
{
	values[0]+=_rhs.values[0];
	values[1]+=_rhs.values[1];
	values[2]+=_rhs.values[2];
	values[3]+=_rhs.values[3];
	return *this;
}

Vec4f& Vec4f::operator -= (const Vec3f &_rhs)
{
	values[0]-=_rhs.GetGlVector()[0];
	values[1]-=_rhs.GetGlVector()[1];
	values[2]-=_rhs.GetGlVector()[2];
	return *this;
}

Vec4f& Vec4f::operator -= (const Vec4f &_rhs)
{
	values[0]-=_rhs.GetGlVector()[0];
	values[1]-=_rhs.GetGlVector()[1];
	values[2]-=_rhs.GetGlVector()[2];
	values[3]-=_rhs.GetGlVector()[3];
	return *this;
}

Vec4f& Vec4f::operator *= (float _scalar)
{
	values[0]*=_scalar;
	values[1]*=_scalar;
	values[2]*=_scalar;
	values[3]*=_scalar;
	return *this;
}

Vec4f& Vec4f::operator /= (float _scalar)
{
	float rezi = 1.0f / _scalar;
	values[0]*=rezi;
	values[1]*=rezi;
	values[2]*=rezi;
	values[3]*=rezi;
	return *this;
}

bool Vec4f::operator == (const Vec4f &_rhs) const
{
	if(values[0]!=_rhs.values[0]) return false;
	if(values[1]!=_rhs.values[1]) return false;
	if(values[2]!=_rhs.values[2]) return false;
	if(values[3]!=_rhs.values[3]) return false;
	return true;
}

bool Vec4f::operator != (const Vec4f &_rhs) const
{
	if(values[0]!=_rhs.values[0]) return true;
	if(values[1]!=_rhs.values[1]) return true;
	if(values[2]!=_rhs.values[2]) return true;
	if(values[3]!=_rhs.values[3]) return true;
	return false;
}

std::ostream& operator <<(std::ostream &os, const Vec4f &obj)
{
	os << "[" << obj.X() << ", " << obj.Y() << ", " << obj.Z() << ", " << obj.W() << "]";
	return os;
}