#define DLL_EXPORT

#include "Mat3f.hpp"
#include <math.h>
#include "../MathMacros.hpp"
#include "../../Core/Hardware/SSEInstructions.hpp"

using namespace Engine::Math::SSE;

Mat3f::Mat3f()
{
	values[0] = 1.0f;
	values[1] = 0.0f;
	values[2] = 0.0f;

	values[3] = 0.0f;
	values[4] = 1.0f;
	values[5] = 0.0f;
	
	values[6] = 0.0f;
	values[7] = 0.0f;
	values[8] = 1.0f;
}
Mat3f::Mat3f(float _m11, float _m12, float _m13, float _m21, float _m22, float _m23, float _m31, float _m32, float _m33)
{
	values[0] = _m11;
	values[1] = _m12;
	values[2] = _m13;

	values[3] = _m21;
	values[4] = _m22;
	values[5] = _m23;

	values[6] = _m31;
	values[7] = _m32;
	values[8] = _m33;
}
Mat3f::Mat3f(float _diag)
{
	values[0] = _diag;
	values[1] = 0.0f;
	values[2] = 0.0f;

	values[3] = 0.0f;
	values[4] = _diag;
	values[5] = 0.0f;
	
	values[6] = 0.0f;
	values[7] = 0.0f;
	values[8] = _diag;
}
Mat3f::Mat3f(const Mat3f& _refToMat)
{
	memcpy(values, _refToMat.values, 9 * sizeof(float));
}
Mat3f::Mat3f(const float* _values)
{
	memcpy(values, _values, 9 * sizeof(float));
}
Mat3f::~Mat3f()
{
}

inline float Mat3f::M11() const
{
	return values[0];
}
inline Mat3f& Mat3f::M11(float _value)
{
	values[0] = _value;
	return *this;
}
inline float Mat3f::M12() const
{
	return values[1];
}
inline Mat3f& Mat3f::M12(float _value)
{
	values[1] = _value;
	return *this;
}
inline float Mat3f::M13() const
{
	return values[2];
}
inline Mat3f& Mat3f::M13(float _value)
{
	values[2] = _value;
	return *this;
}

inline float Mat3f::M21() const
{
	return values[3];
}
inline Mat3f& Mat3f::M21(float _value)
{
	values[3] = _value;
	return *this;
}
inline float Mat3f::M22() const
{
	return values[4];
}
inline Mat3f& Mat3f::M22(float _value)
{
	values[4] = _value;
	return *this;
}
inline float Mat3f::M23() const
{
	return values[5];
}
inline Mat3f& Mat3f::M23(float _value)
{
	values[5] = _value;
	return *this;
}

inline float Mat3f::M31() const
{
	return values[6];
}
inline Mat3f& Mat3f::M31(float _value)
{
	values[6] = _value;
	return *this;
}
inline float Mat3f::M32() const
{
	return values[7];
}
inline Mat3f& Mat3f::M32(float _value)
{
	values[7] = _value;
	return *this;
}
inline float Mat3f::M33() const
{
	return values[8];
}
inline Mat3f& Mat3f::M33(float _value)
{
	values[8] = _value;
	return *this;
}

inline float Mat3f::M(int _column, int _row)
{
	return values[_row * 3 + _column];
}
inline void Mat3f::M(int _column, int _row, float _value)
{
	values[_row * 3 + _column] = _value;
}

inline float* Mat3f::GetGlMatrix() const
{
	return (float*)(&values[0]);
}

Mat3f& Mat3f::Transpose()
{
	float v = values[1];
	values[1] = values[3];
	values[3] = v;

	v = values[2];
	values[2] = values[6];
	values[6] = v;

	v = values[5];
	values[5] = values[7];
	values[7] = v;

	return *this;
	/*__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 s1 = _mm_shuffle_ps(r1, r2, _MM_SHUFFLE(1,0,1,0));
	__m128 s2 = _mm_shuffle_ps(r3, _mm_setzero_ps(), _MM_SHUFFLE(1,0,1,0));
	__m128 s3 = _mm_shuffle_ps(r1, r2, _MM_SHUFFLE(3,2,3,2));
	__m128 s4 = _mm_shuffle_ps(r3, _mm_setzero_ps(), _MM_SHUFFLE(3,2,3,2));
	
	r1 = _mm_shuffle_ps(s1, s2, _MM_SHUFFLE(2,0,2,0));
	r2 = _mm_shuffle_ps(s1, s2, _MM_SHUFFLE(3,1,3,1));
	r3 = _mm_shuffle_ps(s3, s4, _MM_SHUFFLE(2,0,2,0));

	Vec3f::SSEStore(r1, values);
	Vec3f::SSEStore(r2, &values[3]);
	Vec3f::SSEStore(r3, &values[6]);

	return *this;*/
}
Mat3f Mat3f::Transpose(const Mat3f& _matrix)
{
	Mat3f temp(_matrix.M11(), _matrix.M21(), _matrix.M31(), _matrix.M12(), _matrix.M22(), _matrix.M32(), _matrix.M13(), _matrix.M23(), _matrix.M33());

	return temp;
	/*Mat3f temp;

	__m128 r1 = Vec3f::SSELoad(_matrix.values);
	__m128 r2 = Vec3f::SSELoad(&_matrix.values[3]);
	__m128 r3 = Vec3f::SSELoad(&_matrix.values[6]);

	__m128 s1 = _mm_shuffle_ps(r1, r2, _MM_SHUFFLE(1,0,1,0));
	__m128 s2 = _mm_shuffle_ps(r3, _mm_setzero_ps(), _MM_SHUFFLE(1,0,1,0));
	__m128 s3 = _mm_shuffle_ps(r1, r2, _MM_SHUFFLE(3,2,3,2));
	__m128 s4 = _mm_shuffle_ps(r3, _mm_setzero_ps(), _MM_SHUFFLE(3,2,3,2));
	
	r1 = _mm_shuffle_ps(s1, s2, _MM_SHUFFLE(2,0,2,0));
	r2 = _mm_shuffle_ps(s1, s2, _MM_SHUFFLE(3,1,3,1));
	r3 = _mm_shuffle_ps(s3, s4, _MM_SHUFFLE(2,0,2,0));

	Vec3f::SSEStore(r1, temp.values);
	Vec3f::SSEStore(r2, &temp.values[3]);
	Vec3f::SSEStore(r3, &temp.values[6]);

	return temp;*/
}

Mat3f Mat3f::RotateX(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat3f(1.0f, 0.0f, 0.0f, 0.0f, cosa, -sina, 0.0f, sina, cosa);
}
Mat3f& Mat3f::RotateX(Mat3f& _matrix, float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);

	float mz;
	mz = _matrix.M12();
	_matrix.M12(_matrix.M12() * cosa + _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa - mz * sina);
	mz = _matrix.M22();
	_matrix.M22(_matrix.M22() * cosa + _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa - mz * sina);
	mz = _matrix.M32();
	_matrix.M32(_matrix.M32() * cosa + _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa - mz * sina);
	return _matrix;
}
Mat3f Mat3f::RotateY(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat3f(cosa, 0.0f, sina, 0.0f, 1.0f, 0.0f, -sina, 0.0f, cosa);
}
Mat3f& Mat3f::RotateY(Mat3f& _matrix, float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);

	float mz = _matrix.M11();
	_matrix.M11(_matrix.M11() * cosa - _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa + mz * sina);
	mz = _matrix.M21();
	_matrix.M21(_matrix.M21() * cosa - _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa + mz * sina);
	mz = _matrix.M31();
	_matrix.M31(_matrix.M31() * cosa - _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa + mz * sina);

	return _matrix;
}
Mat3f Mat3f::RotateZ(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat3f(cosa, -sina, 0.0f, sina, cosa, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat3f& Mat3f::RotateZ(Mat3f& _matrix, float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);

	float mz = _matrix.M11();
	_matrix.M11(_matrix.M11() * cosa + _matrix.M12() * sina);
	_matrix.M12(_matrix.M12() * cosa - mz * sina);
	mz = _matrix.M21();
	_matrix.M21(_matrix.M21() * cosa + _matrix.M22() * sina);
	_matrix.M22(_matrix.M22() * cosa - mz * sina);
	mz = _matrix.M31();
	_matrix.M31(_matrix.M31() * cosa + _matrix.M32() * sina);
	_matrix.M32(_matrix.M32() * cosa - mz * sina);

	return _matrix;
}
Mat3f& Mat3f::RotateAxis(Mat3f& _matrix, const Vec3f& _axis, float _angle)
{
	_matrix *= Mat3f::RotateAxis(_axis, _angle);
	return _matrix;
}
Mat3f Mat3f::RotateAxis(const Vec3f& _axis, float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	float icos = 1.0f - cosa;
	return Mat3f(cosa + _axis.X() * _axis.X() * icos,
		_axis.X() * _axis.Y() * icos - _axis.Z() * sina,
		_axis.X() * _axis.Z() * icos + _axis.Y() * sina,
		_axis.X() * _axis.Y() * icos + _axis.Z() * sina,
		cosa + _axis.Y() * _axis.Y() * icos,
		_axis.Y() * _axis.Z() * icos - _axis.X() * sina,
		_axis.X() * _axis.Z() * icos - _axis.Y() * sina,
		_axis.Y() * _axis.Z() * icos + _axis.X() * sina,
		cosa + _axis.Z() * _axis.Z() * icos);
}

Mat3f& Mat3f::Invert()
{
	Mat3f tmp = Adj(*this) / Det(*this);
	memcpy(values, tmp.values, 9 * sizeof(float));
	return *this;
}
Mat3f Mat3f::Invert(const Mat3f& _matrix)
{
	return Adj(_matrix) / Det(_matrix);
}

float Mat3f::Det() const
{
	return (values[0]*values[4]*values[8]+values[1]*values[5]*values[6]+values[2]*values[3]*values[7]-
			values[2]*values[4]*values[6]-values[1]*values[3]*values[8]-values[0]*values[5]*values[7]);
}
float Mat3f::Det(const Mat3f& _matrix)
{
	return _matrix.Det();
}

inline Mat3f Mat3f::Adj() const
{
	return Mat3f(Det2x2(values[4], values[5], values[7], values[8]), NegDet2x2(values[1], values[2], values[7], values[8]), Det2x2(values[1], values[2], values[4], values[5]),
				 NegDet2x2(values[3], values[5], values[6], values[8]), Det2x2(values[0], values[2], values[6], values[8]), NegDet2x2(values[0], values[2], values[3], values[5]),
				 Det2x2(values[3], values[4], values[6], values[7]), NegDet2x2(values[0], values[1], values[6], values[7]), Det2x2(values[0], values[1], values[3], values[4]));
	/*Mat3f tmpMatrix;

	__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 s1 = _mm_shuffle_ps(r1, r1, _MM_SHUFFLE(0, 0, 0, 1));
	__m128 s2 = _mm_shuffle_ps(r1, r1, _MM_SHUFFLE(0, 1, 2, 2));
	__m128 s3 = _mm_shuffle_ps(r2, r2, _MM_SHUFFLE(0, 0, 0, 1));
	__m128 s4 = _mm_shuffle_ps(r2, r2, _MM_SHUFFLE(0, 1, 2, 2));
	__m128 s5 = _mm_shuffle_ps(r3, r3, _MM_SHUFFLE(0, 0, 0, 1));
	__m128 s6 = _mm_shuffle_ps(r3, r3, _MM_SHUFFLE(0, 1, 2, 2));

	_declspec(align(16)) float n1[4] = {1.0f, -1.0f, 1.0f, -1.0f };
	_declspec(align(16)) float n2[4] = {-1.0f, 1.0f, -1.0f, 1.0f };

	__m128 neg1 = _mm_load_ps(n1);
	__m128 neg2 = _mm_load_ps(n2);

	__m128 mul1 = _mm_mul_ps(s3, s6);
	__m128 mul2 = _mm_mul_ps(s5, s4);
	mul2 = _mm_sub_ps(mul1, mul2);
	mul2 = _mm_mul_ps(neg1, mul2);
	Vec3f::SSEStore(mul2, tmpMatrix.values);

	mul1 = _mm_mul_ps(s1, s6);
	mul2 = _mm_mul_ps(s5, s2);
	mul2 = _mm_sub_ps(mul1, mul2);
	mul2 = _mm_mul_ps(neg2, mul2);
	Vec3f::SSEStore(mul2, &tmpMatrix.values[3]);

	mul1 = _mm_mul_ps(s1, s4);
	mul2 = _mm_mul_ps(s3, s2);
	mul2 = _mm_sub_ps(mul1, mul2);
	mul2 = _mm_mul_ps(neg1, mul2);
	Vec3f::SSEStore(mul2, &tmpMatrix.values[6]);

	tmpMatrix.Transpose();

	return tmpMatrix;*/
}
inline Mat3f Mat3f::Adj(const Mat3f& _matrix)
{
	return _matrix.Adj();
}

Mat3f Mat3f::operator * (const Mat3f& _toMultiply) const
{
	Mat3f tmp;
	
	tmp.M11(M11() * _toMultiply.M11() + M12() * _toMultiply.M21() + M13() * _toMultiply.M31());
	tmp.M12(M11() * _toMultiply.M12() + M12() * _toMultiply.M22() + M13() * _toMultiply.M32());
	tmp.M13(M11() * _toMultiply.M13() + M12() * _toMultiply.M23() + M13() * _toMultiply.M33());

	tmp.M21(M21() * _toMultiply.M11() + M22() * _toMultiply.M21() + M23() * _toMultiply.M31());
	tmp.M22(M21() * _toMultiply.M12() + M22() * _toMultiply.M22() + M23() * _toMultiply.M32());
	tmp.M23(M21() * _toMultiply.M13() + M22() * _toMultiply.M23() + M23() * _toMultiply.M33());

	tmp.M31(M31() * _toMultiply.M11() + M32() * _toMultiply.M21() + M33() * _toMultiply.M31());
	tmp.M32(M31() * _toMultiply.M12() + M32() * _toMultiply.M22() + M33() * _toMultiply.M32());
	tmp.M33(M31() * _toMultiply.M13() + M32() * _toMultiply.M23() + M33() * _toMultiply.M33());

	return tmp;

	/*Mat3f transpOther = Mat3f::Transpose(*this);

	__m128 a1 = Vec3f::SSELoad(transpOther.values);
	__m128 a2 = Vec3f::SSELoad(&transpOther.values[3]);
	__m128 a3 = Vec3f::SSELoad(&transpOther.values[6]);

	__m128 b1 = Vec3f::SSELoad(_toMultiply.values);
	__m128 b2 = Vec3f::SSELoad(&_toMultiply.values[3]);
	__m128 b3 = Vec3f::SSELoad(&_toMultiply.values[6]);

	Mat3f tmpMatrix;
	__m128 line = _mm_dp_ps(a1, b1, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b1, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b1, B8(01110100)));
	Vec3f::SSEStore(line, tmpMatrix.values);

	line = _mm_dp_ps(a1, b2, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b2, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b2, B8(01110100)));
	Vec3f::SSEStore(line, &tmpMatrix.values[3]);

	line = _mm_dp_ps(a1, b3, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b3, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b3, B8(01110100)));
	Vec3f::SSEStore(line, &tmpMatrix.values[6]);

	return tmpMatrix;*/
}
Vec3f Mat3f::operator * (const Vec3f& _toMultiply) const
{
	Vec3f tmp;
	
	tmp.X(_toMultiply.X() * M11() + _toMultiply.Y() * M12() + _toMultiply.Z() * M13());
	tmp.Y(_toMultiply.X() * M21() + _toMultiply.Y() * M22() + _toMultiply.Z() * M23());
	tmp.Z(_toMultiply.X() * M31() + _toMultiply.Y() * M32() + _toMultiply.Z() * M33());

	return tmp;

	/*Mat3f transpThis = Mat3f::Transpose(*this);

	__m128 a1 = Vec3f::SSELoad(transpThis.values);
	__m128 a2 = Vec3f::SSELoad(&transpThis.values[3]);
	__m128 a3 = Vec3f::SSELoad(&transpThis.values[6]);

	__m128 v = Vec3f::SSELoad(_toMultiply.GetGlVector());

	Vec3f tmpVec;
	__m128 line = _mm_dp_ps(a1, v, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, v, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, v, B8(01110100)));
	Vec3f::SSEStore(line, tmpVec.GetGlVector());

	return tmpVec;*/
}
Mat3f Mat3f::operator * (const float _scalar) const
{
	Mat3f m(values);
	m *= _scalar;
	return m;

	/*__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 scalar = _mm_load1_ps(&_scalar);

	r1 = _mm_mul_ps(r1, scalar);
	r2 = _mm_mul_ps(r2, scalar);
	r3 = _mm_mul_ps(r3, scalar);

	Mat3f tmpMatrix;

	Vec3f::SSEStore(r1, (float*)tmpMatrix.values);
	Vec3f::SSEStore(r2, (float*)&tmpMatrix.values[3]);
	Vec3f::SSEStore(r3, (float*)&tmpMatrix.values[6]);

	return tmpMatrix;*/
}
Mat3f Mat3f::operator / (const float _scalar) const
{
	float rezi = 1.0f / _scalar;
	return (*this) * rezi;

	/*__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 scalar = _mm_load1_ps(&_scalar);

	r1 = _mm_div_ps(r1, scalar);
	r2 = _mm_div_ps(r2, scalar);
	r3 = _mm_div_ps(r3, scalar);

	Mat3f tmpMatrix;

	Vec3f::SSEStore(r1, (float*)tmpMatrix.values);
	Vec3f::SSEStore(r2, (float*)&tmpMatrix.values[3]);
	Vec3f::SSEStore(r3, (float*)&tmpMatrix.values[6]);

	return tmpMatrix;*/
}
Mat3f& Mat3f::operator *= (const Mat3f& _toMultiply)
{
	Mat3f z = *this;

	M11(z.M11() * _toMultiply.M11() + z.M12() * _toMultiply.M21() + z.M13() * _toMultiply.M31());
	M12(z.M11() * _toMultiply.M12() + z.M12() * _toMultiply.M22() + z.M13() * _toMultiply.M32());
	M13(z.M11() * _toMultiply.M13() + z.M12() * _toMultiply.M23() + z.M13() * _toMultiply.M33());

	M21(z.M21() * _toMultiply.M11() + z.M22() * _toMultiply.M21() + z.M23() * _toMultiply.M31());
	M22(z.M21() * _toMultiply.M12() + z.M22() * _toMultiply.M22() + z.M23() * _toMultiply.M32());
	M23(z.M21() * _toMultiply.M13() + z.M22() * _toMultiply.M23() + z.M23() * _toMultiply.M33());

	M31(z.M31() * _toMultiply.M11() + z.M32() * _toMultiply.M21() + z.M33() * _toMultiply.M31());
	M32(z.M31() * _toMultiply.M12() + z.M32() * _toMultiply.M22() + z.M33() * _toMultiply.M32());
	M33(z.M31() * _toMultiply.M13() + z.M32() * _toMultiply.M23() + z.M33() * _toMultiply.M33());

	return *this;

	/*Mat3f transpOther = Mat3f::Transpose(*this);

	__m128 a1 = Vec3f::SSELoad(transpOther.values);
	__m128 a2 = Vec3f::SSELoad(&transpOther.values[3]);
	__m128 a3 = Vec3f::SSELoad(&transpOther.values[6]);

	__m128 b1 = Vec3f::SSELoad(_toMultiply.values);
	__m128 b2 = Vec3f::SSELoad(&_toMultiply.values[3]);
	__m128 b3 = Vec3f::SSELoad(&_toMultiply.values[6]);

	Mat3f tmpMatrix;
	__m128 line = _mm_dp_ps(a1, b1, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b1, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b1, B8(01110100)));
	Vec3f::SSEStore(line, values);

	line = _mm_dp_ps(a1, b2, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b2, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b2, B8(01110100)));
	Vec3f::SSEStore(line, &values[3]);

	line = _mm_dp_ps(a1, b3, B8(01110001));
	line = _mm_add_ps(line, _mm_dp_ps(a2, b3, B8(01110010)));
	line = _mm_add_ps(line, _mm_dp_ps(a3, b3, B8(01110100)));
	Vec3f::SSEStore(line, &values[6]);
	return *this;*/
}
Mat3f& Mat3f::operator *= (const float _scalar)
{
	for (int i = 0; i < 9; i++)
		values[i] *= _scalar;

	return *this;
	/*__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 scalar = _mm_load1_ps(&_scalar);

	r1 = _mm_mul_ps(r1, scalar);
	r2 = _mm_mul_ps(r2, scalar);
	r3 = _mm_mul_ps(r3, scalar);

	Mat3f tmpMatrix;

	Vec3f::SSEStore(r1, (float*)values);
	Vec3f::SSEStore(r2, (float*)&values[3]);
	Vec3f::SSEStore(r3, (float*)&values[6]);

	return *this;*/
}
Mat3f& Mat3f::operator /= (const float _scalar)
{
	float rezi = 1.0f / _scalar;
	*this *= rezi;
	return *this;

	/*__m128 r1 = Vec3f::SSELoad(this->values);
	__m128 r2 = Vec3f::SSELoad(&this->values[3]);
	__m128 r3 = Vec3f::SSELoad(&this->values[6]);

	__m128 scalar = _mm_load1_ps(&_scalar);

	r1 = _mm_div_ps(r1, scalar);
	r2 = _mm_div_ps(r2, scalar);
	r3 = _mm_div_ps(r3, scalar);

	Mat3f tmpMatrix;

	Vec3f::SSEStore(r1, (float*)values);
	Vec3f::SSEStore(r2, (float*)&values[3]);
	Vec3f::SSEStore(r3, (float*)&values[6]);
	return *this;*/
}
bool Mat3f::operator == (const Mat3f& _toCompare) const
{
	if (values[0] != _toCompare.values[0]) return false;
	if (values[1] != _toCompare.values[1]) return false;
	if (values[2] != _toCompare.values[2]) return false;
	if (values[3] != _toCompare.values[3]) return false;
	if (values[4] != _toCompare.values[4]) return false;
	if (values[5] != _toCompare.values[5]) return false;
	if (values[6] != _toCompare.values[6]) return false;
	if (values[7] != _toCompare.values[7]) return false;
	if (values[8] != _toCompare.values[8]) return false;

	return true;
}
bool Mat3f::operator!= (const Mat3f& _toCompare) const
{
	if (values[0] != _toCompare.values[0]) return true;
	if (values[1] != _toCompare.values[1]) return true;
	if (values[2] != _toCompare.values[2]) return true;
	if (values[3] != _toCompare.values[3]) return true;
	if (values[4] != _toCompare.values[4]) return true;
	if (values[5] != _toCompare.values[5]) return true;
	if (values[6] != _toCompare.values[6]) return true;
	if (values[7] != _toCompare.values[7]) return true;
	if (values[8] != _toCompare.values[8]) return true;

	return false;
}
Mat3f Mat3f::operator = (Mat3f& _copy)
{
	if (this != &_copy)
	{
		memcpy(values, _copy.values, 9 * sizeof(float));
	}
	return *this;
}

inline const bool Mat3f::IsColumnMajor()
{
	return false;
}

std::ostream& operator <<(std::ostream &os, const Mat3f &obj)
{
	os << "[" << obj.M11() << ", " << obj.M12() << ", " << obj.M13() << "; " << obj.M21() << ", " <<  obj.M22() << ", " <<  obj.M23() << "; " <<  obj.M31() << ", " <<  obj.M32() << ", " <<  obj.M33() << "]";
	return os;
}