#ifndef _ENGINE_MATH_SSE_MAT3F_HPP
#define _ENGINE_MATH_SSE_MAT3F_HPP

#include "../../dllconfig.hpp"
#include "Vec3f.hpp"

#include <iostream>

namespace Engine
{
	namespace Math
	{
		namespace SSE
		{
			class Mat4f;

			/// <summary>
			/// Defines a 3x3 Matrix and proviedes functions to work with them
			/// </summary>
			class _DLL_DECLSPEC Mat3f
			{
				friend Mat4f;

			public:
				/// <summary>
				/// Initializes a new instance of the Mat3f class and initialized it as a identity matrix
				/// </summary>
				Mat3f();
				/// <summary>
				/// Initializes a new instance of the Mat3f class and initialized it as a identity matrix
				/// </summary>
				/// <param name="_m11">Value of the first column in the first row</param>
				/// <param name="_m12">Value of the second column in the first row</param>
				/// <param name="_m13">Value of the third column in the first row</param>
				/// <param name="_m21">Value of the first column in the second row</param>
				/// <param name="_m22">Value of the second column in the second row</param>
				/// <param name="_m23">Value of the third column in the second row</param>
				/// <param name="_m31">Value of the first column in the third row</param>
				/// <param name="_m32">Value of the second column in the third row</param>
				/// <param name="_m33">Value of the third column in the third row</param>
				Mat3f(float _m11, float _m12, float _m13, float _m21, float _m22, float _m23, float _m31, float _m32, float _m33);
				/// <summary>
				/// Initializes a new instance of the Mat3f class
				/// </summary>
				/// <param name="_diag">Value of the main diagonal</param>
				Mat3f(float _diag);
				/// <summary>
				/// Initializes a new instance of the Mat3f class
				/// </summary>
				/// <param name="_refToMat">Matrix to perfom a deep copy</param>
				Mat3f(const Mat3f& _refToMat);

				Mat3f(const float* _values);
				/// <summary>
				/// Releases the data from a Mat3 class
				/// </summary>
				~Mat3f();

				/// <summary>
				/// Sets or retrieves the value of the first column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M11() const;
				/// <summary>
				/// Sets or retrieves the value of the first column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M11(float _value);
				/// <summary>
				/// Sets or retrieves the value of the second column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M12() const;
				/// <summary>
				/// Sets or retrieves the value of the second column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M12(float _value);
				/// <summary>
				/// Sets or retrieves the value of the third column in the first row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M13() const;
				/// <summary>
				/// Sets or retrieves the value of the third column in the first row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M13(float _value);

				/// <summary>
				/// Sets or retrieves the value of the first column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M21() const;
				/// <summary>
				/// Sets or retrieves the value of the first column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M21(float _value);
				/// <summary>
				/// Sets or retrieves the value of the second column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M22() const;
				/// <summary>
				/// Sets or retrieves the value of the second column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M22(float _value);
				/// <summary>
				/// Sets or retrieves the value of the third column in the second row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M23() const;
				/// <summary>
				/// Sets or retrieves the value of the third column in the second row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M23(float _value);

				/// <summary>
				/// Sets or retrieves the value of the first column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M31() const;
				/// <summary>
				/// Sets or retrieves the value of the first column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M31(float _value);
				/// <summary>
				/// Sets or retrieves the value of the second column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M32() const;
				/// <summary>
				/// Sets or retrieves the value of the second column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M32(float _value);
				/// <summary>
				/// Sets or retrieves the value of the third column in the third row
				/// </summary>
				/// <returns>The value of the matrix element</returns>
				float M33() const;
				/// <summary>
				/// Sets or retrieves the value of the third column in the third row
				/// </summary>
				/// <param name="_value">New value for the matrix element</param>
				/// <returns>Reference to the Mat3f class</returns>
				Mat3f& M33(float _value);

				/// <summary>
				/// Returns a pointer to a OpenGL matrix
				/// </summary>
				/// <returns>Pointer to the matrix</returns>
				float* GetGlMatrix() const;

				/// <summary>
				/// Transposes a Mat3f object
				/// </summary>
				/// <returns>Reference to the transposed matrix</returns>
				Mat3f& Transpose();
				/// <summary>
				/// Transposes a Mat3f object
				/// </summary>
				/// <param name="_matrix">Mat3f to transpose</param>
				/// <returns>The transposed matrix</returns>
				static Mat3f Transpose(const Mat3f& _matrix);
				/// <summary>
				/// Rotates a matrix along the x axis by a give angle
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat3f& RotateX(Mat3f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix for a rotation along the x axis
				/// </summary>
				/// <param name="_angle">The angle to rotate in radian</param>
				/// <returns>A rotation matrix</returns>
				static Mat3f RotateX(float _angle);
				/// <summary>
				/// Rotates a matrix along the y axis by a give angle
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat3f& RotateY(Mat3f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix for a rotation along the y axis
				/// </summary>
				/// <param name="_angle">The angle to rotate in radian</param>
				/// <returns>A rotation matrix</returns>
				static Mat3f RotateY(float _angle);
				/// <summary>
				/// Rotates a matrix along the z axis by a give angle
				/// </summary>
				/// <param name="_matrix">The matrix to rotate</param>
				/// <param name="_angle">The angle in radian</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat3f& RotateZ(Mat3f& _matrix, float _angle);
				/// <summary>
				/// Creates a rotation matrix for a rotation along the z axis
				/// </summary>
				/// <param name="_angle">The angle to rotate in radian</param>
				/// <returns>A rotation matrix</returns>
				static Mat3f RotateZ(float _angle);
				/// <summary>
				/// Rotates a matrix along a given axis by a given value
				/// </summary>
				/// <param name="_matrix">Matrix to rotate</param>
				/// <param name="_axis">Axis to rotate along</param>
				/// <param name="_angle">The angle in radian</param>
				/// <returns>A reference to the rotated matrix</returns>
				static Mat3f& RotateAxis(Mat3f& _matrix, const Vec3f& _axis, float _angle);
				/// <summary>
				/// Creates a rotation matrix for rotation along a given angle
				/// </summary>
				/// <param name="_axis">Axis to rotate along</param>
				/// <param name="_angle">The angle in radian</param>
				/// <returns>A rotation matrix</returns>
				static Mat3f RotateAxis(const Vec3f& _axis, float _angle);

				Mat3f& Invert();
				static Mat3f Invert(const Mat3f& _matrix);
				float Det() const;
				static float Det(const Mat3f& _matrix);
				Mat3f Adj() const;
				static Mat3f Adj(const Mat3f& _matrix);

				/// <summary>
				/// Defines the multiplication of two matrices
				/// </summary>
				/// <param name="_toMultiply>The matrix to multiply with</param>
				/// <returns>The rotated matrix</returns>
				Mat3f operator * (const Mat3f& _toMultiply) const;
				/// <summary>
				/// Defines the multiplication of a vector and a matrix
				/// </summary>
				/// <param name="_toMultiply>The vector</param>
				/// <returns>The multiplied vector</param>
				Vec3f operator * (const Vec3f& _toMultiply) const;

				Mat3f operator * (const float _scalar) const;

				Mat3f operator / (const float _scalar) const;

				/// <summary>
				/// Defines the multiplication of two matrices
				/// </summary>
				/// <param name="_toMultiply">To matrix to multiply with</param>
				/// <returns>A reference to the multiplied matrix</returns>
				Mat3f& operator *= (const Mat3f& _toMultiply);

				Mat3f& operator *= (const float _scalar);

				Mat3f& operator /= (const float _scalar);

				/// <summary>
				/// Defines the equal operator for two matrices
				/// </summary>
				/// <param name="_toCompare">The matrix to compare</param>
				/// <returns>True if the two matrices are equal</returns>
				bool operator == (const Mat3f& _toCompare) const;
				/// <summary>
				/// Defines the unequal operator for two matrices
				/// </summary>
				/// <param name="_toCompare">The matrix to compare</param>
				/// <returns>True if the two matrices are unequal</returns>
				bool operator != (const Mat3f& _toCompare) const;
				/// <summary>
				/// Defines the assignment operator to get a deep copy
				/// </summary>
				/// <param name="_copy">Source to copy from</param>
				/// <returns>Copied Mat3 object</returns>
				Mat3f operator = (Mat3f& _copy);

				/// <summary>
				/// Defines if the matrix is stored column major or row major
				/// </summary>
				/// <returns>True if the matrix is column major, false if not</returns>
				static const bool IsColumnMajor();

			private:
				float M(int _column, int _row);
				void M(int _column, int _row, float _value);

				float values[9];
			};
		}
	}
}

_DLL_DECLSPEC std::ostream& operator << (std::ostream &os, const Engine::Math::SSE::Mat3f &obj);

#endif