#define DLL_EXPORT

#include "Mat4f.hpp"
#include "Mat3f.hpp"
#include <math.h>
#include "../MathMacros.hpp"
#include "../../Core/Hardware/SSEInstructions.hpp"

using namespace Engine::Math::SSE;

Mat4f::Mat4f()
{
	for (int i = 0; i < 16; i++)
		values[i] = 0.0f;

	values[0] = 1.0f;
	values[5] = 1.0f;
	values[10] = 1.0f;
	values[15] = 1.0f;
}

Mat4f::Mat4f(float _diag)
{
	for (int i = 0; i < 16; i++)
		values[i] = 0.0f;

	values[0] = _diag;
	values[5] = _diag;
	values[10] = _diag;
	values[15] = _diag;
}

Mat4f::Mat4f(float _m11, float _m12, float _m13, float _m14, float _m21, float _m22, float _m23, float _m24, float _m31, float _m32, float _m33, float _m34, float _m41, float _m42, float _m43, float _m44)
{
	values[0] = _m11;
	values[1] = _m12;
	values[2] = _m13;
	values[3] = _m14;
	values[4] = _m21;
	values[5] = _m22;
	values[6] = _m23;
	values[7] = _m24;
	values[8] = _m31;
	values[9] = _m32;
	values[10] = _m33;
	values[11] = _m34;
	values[12] = _m41;
	values[13] = _m42;	
	values[14] = _m43;
	values[15] = _m44;
}

Mat4f::Mat4f(const float* _values)
{
	memcpy(values, _values, 16*sizeof(float));
}

Mat4f::Mat4f(const Mat4f& _toCopy)
{
	memcpy(values, _toCopy.values, 16*sizeof(float));
}

Mat4f::Mat4f(Mat3f& _toCopy)
{
	values[0] = _toCopy.GetGlMatrix()[0];
	values[1] = _toCopy.GetGlMatrix()[1];
	values[2] = _toCopy.GetGlMatrix()[2];
	values[3] = 0.0f;

	values[4] = _toCopy.GetGlMatrix()[3];
	values[5] = _toCopy.GetGlMatrix()[4];
	values[6] = _toCopy.GetGlMatrix()[5];
	values[7] = 0.0f;

	values[8] = _toCopy.GetGlMatrix()[6];
	values[9] = _toCopy.GetGlMatrix()[7];
	values[10] = _toCopy.GetGlMatrix()[8];
	values[11] = 0.0f;

	values[12] = 0.0f;
	values[13] = 0.0f;
	values[14] = 0.0f;
	values[15] = 1.0f;
}

Mat4f::Mat4f(__m128 _row0, __m128 _row1, __m128 _row2, __m128 _row3)
{
	_mm_store_ps(values, _row0);
	_mm_store_ps(values+4, _row1);
	_mm_store_ps(values+8, _row2);
	_mm_store_ps(values+12, _row3);
}

Mat4f::~Mat4f() { }

inline float Mat4f::M11() const
{
	return values[0];
}
inline Mat4f& Mat4f::M11(float _value)
{
	values[0] = _value;
	return *this;
}
inline float Mat4f::M12() const
{
	return values[1];
}
inline Mat4f& Mat4f::M12(float _value)
{
	values[1] = _value;
	return *this;
}
inline float Mat4f::M13() const
{
	return values[2];
}
inline Mat4f& Mat4f::M13(float _value)
{
	values[2] = _value;
	return *this;
}
inline float Mat4f::M14() const
{
	return values[3];
}
inline Mat4f& Mat4f::M14(float _value)
{
	values[3] = _value;
	return *this;
}

inline float Mat4f::M21() const
{
	return values[4];
}
inline Mat4f& Mat4f::M21(float _value)
{
	values[4] = _value;
	return *this;
}
inline float Mat4f::M22() const
{
	return values[5];
}
inline Mat4f& Mat4f::M22(float _value)
{
	values[5] = _value;
	return *this;
}
inline float Mat4f::M23() const
{
	return values[6];
}
inline Mat4f& Mat4f::M23(float _value)
{
	values[6] = _value;
	return *this;
}
inline float Mat4f::M24() const
{
	return values[7];
}
inline Mat4f& Mat4f::M24(float _value)
{
	values[7] = _value;
	return *this;
}

inline float Mat4f::M31() const
{
	return values[8];
}
inline Mat4f& Mat4f::M31(float _value)
{
	values[8] = _value;
	return *this;
}
inline float Mat4f::M32() const
{
	return values[9];
}
inline Mat4f& Mat4f::M32(float _value)
{
	values[9] = _value;
	return *this;
}
inline float Mat4f::M33() const
{
	return values[10];
}
inline Mat4f& Mat4f::M33(float _value)
{
	values[10] = _value;
	return *this;
}
inline float Mat4f::M34() const
{
	return values[11];
}
inline Mat4f& Mat4f::M34(float _value)
{
	values[11] = _value;
	return *this;
}

inline float Mat4f::M41() const
{
	return values[12];
}
inline Mat4f& Mat4f::M41(float _value)
{
	values[12] = _value;
	return *this;
}
inline float Mat4f::M42() const
{
	return values[13];
}
inline Mat4f& Mat4f::M42(float _value)
{
	values[13] = _value;
	return *this;
}
inline float Mat4f::M43() const
{
	return values[14];
}
inline Mat4f& Mat4f::M43(float _value)
{
	values[14] = _value;
	return *this;
}
inline float Mat4f::M44() const
{
	return values[15];
}
inline Mat4f& Mat4f::M44(float _value)
{
	values[15] = _value;
	return *this;
}

inline float* Mat4f::GetGlMatrix() const
{
	return (float*)(&values[0]);
}

inline const bool Mat4f::IsColumnMajor()
{
	return false;
}

Mat4f& Mat4f::Transpose()
{
	__m128 row0, row1, row2, row3;

	row0 = _mm_load_ps(values);
	row1 = _mm_load_ps(values+4);
	row2 = _mm_load_ps(values+8);
	row3 = _mm_load_ps(values+12);

	_MM_TRANSPOSE4_PS(row0, row1, row2, row3);

	_mm_store_ps(values, row0);
	_mm_store_ps(values+4, row1);
	_mm_store_ps(values+8, row2);
	_mm_store_ps(values+12, row3);

	/*float z = M12();
	M12(M21());
	M21(z);

	z = M13();
	M13(M31());
	M31(z);

	z = M14();
	M14(M41());
	M41(z);

	z = M23();
	M23(M32());
	M32(z);

	z = M24();
	M24(M42());
	M42(z);

	z = M34();
	M34(M43());
	M43(z);*/

	return *this;
}
Mat4f Mat4f::Transpose(const Mat4f& _matrix)
{
	__m128 row0, row1, row2, row3;

	row0 = _mm_load_ps(_matrix.values);
	row1 = _mm_load_ps(_matrix.values+4);
	row2 = _mm_load_ps(_matrix.values+8);
	row3 = _mm_load_ps(_matrix.values+12);

	_MM_TRANSPOSE4_PS(row0, row1, row2, row3);

	return Mat4f(row0, row1, row2, row3);
}

Mat4f& Mat4f::Invert()
{
	__m128 minor0, minor1, minor2, minor3;
	__m128 row0, row1, row2, row3;
	__m128 det, tmp1;

	//Transpose
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values)), (__m64*)(values+ 4));
	row1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+8)), (__m64*)(values+12));
	row0 = _mm_shuffle_ps(tmp1, row1, 0x88);
	row1 = _mm_shuffle_ps(row1, tmp1, 0xDD);
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+ 2)), (__m64*)(values+ 6));
	row3 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+10)), (__m64*)(values+14));
	row2 = _mm_shuffle_ps(tmp1, row3, 0x88);
	row3 = _mm_shuffle_ps(row3, tmp1, 0xDD);

	//Calculate adj
	tmp1 = _mm_mul_ps(row2, row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_mul_ps(row1, tmp1);
	minor1 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(_mm_mul_ps(row1, tmp1), minor0);
	minor1 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor1);
	minor1 = _mm_shuffle_ps(minor1, minor1, 0x4E);

	tmp1 = _mm_mul_ps(row1, row2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor0);
	minor3 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row3, tmp1));
	minor3 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor3);
	minor3 = _mm_shuffle_ps(minor3, minor3, 0x4E);
	
	tmp1 = _mm_mul_ps(_mm_shuffle_ps(row1, row1, 0x4E), row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	row2 = _mm_shuffle_ps(row2, row2, 0x4E);
	minor0 = _mm_add_ps(_mm_mul_ps(row2, tmp1), minor0);
	minor2 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row2, tmp1));
	minor2 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor2);
	minor2 = _mm_shuffle_ps(minor2, minor2, 0x4E);
	
	tmp1 = _mm_mul_ps(row0, row1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor2 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor2);
	minor3 = _mm_sub_ps(_mm_mul_ps(row2, tmp1), minor3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor2 = _mm_sub_ps(_mm_mul_ps(row3, tmp1), minor2);
	minor3 = _mm_sub_ps(minor3, _mm_mul_ps(row2, tmp1));
	
	tmp1 = _mm_mul_ps(row0, row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor1 = _mm_sub_ps(minor1, _mm_mul_ps(row2, tmp1));
	minor2 = _mm_add_ps(_mm_mul_ps(row1, tmp1), minor2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor1 = _mm_add_ps(_mm_mul_ps(row2, tmp1), minor1);
	minor2 = _mm_sub_ps(minor2, _mm_mul_ps(row1, tmp1));
	
	tmp1 = _mm_mul_ps(row0, row2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor1 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor1);
	minor3 = _mm_sub_ps(minor3, _mm_mul_ps(row1, tmp1));
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor1 = _mm_sub_ps(minor1, _mm_mul_ps(row3, tmp1));
	minor3 = _mm_add_ps(_mm_mul_ps(row1, tmp1), minor3);

	det = _mm_mul_ps(row0, minor0);
	det = _mm_add_ps(_mm_shuffle_ps(det, det, 0x4E), det);
	det = _mm_add_ss(_mm_shuffle_ps(det, det, 0xB1), det);
	det = _mm_shuffle_ps(det, det, 0x00);
	//tmp1 = _mm_rcp_ss(det);
	tmp1 = _mm_set1_ps(1.0f);
	det = _mm_div_ps(tmp1, det);
	//det = _mm_sub_ss(_mm_add_ss(tmp1, tmp1), _mm_mul_ss(det, _mm_mul_ss(tmp1, tmp1)));
	
	minor0 = _mm_mul_ps(det, minor0);
	_mm_store_ps(values, minor0);
	minor1 = _mm_mul_ps(det, minor1);
	_mm_store_ps(values+4, minor1);
	minor2 = _mm_mul_ps(det, minor2);
	_mm_store_ps(values+8, minor2);
	minor3 = _mm_mul_ps(det, minor3);
	_mm_store_ps(values+12, minor3);

	return *this;
}
Mat4f Mat4f::Invert(const Mat4f& _matrix)
{
	__m128 minor0, minor1, minor2, minor3;
	__m128 row0, row1, row2, row3;
	__m128 det, tmp1;

	//Transpose
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(_matrix.values)), (__m64*)(_matrix.values+ 4));
	row1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(_matrix.values+8)), (__m64*)(_matrix.values+12));
	row0 = _mm_shuffle_ps(tmp1, row1, 0x88);
	row1 = _mm_shuffle_ps(row1, tmp1, 0xDD);
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(_matrix.values+ 2)), (__m64*)(_matrix.values+ 6));
	row3 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(_matrix.values+10)), (__m64*)(_matrix.values+14));
	row2 = _mm_shuffle_ps(tmp1, row3, 0x88);
	row3 = _mm_shuffle_ps(row3, tmp1, 0xDD);

	//Calculate adj
	tmp1 = _mm_mul_ps(row2, row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_mul_ps(row1, tmp1);
	minor1 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(_mm_mul_ps(row1, tmp1), minor0);
	minor1 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor1);
	minor1 = _mm_shuffle_ps(minor1, minor1, 0x4E);

	tmp1 = _mm_mul_ps(row1, row2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor0);
	minor3 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row3, tmp1));
	minor3 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor3);
	minor3 = _mm_shuffle_ps(minor3, minor3, 0x4E);
	
	tmp1 = _mm_mul_ps(_mm_shuffle_ps(row1, row1, 0x4E), row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	row2 = _mm_shuffle_ps(row2, row2, 0x4E);
	minor0 = _mm_add_ps(_mm_mul_ps(row2, tmp1), minor0);
	minor2 = _mm_mul_ps(row0, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row2, tmp1));
	minor2 = _mm_sub_ps(_mm_mul_ps(row0, tmp1), minor2);
	minor2 = _mm_shuffle_ps(minor2, minor2, 0x4E);
	
	tmp1 = _mm_mul_ps(row0, row1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor2 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor2);
	minor3 = _mm_sub_ps(_mm_mul_ps(row2, tmp1), minor3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor2 = _mm_sub_ps(_mm_mul_ps(row3, tmp1), minor2);
	minor3 = _mm_sub_ps(minor3, _mm_mul_ps(row2, tmp1));
	
	tmp1 = _mm_mul_ps(row0, row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor1 = _mm_sub_ps(minor1, _mm_mul_ps(row2, tmp1));
	minor2 = _mm_add_ps(_mm_mul_ps(row1, tmp1), minor2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor1 = _mm_add_ps(_mm_mul_ps(row2, tmp1), minor1);
	minor2 = _mm_sub_ps(minor2, _mm_mul_ps(row1, tmp1));
	
	tmp1 = _mm_mul_ps(row0, row2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor1 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor1);
	minor3 = _mm_sub_ps(minor3, _mm_mul_ps(row1, tmp1));
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor1 = _mm_sub_ps(minor1, _mm_mul_ps(row3, tmp1));
	minor3 = _mm_add_ps(_mm_mul_ps(row1, tmp1), minor3);

	det = _mm_mul_ps(row0, minor0);
	det = _mm_add_ps(_mm_shuffle_ps(det, det, 0x4E), det);
	det = _mm_add_ss(_mm_shuffle_ps(det, det, 0xB1), det);
	det = _mm_shuffle_ps(det, det, 0x00);
	tmp1 = _mm_set1_ps(1.0f);
	det = _mm_div_ps(tmp1, det);
	
	minor0 = _mm_mul_ps(det, minor0);
	minor1 = _mm_mul_ps(det, minor1);
	minor2 = _mm_mul_ps(det, minor2);
	minor3 = _mm_mul_ps(det, minor3);

	return Mat4f(minor0, minor1, minor2, minor3);

	/*Mat4f tmp(_matrix);
	tmp.Invert();
	return tmp;*/
}
float Mat4f::Det() const
{
	__m128 minor0;
	__m128 row0, row1, row2, row3;
	__m128 det, tmp1;

	//Transpose
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values)), (__m64*)(values+ 4));
	row1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+8)), (__m64*)(values+12));
	row0 = _mm_shuffle_ps(tmp1, row1, 0x88);
	row1 = _mm_shuffle_ps(row1, tmp1, 0xDD);
	tmp1 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+ 2)), (__m64*)(values+ 6));
	row3 = _mm_loadh_pi(_mm_loadl_pi(_mm_setzero_ps(), (__m64*)(values+10)), (__m64*)(values+14));
	row2 = _mm_shuffle_ps(tmp1, row3, 0x88);
	row3 = _mm_shuffle_ps(row3, tmp1, 0xDD);

	//Calculate adj
	tmp1 = _mm_mul_ps(row2, row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_mul_ps(row1, tmp1);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(_mm_mul_ps(row1, tmp1), minor0);

	tmp1 = _mm_mul_ps(row1, row2);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	minor0 = _mm_add_ps(_mm_mul_ps(row3, tmp1), minor0);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row3, tmp1));
	
	tmp1 = _mm_mul_ps(_mm_shuffle_ps(row1, row1, 0x4E), row3);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0xB1);
	row2 = _mm_shuffle_ps(row2, row2, 0x4E);
	minor0 = _mm_add_ps(_mm_mul_ps(row2, tmp1), minor0);
	tmp1 = _mm_shuffle_ps(tmp1, tmp1, 0x4E);
	minor0 = _mm_sub_ps(minor0, _mm_mul_ps(row2, tmp1));

	det = _mm_mul_ps(row0, minor0);
	det = _mm_add_ps(_mm_shuffle_ps(det, det, 0x4E), det);
	det = _mm_add_ss(_mm_shuffle_ps(det, det, 0xB1), det);
	det = _mm_shuffle_ps(det, det, 0x00);

	float retVal;
	_mm_store_ss(&retVal, det);
	return retVal;
}
float Mat4f::Det(const Mat4f& _matrix)
{
	return _matrix.Det();
}

Mat4f Mat4f::RotateX(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat4f(1.0f, 0.0f, 0.0f, 0.0f, 0.0f, cosa, -sina, 0.0f, 0.0f, sina, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateX(Mat4f& _matrix, float _angle) {

	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M12();
	_matrix.M12(z1 * cosa + _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa - z1 * sina);

	z1 = _matrix.M22();
	_matrix.M22(z1 * cosa + _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa - z1 * sina);

	z1 = _matrix.M32();
	_matrix.M32(z1 * cosa + _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa - z1 * sina);

	z1 = _matrix.M42();
	_matrix.M42(z1 * cosa + _matrix.M43() * sina);
	_matrix.M43(_matrix.M43() * cosa - z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateY(float _angle)
{
	float sina = sin(_angle);
	float cosa = cos(_angle);
	return Mat4f(cosa, 0.0f, sina, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, -sina, 0.0f, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateY(Mat4f& _matrix, float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M11();
	_matrix.M11(z1 * cosa - _matrix.M13() * sina);
	_matrix.M13(_matrix.M13() * cosa + z1 * sina);

	z1 = _matrix.M21();
	_matrix.M21(z1 * cosa - _matrix.M23() * sina);
	_matrix.M23(_matrix.M23() * cosa + z1 * sina);

	z1 = _matrix.M31();
	_matrix.M31(z1 * cosa - _matrix.M33() * sina);
	_matrix.M33(_matrix.M33() * cosa + z1 * sina);

	z1 = _matrix.M41();
	_matrix.M41(z1 * cosa - _matrix.M43() * sina);
	_matrix.M43(_matrix.M43() * cosa + z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateZ(float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);

	return Mat4f(cosa, -sina, 0.0f, 0.0f, sina, cosa, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateZ(Mat4f& _matrix, float _angle)
{ 
	float cosa = cos(_angle);
	float sina = sin(_angle);

	float z1 = _matrix.M11();
	_matrix.M11(z1 * cosa + _matrix.M12() * sina);
	_matrix.M12(_matrix.M12() * cosa - z1 * sina);

	z1 = _matrix.M21();
	_matrix.M21(z1 * cosa + _matrix.M22() * sina);
	_matrix.M22(_matrix.M22() * cosa - z1 * sina);

	z1 = _matrix.M31();
	_matrix.M31(z1 * cosa + _matrix.M32() * sina);
	_matrix.M32(_matrix.M32() * cosa - z1 * sina);

	z1 = _matrix.M41();
	_matrix.M41(z1 * cosa + _matrix.M42() * sina);
	_matrix.M41(_matrix.M42() * cosa - z1 * sina);

	return _matrix;
}
Mat4f Mat4f::RotateAxis(const Vec3f& _axis, float _angle)
{
	float cosa = cos(_angle);
	float sina = sin(_angle);
	float icos = 1.0f - cosa;

	return Mat4f(cosa + _axis.X() * _axis.X() * icos,
		_axis.X() * _axis.Y() * icos - _axis.Z() * sina,
		_axis.X() * _axis.Z() * icos + _axis.Y() * sina,
		0.0f,
		_axis.X() * _axis.Y() * icos + _axis.Z() * sina,
		cosa + _axis.Y() * _axis.Y() * icos,
		_axis.Y() * _axis.Z() * icos - _axis.X() * sina,
		0.0f,
		_axis.X() * _axis.Z() * icos - _axis.Y() * sina,
		_axis.Y() * _axis.Z() * icos + _axis.X() * sina,
		cosa + _axis.Z() * _axis.Z() * icos,
		0.0f,
		0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::RotateAxis(Mat4f& _matrix, const Vec3f& _axis, float _angle)
{	
	_matrix *= Mat4f::RotateAxis(_axis, _angle);
	return _matrix;
}
Mat4f Mat4f::Translate(const Vec3f& _direction)
{
	return Mat4f(1.0f, 0.0f, 0.0f, _direction.X(), 0.0f, 1.0f, 0.0f, _direction.Y(), 0.0f, 0.0f, 1.0f, _direction.Z(), 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::Translate(Mat4f& _matrix, const Vec3f& _direction)
{
	_matrix.M14(_matrix.M14() + _matrix.M11() * _direction.X() + _matrix.M12() * _direction.Y() + _matrix.M13() * _direction.Z());
	_matrix.M24(_matrix.M24() + _matrix.M21() * _direction.X() + _matrix.M22() * _direction.Y() + _matrix.M23() * _direction.Z());
	_matrix.M34(_matrix.M34() + _matrix.M31() * _direction.X() + _matrix.M32() * _direction.Y() + _matrix.M33() * _direction.Z());
	_matrix.M44(_matrix.M44() + _matrix.M41() * _direction.X() + _matrix.M42() * _direction.Y() + _matrix.M43() * _direction.Z());

	return _matrix;
}
Mat4f Mat4f::Scale(const Vec3f& _scales)
{
	return Mat4f(_scales.X(), 0.0f, 0.0f, 0.0f, 0.0f, _scales.Y(), 0.0f, 0.0f, 0.0f, 0.0f, _scales.Z(), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
}
Mat4f& Mat4f::Scale(Mat4f& _matrix, const Vec3f& _scale)
{
	_matrix.M11(_matrix.M11() * _scale.X());
	_matrix.M21(_matrix.M21() * _scale.X());
	_matrix.M31(_matrix.M31() * _scale.X());
	_matrix.M41(_matrix.M41() * _scale.X());

	_matrix.M12(_matrix.M12() * _scale.Y());
	_matrix.M22(_matrix.M22() * _scale.Y());
	_matrix.M32(_matrix.M32() * _scale.Y());
	_matrix.M42(_matrix.M42() * _scale.Y());

	_matrix.M13(_matrix.M13() * _scale.Z());
	_matrix.M23(_matrix.M23() * _scale.Z());
	_matrix.M33(_matrix.M33() * _scale.Z());
	_matrix.M43(_matrix.M43() * _scale.Z());

	return _matrix;
}
Mat4f Mat4f::Persp(float _fov, float _aspectRation, float _nearPlane, float _farPlane)
{
	float top = tan(_fov * 0.5f) * _nearPlane;
	float bottom = -top;
	float left = _aspectRation * bottom;
	float right = _aspectRation * top;

	return Mat4f(
		(2.0f * _nearPlane) / (right - left), 0.0f, (right + left) / (right - left), 0.0f,
		0.0f, (2.0f * _nearPlane) / (top - bottom), (top + bottom) / (top - bottom), 0.0f,
		0.0f, 0.0f, - (_farPlane + _nearPlane) / (_farPlane - _nearPlane), -2.0f * _farPlane * _nearPlane / (_farPlane - _nearPlane),
		0.0f, 0.0f, -1.0f, 0.0);

	/*float h = 1.0f / tan(_fov * 0.5f);
	float negDepth = _nearPlane - _farPlane;

	return Mat4f(
		h / _aspectRation, 0.0f, 0.0f, 0.0f,
		0.0f, h, 0.0f, 0.0f,
		0.0f, 0.0f, _farPlane / negDepth, 2.0f * (_nearPlane * _farPlane) / negDepth,
		0.0f, 0.0f, -1.0f, 0.0f
		);*/
}
Mat4f Mat4f::Ortho(float _left, float _right, float _bottom, float _top, float _nearPlane, float _farPlane)
{
	return Mat4f(
		2.0f / (_right - _left), 0.0f, 0.0f, -(_right + _left) / (_right - _left),
		0.0f, 2.0f / (_top - _bottom), 0.0f,  -(_top + _bottom) / (_top - _bottom),
		0.0f, 0.0f, -2.0f / (_farPlane - _nearPlane), -(_farPlane + _nearPlane) / (_farPlane - _nearPlane),
		0.0f, 0.0f, 0.0f, 1.0f);
}

Mat4f Mat4f::GetRotation(const Mat4f& _matrix)
{
	Mat4f mat(_matrix);

	//Remove scaling
	__m128 row1 = _mm_load_ps(mat.values);
	__m128 row2 = _mm_load_ps(mat.values + 4);
	__m128 row3 = _mm_load_ps(mat.values + 8);

	__m128 row1s = _mm_mul_ps(row1, row1);
	__m128 row2s = _mm_mul_ps(row2, row2);
	__m128 row3s = _mm_mul_ps(row3, row3);

	row1s = _mm_add_ps(_mm_add_ps(row1s, row2s), row3s); //dot product over columns
	row2s = _mm_sqrt_ps(row1s); //Scalings

	row1 = _mm_div_ps(row1, row2s);
	row2 = _mm_div_ps(row2, row2s);
	row3 = _mm_div_ps(row3, row2s);

	_mm_store_ps(mat.values, row1);
	_mm_store_ps(mat.values + 4, row2);
	_mm_store_ps(mat.values + 8, row3);

	//Remote translation
	mat.M14(0.0f);
	mat.M24(0.0f);
	mat.M34(0.0f);

	return mat;
}

Mat4f Mat4f::RemoveScaling(const Mat4f& _matrix)
{
	Mat4f mat(_matrix);

	//Remove scaling
	__m128 row1 = _mm_load_ps(mat.values);
	__m128 row2 = _mm_load_ps(mat.values + 4);
	__m128 row3 = _mm_load_ps(mat.values + 8);

	__m128 row1s = _mm_mul_ps(row1, row1);
	__m128 row2s = _mm_mul_ps(row2, row2);
	__m128 row3s = _mm_mul_ps(row3, row3);

	row1s = _mm_add_ps(_mm_add_ps(row1s, row2s), row3s); //dot product over columns
	row2s = _mm_sqrt_ps(row1s); //Scalings

	row1 = _mm_div_ps(row1, row2s);
	row2 = _mm_div_ps(row2, row2s);
	row3 = _mm_div_ps(row3, row2s);

	_mm_store_ps(mat.values, row1);
	_mm_store_ps(mat.values + 4, row2);
	_mm_store_ps(mat.values + 8, row3);

	//Remote translation
	mat.M14(_matrix.M14());
	mat.M24(_matrix.M24());
	mat.M34(_matrix.M34());

	return mat;
}

Mat4f Mat4f::operator * (const Mat4f& _matrix) const
{
	__m128 arow0, arow1, arow2, arow3;
	__m128 brow0, brow1, brow2, brow3;
	__m128 crow0, crow1, crow2, crow3;

	arow0 = _mm_load_ps(values);
	arow1 = _mm_load_ps(values+4);
	arow2 = _mm_load_ps(values+8);
	arow3 = _mm_load_ps(values+12);

	brow0 = _mm_load_ps(_matrix.values);
	brow1 = _mm_load_ps(_matrix.values+4);
	brow2 = _mm_load_ps(_matrix.values+8);
	brow3 = _mm_load_ps(_matrix.values+12);

	_MM_TRANSPOSE4_PS(brow0, brow1, brow2, brow3);

	crow0 = _mm_dp_ps(arow0, brow0, B8(11110001));
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow1, B8(11110010)), crow0);
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow2, B8(11110100)), crow0);
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow3, B8(11111000)), crow0);

	crow1 = _mm_dp_ps(arow1, brow0, B8(11110001));
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow1, B8(11110010)), crow1);
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow2, B8(11110100)), crow1);
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow3, B8(11111000)), crow1);

	crow2 = _mm_dp_ps(arow2, brow0, B8(11110001));
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow1, B8(11110010)), crow2);
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow2, B8(11110100)), crow2);
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow3, B8(11111000)), crow2);

	crow3 = _mm_dp_ps(arow3, brow0, B8(11110001));
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow1, B8(11110010)), crow3);
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow2, B8(11110100)), crow3);
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow3, B8(11111000)), crow3);

	return Mat4f(crow0, crow1, crow2, crow3);

	/*Mat4f tmp(
		M11() * _matrix.M11() + M12() * _matrix.M21() + M13() * _matrix.M31() + M14() * _matrix.M41(),
		M11() * _matrix.M12() + M12() * _matrix.M22() + M13() * _matrix.M32() + M14() * _matrix.M42(),
		M11() * _matrix.M13() + M12() * _matrix.M23() + M13() * _matrix.M33() + M14() * _matrix.M43(),
		M11() * _matrix.M14() + M12() * _matrix.M24() + M13() * _matrix.M34() + M14() * _matrix.M44(),

		M21() * _matrix.M11() + M22() * _matrix.M21() + M23() * _matrix.M31() + M24() * _matrix.M41(),
		M21() * _matrix.M12() + M22() * _matrix.M22() + M23() * _matrix.M32() + M24() * _matrix.M42(),
		M21() * _matrix.M13() + M22() * _matrix.M23() + M23() * _matrix.M33() + M24() * _matrix.M43(),
		M21() * _matrix.M14() + M22() * _matrix.M24() + M23() * _matrix.M34() + M24() * _matrix.M44(),

		M31() * _matrix.M11() + M32() * _matrix.M21() + M33() * _matrix.M31() + M34() * _matrix.M41(),
		M31() * _matrix.M12() + M32() * _matrix.M22() + M33() * _matrix.M32() + M34() * _matrix.M42(),
		M31() * _matrix.M13() + M32() * _matrix.M23() + M33() * _matrix.M33() + M34() * _matrix.M43(),
		M31() * _matrix.M14() + M32() * _matrix.M24() + M33() * _matrix.M34() + M34() * _matrix.M44(),

		M41() * _matrix.M11() + M42() * _matrix.M21() + M43() * _matrix.M31() + M44() * _matrix.M41(),
		M41() * _matrix.M12() + M42() * _matrix.M22() + M43() * _matrix.M32() + M44() * _matrix.M42(),
		M41() * _matrix.M13() + M42() * _matrix.M23() + M43() * _matrix.M33() + M44() * _matrix.M43(),
		M41() * _matrix.M14() + M42() * _matrix.M24() + M43() * _matrix.M34() + M44() * _matrix.M44()
		);

	return tmp;*/
}
Mat4f Mat4f::operator * (const float _scalar) const
{
	__m128 fac = _mm_load1_ps(&_scalar);
	__m128 row0, row1, row2, row3;

	row0 = _mm_mul_ps(_mm_load_ps(values), fac);
	row1 =  _mm_mul_ps(_mm_load_ps(values+4), fac);
	row2 =  _mm_mul_ps(_mm_load_ps(values+8), fac);
	row3 =  _mm_mul_ps(_mm_load_ps(values+12), fac);

	return Mat4f(row0, row1, row2, row3);

	/*Mat4f m(values);
	m *= _scalar;
	return m;*/
}
Vec4f Mat4f::operator * (const Vec4f& _vector) const
{
	__m128 row0, row1, row2, row3;
	__m128 vec = _mm_load_ps(_vector.GetGlVector());

	row0 = _mm_load_ps(values);
	row1 = _mm_load_ps(values+4);
	row2 = _mm_load_ps(values+8);
	row3 = _mm_load_ps(values+12);

	__m128 result = _mm_dp_ps(row0, vec, B8(11110001));
	result = _mm_add_ps(_mm_dp_ps(row1, vec, B8(11110010)), result);
	result = _mm_add_ps(_mm_dp_ps(row2, vec, B8(11110100)), result);
	result = _mm_add_ps(_mm_dp_ps(row3, vec, B8(11111000)), result);

	Vec4f tmpVec;
	_mm_store_ps(tmpVec.GetGlVector(), result);
	return tmpVec;

	/*return Vec4f(
		M11() * _vector.X() + M12() * _vector.Y() + M13() * _vector.Z() + M14() * _vector.W(),
		M21() * _vector.X() + M22() * _vector.Y() + M23() * _vector.Z() + M24() * _vector.W(),
		M31() * _vector.X() + M32() * _vector.Y() + M33() * _vector.Z() + M34() * _vector.W(),
		M41() * _vector.X() + M42() * _vector.Y() + M43() * _vector.Z() + M44() * _vector.W()
		);*/
}
Vec3f Mat4f::operator * (const Vec3f& _vector) const
{
	return Vec3f(
		M11() * _vector.X() + M12() * _vector.Y() + M13() * _vector.Z(),
		M21() * _vector.X() + M22() * _vector.Y() + M23() * _vector.Z(),
		M31() * _vector.X() + M32() * _vector.Y() + M33() * _vector.Z()
		);
}
Mat4f Mat4f::operator / (const float _scalar) const
{
	__m128 fac = _mm_div_ps(_mm_set1_ps(1.0f), _mm_load1_ps(&_scalar));
	__m128 row0, row1, row2, row3;

	row0 = _mm_mul_ps(_mm_load_ps(values), fac);
	row1 =  _mm_mul_ps(_mm_load_ps(values+4), fac);
	row2 =  _mm_mul_ps(_mm_load_ps(values+8), fac);
	row3 =  _mm_mul_ps(_mm_load_ps(values+12), fac);

	return Mat4f(row0, row1, row2, row3);

	/*float rezi = 1.0f / _scalar;
	Mat4f m(values);
	m *= rezi;
	return m;*/
}
Mat4f& Mat4f::operator *= (const Mat4f& _matrix)
{
	__m128 arow0, arow1, arow2, arow3;
	__m128 brow0, brow1, brow2, brow3;
	__m128 crow0, crow1, crow2, crow3;

	arow0 = _mm_load_ps(values);
	arow1 = _mm_load_ps(values+4);
	arow2 = _mm_load_ps(values+8);
	arow3 = _mm_load_ps(values+12);

	brow0 = _mm_load_ps(_matrix.values);
	brow1 = _mm_load_ps(_matrix.values+4);
	brow2 = _mm_load_ps(_matrix.values+8);
	brow3 = _mm_load_ps(_matrix.values+12);

	_MM_TRANSPOSE4_PS(brow0, brow1, brow2, brow3);

	crow0 = _mm_dp_ps(arow0, brow0, B8(11110001));
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow1, B8(11110010)), crow0);
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow2, B8(11110100)), crow0);
	crow0 = _mm_add_ps(_mm_dp_ps(arow0, brow3, B8(11111000)), crow0);

	crow1 = _mm_dp_ps(arow1, brow0, B8(11110001));
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow1, B8(11110010)), crow1);
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow2, B8(11110100)), crow1);
	crow1 = _mm_add_ps(_mm_dp_ps(arow1, brow3, B8(11111000)), crow1);

	crow2 = _mm_dp_ps(arow2, brow0, B8(11110001));
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow1, B8(11110010)), crow2);
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow2, B8(11110100)), crow2);
	crow2 = _mm_add_ps(_mm_dp_ps(arow2, brow3, B8(11111000)), crow2);

	crow3 = _mm_dp_ps(arow3, brow0, B8(11110001));
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow1, B8(11110010)), crow3);
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow2, B8(11110100)), crow3);
	crow3 = _mm_add_ps(_mm_dp_ps(arow3, brow3, B8(11111000)), crow3);

	_mm_store_ps(values, crow0);
	_mm_store_ps(values+4, crow1);
	_mm_store_ps(values+8, crow2);
	_mm_store_ps(values+12, crow3);

	return *this;

	/*Mat4f z = *this;

	M11(z.M11() * _matrix.M11() + z.M12() * _matrix.M21() + z.M13() * _matrix.M31() + z.M14() * _matrix.M41());
	M12(z.M11() * _matrix.M12() + z.M12() * _matrix.M22() + z.M13() * _matrix.M32() + z.M14() * _matrix.M42());
	M13(z.M11() * _matrix.M13() + z.M12() * _matrix.M23() + z.M13() * _matrix.M33() + z.M14() * _matrix.M43());
	M14(z.M11() * _matrix.M14() + z.M12() * _matrix.M24() + z.M13() * _matrix.M34() + z.M14() * _matrix.M44());

	M21(z.M21() * _matrix.M11() + z.M22() * _matrix.M21() + z.M23() * _matrix.M31() + z.M24() * _matrix.M41());
	M22(z.M21() * _matrix.M12() + z.M22() * _matrix.M22() + z.M23() * _matrix.M32() + z.M24() * _matrix.M42());
	M23(z.M21() * _matrix.M13() + z.M22() * _matrix.M23() + z.M23() * _matrix.M33() + z.M24() * _matrix.M43());
	M24(z.M21() * _matrix.M14() + z.M22() * _matrix.M24() + z.M23() * _matrix.M34() + z.M24() * _matrix.M44());

	M31(z.M31() * _matrix.M11() + z.M32() * _matrix.M21() + z.M33() * _matrix.M31() + z.M34() * _matrix.M41());
	M32(z.M31() * _matrix.M12() + z.M32() * _matrix.M22() + z.M33() * _matrix.M32() + z.M34() * _matrix.M42());
	M33(z.M31() * _matrix.M13() + z.M32() * _matrix.M23() + z.M33() * _matrix.M33() + z.M34() * _matrix.M43());
	M34(z.M31() * _matrix.M14() + z.M32() * _matrix.M24() + z.M33() * _matrix.M34() + z.M34() * _matrix.M44());

	M41(z.M41() * _matrix.M11() + z.M42() * _matrix.M21() + z.M43() * _matrix.M31() + z.M44() * _matrix.M41());
	M42(z.M41() * _matrix.M12() + z.M42() * _matrix.M22() + z.M43() * _matrix.M32() + z.M44() * _matrix.M42());
	M43(z.M41() * _matrix.M13() + z.M42() * _matrix.M23() + z.M43() * _matrix.M33() + z.M44() * _matrix.M43());
	M44(z.M41() * _matrix.M14() + z.M42() * _matrix.M24() + z.M43() * _matrix.M34() + z.M44() * _matrix.M44());

	return *this;*/
}
Mat4f& Mat4f::operator *= (const float _scalar)
{
	__m128 fac = _mm_load1_ps(&_scalar);
	__m128 row0, row1, row2, row3;

	row0 = _mm_mul_ps(_mm_load_ps(values), fac);
	row1 =  _mm_mul_ps(_mm_load_ps(values+4), fac);
	row2 =  _mm_mul_ps(_mm_load_ps(values+8), fac);
	row3 =  _mm_mul_ps(_mm_load_ps(values+12), fac);

	_mm_store_ps(values, row0);
	_mm_store_ps(values+4, row1);
	_mm_store_ps(values+8, row2);
	_mm_store_ps(values+12, row3);
	
	return *this;
}
Mat4f& Mat4f::operator /= (const float _scalar)
{
	__m128 fac = _mm_div_ps(_mm_set1_ps(1.0f), _mm_load1_ps(&_scalar));
	__m128 row0, row1, row2, row3;

	row0 = _mm_mul_ps(_mm_load_ps(values), fac);
	row1 =  _mm_mul_ps(_mm_load_ps(values+4), fac);
	row2 =  _mm_mul_ps(_mm_load_ps(values+8), fac);
	row3 =  _mm_mul_ps(_mm_load_ps(values+12), fac);

	_mm_store_ps(values, row0);
	_mm_store_ps(values+4, row1);
	_mm_store_ps(values+8, row2);
	_mm_store_ps(values+12, row3);
	
	return *this;
}
Mat4f& Mat4f::operator = (const Mat4f& _matrix)
{
	if (this != &_matrix)
	{
		memcpy(values, _matrix.values, 16*sizeof(float));
	}

	return *this;
}
bool Mat4f::operator == (const Mat4f& _matrix) const
{
	for (int i = 0; i < 16; i++)
		if (values[i] != _matrix.values[i])
			return false;

	return true;
}
bool Mat4f::operator != (const Mat4f& _matrix) const
{
	for (int i = 0; i < 16; i++)
		if (values[i] != _matrix.values[i])
			return true;

	return false;
}

inline float Mat4f::M(int _column, int _row) const
{
	return values[_row * 4 + _column];
}
inline void Mat4f::M(int _column, int _row, float _value)
{
	values[_row * 4 + _column] = _value;
}

std::ostream& operator <<(std::ostream &os, const Mat4f &obj)
{
	os << "[" << obj.M11() << ", " << obj.M12() << ", " << obj.M13() << ", " << obj.M14() << "; " << obj.M21() << ", " <<  obj.M22() << ", " <<  obj.M23() << ", " << obj.M24() << "; " <<  obj.M31() << ", " <<  obj.M32() << ", " <<  obj.M33() << ", " << obj.M34() << "; " << obj.M41() << ", " << obj.M42() << ", " << obj.M43() << ", " << obj.M44() << "]";
	return os;
}