#define DLL_EXPORT

#include "Vec2f.hpp"
#include "../../Core/Hardware/SSEInstructions.hpp"
#include "../../Core/Concurrency/ThreadManager.hpp"
#include "../MathMacros.hpp"

#include <smmintrin.h>

using namespace Engine::Math::SSE;
using namespace Engine::Core::Concurrency;

Vec2f::Vec2f()
{
	values[0] = 0.0;
	values[1] = 0.0;
}

Vec2f::Vec2f(float _x, float _y)
{
	values[0] = _x;
	values[1] = _y;
}

Vec2f::Vec2f(float* _values)
{
	values[0] = _values[0];
	values[1] = _values[1];
}

Vec2f::Vec2f(const Vec2f& _refToVec)
{	
	memcpy(&values, &(_refToVec.values), sizeof(float) * 2);
}

Vec2f::~Vec2f()
{
}

inline float Vec2f::X() const
{
	return this->values[0];
}

inline Vec2f& Vec2f::X(float _x)
{
	values[0] = _x;
	return *this;
}

inline float Vec2f::Y() const
{
	return this->values[1];
}

inline Vec2f& Vec2f::Y(float _y)
{
	values[1] = _y;
	return *this;
}

inline float* Vec2f::GetGlVector() const
{
	return (float*)(&values[0]);
}

Vec2f& Vec2f::Normalize()
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 length = _mm_dp_ps(vec, vec, B8(00111111));
	length = _mm_sqrt_ps(length);

	vec = _mm_div_ps(vec, length);
	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec);

	memcpy(&this->values, ThreadManager::tmpSSEMemory<float, 0>(), 2*sizeof(float));
	return *this;
}

Vec2f Vec2f::Normalize(const Vec2f &_vector)
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_vector.values);
	__m128 length = _mm_dp_ps(vec, vec, B8(00111111));
	length = _mm_sqrt_ps(length);

	vec = _mm_div_ps(vec, length);
	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec);

	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());
}

float Vec2f::Length() const
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 length = _mm_dp_ps(vec, vec, B8(00111111));
	length = _mm_sqrt_ps(length);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), length);
	return *ThreadManager::tmpSSEMemory<float, 0>();
}

float Vec2f::Length(const Vec2f &_vector)
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_vector.values);
	__m128 length = _mm_dp_ps(vec, vec, B8(00111111));
	length = _mm_sqrt_ps(length);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), length);
	return *ThreadManager::tmpSSEMemory<float, 0>();
}

Vec2f Vec2f::ComponentMax(const Vec2f& _v1, const Vec2f& _v2)
{
	__m128 vec1 = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v1.values);
	__m128 vec2 = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v2.values);

	vec1 = _mm_max_ps(vec1, vec2);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec1);
	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec2f Vec2f::ComponentMin(const Vec2f& _v1, const Vec2f& _v2)
{
	__m128 vec1 = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v1.values);
	__m128 vec2 = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v2.values);

	vec1 = _mm_min_ps(vec1, vec2);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec1);
	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec2f Vec2f::Interpolate(const Vec2f& _v1, const Vec2f& _v2, float _a1, float _a2)
{
	//Diese Variante baut komplett auf SSE auf, eventuell wre es sinnvoll den factor vorher zu berechnen
	//18.6: Versuch zeigt, dass es egal ist.
	__m128 a1register = _mm_load1_ps(&_a1);
	__m128 a2register = _mm_load1_ps(&_a2);

	a2register = _mm_add_ps(a1register, a2register); // _a1 + _a2
	a1register = _mm_div_ps(a1register, a2register); // _a1 / (_a1 + _a2)

	__m128 v1register = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v1.values);
	__m128 v2register = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v2.values);

	v2register = _mm_sub_ps(v2register, v1register); // _v2 - _v1
	v2register = _mm_mul_ps(v2register, a1register); // (_v2 - _v1) * fac

	v1register = _mm_add_ps(v1register, v2register); // _v1 + facvec

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), v1register);
	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());

	//Hier wird der Factor vorberechnet
	/*float fac = _a1 / (_a1 + _a2);
	__m128 facregister = _mm_load1_ps(&fac);

	__m128 v1register = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v1.values);
	__m128 v2register = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&_v2.values);

	v2register = _mm_sub_ps(v2register, v1register); // _v2 - _v1
	v2register = _mm_mul_ps(v2register, facregister); // (_v2 - _v1) * fac

	v1register = _mm_add_ps(v1register, v2register); // _v1 + facvec

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), v1register);
	return Vec2f(SystemConfiguration::tmpSSEMemory<float, 0>());*/
}

Vec2f Vec2f::operator+(const Vec2f& _toAdd) const
{
	/*__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 add = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)_toAdd.values);
	
	vec = _mm_add_ps(vec, add);

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), vec);
	return Vec2f(SystemConfiguration::tmpSSEMemory<float, 0>());*/

	return Vec2f(values[0] + _toAdd.values[0], values[1] + _toAdd.values[1]);
}

Vec2f& Vec2f::operator+=(const Vec2f& _toAdd)
{
	values[0] += _toAdd.values[0];
	values[1] += _toAdd.values[1];
	return *this;

	/*__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 add = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)_toAdd.values);
	
	vec = _mm_add_ps(vec, add);

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), vec);
	memcpy(&this->values, SystemConfiguration::tmpSSEMemory<float, 0>(), sizeof(float)*2);
	return *this;*/
}

Vec2f Vec2f::operator-(const Vec2f& _toSubtract) const
{
	/*
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 sub = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)_toSubtract.values);
	
	vec = _mm_sub_ps(vec, sub);

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), vec);
	return Vec2f(SystemConfiguration::tmpSSEMemory<float, 0>());
	*/

	return Vec2f(values[0] - _toSubtract.values[0], values[1] - _toSubtract.values[1]);
}

Vec2f& Vec2f::operator-=(const Vec2f& _toSubtract)
{
	values[0] -= _toSubtract.values[0];
	values[1] -= _toSubtract.values[1];
	return *this;

	/*__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 sub = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)_toSubtract.values);
	
	vec = _mm_sub_ps(vec, sub);

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), vec);
	memcpy(&this->values, SystemConfiguration::tmpSSEMemory<float, 0>(), sizeof(float)*2);
	return *this;*/
}

Vec2f Vec2f::operator*(float _scalar) const
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 scalar = _mm_load1_ps(&_scalar);

	vec = _mm_mul_ps(vec, scalar);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec);
	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec2f& Vec2f::operator*=(float _scalar)
{
	values[0] *= _scalar;
	values[1] *= _scalar;
	return *this;

	/*__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 scalar = _mm_load1_ps(&_scalar);

	vec = _mm_mul_ps(vec, scalar);

	_mm_storel_pi(SystemConfiguration::tmpSSEMemory<__m64, 0>(), vec);
	memcpy(&this->values, SystemConfiguration::tmpSSEMemory<float, 0>(), sizeof(float)*2);
	return *this;*/
}

Vec2f Vec2f::operator/ (float _scalar) const
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 scalar = _mm_load1_ps(&_scalar);

	vec = _mm_div_ps(vec, scalar);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec);
	return Vec2f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec2f& Vec2f::operator /= (float _scalar)
{
	__m128 vec = _mm_loadl_pi(_mm_setzero_ps(), (__m64*)&this->values);
	__m128 scalar = _mm_load1_ps(&_scalar);

	vec = _mm_div_ps(vec, scalar);

	_mm_storel_pi(ThreadManager::tmpSSEMemory<__m64, 0>(), vec);
	memcpy(&this->values, ThreadManager::tmpSSEMemory<float, 0>(), sizeof(float)*2);
	return *this;
}

bool Vec2f::operator==(const Vec2f& _toCompare) const
{
	return (_toCompare.values[0] == values[0] && _toCompare.values[1] == values[1]);
}

bool Vec2f::operator!=(const Vec2f& _toCompare) const
{
	return (_toCompare.values[0] != values[0] || _toCompare.values[1] != values[1]);
}

Vec2f& Vec2f::operator=(const Vec2f& _copy)
{
	memcpy(&this->values, &_copy.values, 2 * sizeof(float));
	return *this;
}

std::ostream& operator <<(std::ostream &os, const Vec2f &obj)
{
	os << "[" << obj.X() << ", " << obj.Y() << "]";
	return os;
}