#define DLL_EXPORT

#include "Vec4f.hpp"

#include "../../Core/Hardware/SSEInstructions.hpp"
#include "..\MathMacros.hpp"
#include "../../Core/Concurrency/ThreadManager.hpp"
#include <math.h>
#include <smmintrin.h>

using namespace Engine::Math::SSE;
using namespace Engine::Core::Concurrency;

Vec4f::Vec4f()
{
	values[0]=0;
	values[1]=0;
	values[2]=0;
	values[3]=1;
}

Vec4f::Vec4f(float* _values)
{
	memcpy(values, _values, 4 * sizeof(float));
}

Vec4f::Vec4f(float _x, float _y, float _z)
{
	values[0]=_x;
	values[1]=_y;
	values[2]=_z;
	values[3]=1;
}

Vec4f::Vec4f(float _x, float _y, float _z, float _w)
{
	values[0]=_x;
	values[1]=_y;
	values[2]=_z;
	values[3]=_w;
}

Vec4f::Vec4f(const Vec4f &_refToVec)
{
	memcpy(values, &_refToVec.values, 4 * sizeof(float));
}

Vec4f::Vec4f(const Vec3f &_refToVec, float _w)
{
	values[0]=_refToVec.X();
	values[1]=_refToVec.Y();
	values[2]=_refToVec.Z();
	values[3]=_w;
}

Vec4f& Vec4f::operator = (const Vec4f &_rhs)
{
	memcpy(values, &_rhs.values, 4 * sizeof(float));
	return *this;
}

Vec4f::~Vec4f()
{
}


inline float Vec4f::X() const
{
	return values[0];
}

inline Vec4f& Vec4f::X(float _x)
{
	values[0]=_x;
	return *this;
}

inline float Vec4f::Y() const
{
	return values[1];
}

inline Vec4f& Vec4f::Y(float _y)
{
	values[1]=_y;
	return *this;
}

inline float Vec4f::Z() const
{
	return values[2];
}

inline Vec4f& Vec4f::Z(float _z)
{
	values[2]=_z;
	return *this;
}

inline float Vec4f::W() const
{
	return values[3];
}

inline Vec4f& Vec4f::W(float _w)
{
	values[3]=_w;
	return *this;
}

inline float* Vec4f::GetGlVector() const
{
	return (float*)(&(values[0]));
}

inline Vec3f Vec4f::GetVec3f() const
{
	return Vec3f((float*)values);
}


Vec4f& Vec4f::Homogenize()
{
	__m128 vecregister = _mm_load_ps(values);
	__m128 divregister = _mm_load1_ps(&values[3]);

	vecregister = _mm_div_ps(vecregister, divregister);

	_mm_store_ps(values, vecregister);
	return *this;
}

Vec4f Vec4f::Homogenize(const Vec4f& _vector)
{
	__m128 vecregister = _mm_load_ps(_vector.values);
	__m128 divregister = _mm_load1_ps(&_vector.values[3]);

	vecregister = _mm_div_ps(vecregister, divregister);

	Vec4f retVec;
	_mm_store_ps(retVec.values, vecregister);
	return retVec;
}

Vec4f& Vec4f::Normalize3()
{
	__m128 vecregister = _mm_load_ps(values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(01111111));
	length = _mm_sqrt_ps(length);

	vecregister = _mm_div_ps(vecregister, length);

	_mm_store_ps(ThreadManager::tmpSSEMemory<float, 0>(), vecregister);
	memcpy(values, ThreadManager::tmpSSEMemory<float, 0>(), 3 * sizeof(float));

	return *this;
}

Vec4f Vec4f::Normalize3(const Vec4f &_vector)
{
	__m128 vecregister = _mm_load_ps(_vector.values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(01111111));
	length = _mm_sqrt_ps(length);

	vecregister = _mm_div_ps(vecregister, length);

	_mm_store_ps(ThreadManager::tmpSSEMemory<float, 0>(), vecregister);
	ThreadManager::tmpSSEMemory<float, 0>()[3] = _vector.values[3];

	return Vec4f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec4f& Vec4f::Normalize()
{
	__m128 vecregister = _mm_load_ps(values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(11111111));
	length = _mm_sqrt_ps(length);

	vecregister = _mm_div_ps(vecregister, length);

	_mm_store_ps(values, vecregister);
	return *this;
}

Vec4f Vec4f::Normalize(const Vec4f &_vector)
{
	__m128 vecregister = _mm_load_ps(_vector.values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(11111111));
	length = _mm_sqrt_ps(length);

	vecregister = _mm_div_ps(vecregister, length);

	Vec4f retVec;
	_mm_store_ps(retVec.values, vecregister);
	return retVec;
}
			
float Vec4f::Length3() const
{
	__m128 vecregister = _mm_load_ps(values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(01111111));
	length = _mm_sqrt_ps(length);
	float l;
	_mm_store_ss(&l, length);
	
	return l;
}

float Vec4f::Length3(const Vec4f &_vector){
	__m128 vecregister = _mm_load_ps(_vector.values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(01111111));
	length = _mm_sqrt_ps(length);
	float l;
	_mm_store_ss(&l, length);
	
	return l;
}

float Vec4f::Length() const
{
	__m128 vecregister = _mm_load_ps(values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(11111111));
	length = _mm_sqrt_ps(length);
	float l;
	_mm_store_ss(&l, length);
	
	return l;
}

float Vec4f::Length(const Vec4f &_vector)
{
	__m128 vecregister = _mm_load_ps(_vector.values);
	__m128 length = _mm_dp_ps(vecregister, vecregister, B8(11111111));
	length = _mm_sqrt_ps(length);
	float l;
	_mm_store_ss(&l, length);
	
	return l;
}

float Vec4f::Dot3(const Vec4f &_left, const Vec4f &_right)
{
	__m128 vec1register = _mm_load_ps(_left.values);
	__m128 vec2register = _mm_load_ps(_right.values);
	__m128 dp = _mm_dp_ps(vec1register, vec2register, B8(01111111));
	float f;
	_mm_store_ss(&f, dp);
	
	return f;
}

float Vec4f::Dot(const Vec4f &_left, const Vec4f &_right)
{
	__m128 vec1register = _mm_load_ps(_left.values);
	__m128 vec2register = _mm_load_ps(_right.values);
	__m128 dp = _mm_dp_ps(vec1register, vec2register, B8(11111111));
	float f;
	_mm_store_ss(&f, dp);
	
	return f;
}

Vec4f Vec4f::Cross3(const Vec4f &_left, const Vec4f &_right)
{
	__m128 v1 = _mm_load_ps(_left.values);
	__m128 v2 = _mm_load_ps(_right.values);

	__m128 v1shift1 = _mm_shuffle_ps(v1, v1, _MM_SHUFFLE(0, 0, 2, 1));
	__m128 v2shift1 = _mm_shuffle_ps(v2, v2, _MM_SHUFFLE(0, 0, 2, 1));

	__m128 v1shift2 = _mm_shuffle_ps(v1, v1, _MM_SHUFFLE(0, 1, 0, 2));
	__m128 v2shift2 = _mm_shuffle_ps(v2, v2, _MM_SHUFFLE(0, 1, 0, 2));

	v1shift1 = _mm_mul_ps(v1shift1, v2shift2);
	v2shift1 = _mm_mul_ps(v2shift1, v1shift2);

	v1shift1 = _mm_sub_ps(v1shift1, v2shift1);

	_mm_store_ps(ThreadManager::tmpSSEMemory<float, 0>(), v1shift1);
	ThreadManager::tmpSSEMemory<float, 0>()[3] = 1.0f;

	return Vec4f(ThreadManager::tmpSSEMemory<float, 0>());
}

Vec4f Vec4f::ComponentMax(const Vec4f& _v1, const Vec4f& _v2)
{
	__m128 v1 = _mm_load_ps(_v1.values);
	__m128 v2 = _mm_load_ps(_v2.values);

	v1 = _mm_max_ps(v1, v2);

	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec4f Vec4f::ComponentMin(const Vec4f& _v1, const Vec4f& _v2)
{
	__m128 v1 = _mm_load_ps(_v1.values);
	__m128 v2 = _mm_load_ps(_v2.values);

	v1 = _mm_min_ps(v1, v2);

	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec4f Vec4f::Interpolate(const Vec4f& _v1, const Vec4f& _v2, float _a1, float _a2)
{
	__m128 a1register = _mm_load1_ps(&_a1);
	__m128 a2register = _mm_load1_ps(&_a2);

	a2register = _mm_add_ps(a1register, a2register); // _a1 + _a2
	a1register = _mm_div_ps(a1register, a2register); // _a1 / (_a1 + _a2)

	__m128 v1register = _mm_load_ps(_v1.values);
	__m128 v2register = _mm_load_ps(_v2.values);

	v2register = _mm_sub_ps(v2register, v1register); // _v2 - _v1
	v2register = _mm_mul_ps(v2register, a1register); // (_v2 - _v1) * fac

	v1register = _mm_add_ps(v1register, v2register); // _v1 + facvec

	Vec4f vec;
	_mm_store_ps(vec.values, v1register);
	return vec;
}

Vec4f Vec4f::operator + (const Vec4f &_rhs) const
{
	__m128 v1 = _mm_load_ps(values);
	__m128 v2 = _mm_load_ps(_rhs.values);

	v1 = _mm_add_ps(v1, v2);
	
	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec3f Vec4f::operator + (const Vec3f &_rhs) const
{
	/*__m128 v1 = _mm_load_ps(values);
	__m128i low = _mm_loadl_epi64((const __m128i*)_rhs.GetGlVector());
	__m128 v2 = _mm_load_ss(&_rhs.GetGlVector()[2]);
	v2 = _mm_movelh_ps(_mm_castsi128_ps(low), v2);

	v1 = _mm_add_ps(v1, v2);
	
	_mm_store_ps(SystemConfiguration::tmpSSEMemory<float, 0>(), v1);

	return Vec3f(SystemConfiguration::tmpSSEMemory<float, 0>());*/

	return Vec3f(values[0]+_rhs.GetGlVector()[0], values[1]+_rhs.GetGlVector()[1], values[2]+_rhs.GetGlVector()[2]);
}
			
Vec4f Vec4f::operator - (const Vec4f &_rhs) const
{
	__m128 v1 = _mm_load_ps(values);
	__m128 v2 = _mm_load_ps(_rhs.values);

	v1 = _mm_sub_ps(v1, v2);
	
	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec3f Vec4f::operator - (const Vec3f &_rhs) const
{
	/*__m128 v1 = _mm_load_ps(values);
	__m128i low = _mm_loadl_epi64((const __m128i*)_rhs.GetGlVector());
	__m128 v2 = _mm_load_ss(&_rhs.GetGlVector()[2]);
	v2 = _mm_movelh_ps(_mm_castsi128_ps(low), v2);

	v1 = _mm_sub_ps(v1, v2);
	
	_mm_store_ps(SystemConfiguration::tmpSSEMemory<float, 0>(), v1);

	return Vec3f(SystemConfiguration::tmpSSEMemory<float, 0>());*/
	return Vec3f(values[0]-_rhs.GetGlVector()[0], values[1]-_rhs.GetGlVector()[1], values[2]-_rhs.GetGlVector()[2]);
}

Vec4f Vec4f::operator * (float _scalar) const		
{
	__m128 v1 = _mm_load_ps(values);
	__m128 fac = _mm_load1_ps(&_scalar);

	v1 = _mm_mul_ps(v1, fac);

	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec4f Vec4f::operator / (float _scalar) const		
{
	__m128 v1 = _mm_load_ps(values);
	__m128 fac = _mm_load1_ps(&_scalar);

	v1 = _mm_div_ps(v1, fac);

	Vec4f vec;
	_mm_store_ps(vec.values, v1);

	return vec;
}

Vec4f& Vec4f::operator += (const Vec3f &_rhs)
{
	/*__m128 v1 = _mm_load_ps(values);
	__m128i low = _mm_loadl_epi64((const __m128i*)_rhs.GetGlVector());
	__m128 v2 = _mm_load_ss(&_rhs.GetGlVector()[2]);
	v2 = _mm_movelh_ps(_mm_castsi128_ps(low), v2);

	v1 = _mm_add_ps(v1, v2);
	
	_mm_store_ps(values, v1);

	return *this;*/
	values[0]+=_rhs.GetGlVector()[0];
	values[1]+=_rhs.GetGlVector()[1];
	values[2]+=_rhs.GetGlVector()[2];
	return *this;
}

Vec4f& Vec4f::operator += (const Vec4f &_rhs)
{
	__m128 v1 = _mm_load_ps(values);
	__m128 v2 = _mm_load_ps(_rhs.values);

	v1 = _mm_add_ps(v1, v2);
	_mm_store_ps(values, v1);

	return *this;
}

Vec4f& Vec4f::operator -= (const Vec3f &_rhs)
{
	/*__m128 v1 = _mm_load_ps(values);
	__m128i low = _mm_loadl_epi64((const __m128i*)_rhs.GetGlVector());
	__m128 v2 = _mm_load_ss(&_rhs.GetGlVector()[2]);
	v2 = _mm_movelh_ps(_mm_castsi128_ps(low), v2);

	v1 = _mm_sub_ps(v1, v2);
	
	_mm_store_ps(values, v1);

	return *this;*/
	values[0]-=_rhs.GetGlVector()[0];
	values[1]-=_rhs.GetGlVector()[1];
	values[2]-=_rhs.GetGlVector()[2];
	return *this;
}

Vec4f& Vec4f::operator -= (const Vec4f &_rhs)
{
	__m128 v1 = _mm_load_ps(values);
	__m128 v2 = _mm_load_ps(_rhs.values);

	v1 = _mm_sub_ps(v1, v2);
	_mm_store_ps(values, v1);

	return *this;
}

Vec4f& Vec4f::operator *= (float _scalar)
{
	__m128 v1 = _mm_load_ps(values);
	__m128 fac = _mm_load1_ps(&_scalar);

	v1 = _mm_mul_ps(v1, fac);

	_mm_store_ps(values, v1);

	return *this;
}

Vec4f& Vec4f::operator /= (float _scalar)
{
	__m128 v1 = _mm_load_ps(values);
	__m128 fac = _mm_load1_ps(&_scalar);

	v1 = _mm_div_ps(v1, fac);

	_mm_store_ps(values, v1);

	return *this;
}

bool Vec4f::operator == (const Vec4f &_rhs) const
{
	if(values[0]!=_rhs.values[0]) return false;
	if(values[1]!=_rhs.values[1]) return false;
	if(values[2]!=_rhs.values[2]) return false;
	if(values[3]!=_rhs.values[3]) return false;
	return true;
}

bool Vec4f::operator != (const Vec4f &_rhs) const
{
	if(values[0]!=_rhs.values[0]) return true;
	if(values[1]!=_rhs.values[1]) return true;
	if(values[2]!=_rhs.values[2]) return true;
	if(values[3]!=_rhs.values[3]) return true;
	return false;
}

__m128 Vec4f::CopyToRegister() const
{
	return _mm_load_ps(values);
}

std::ostream& operator <<(std::ostream &os, const Vec4f &obj)
{
	os << "[" << obj.X() << ", " << obj.Y() << ", " << obj.Z() << ", " << obj.W() << "]";
	return os;
}