#define DLL_EXPORT

#include "Animation.hpp"
#include "Curve.hpp"
#include "CurveData.hpp"

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Animations;

Animation::Animation(std::string _name, Curve** _curves, unsigned int _count, bool _isLooped, float _speed)
	: name(_name), curves(_curves), count(_count), isLooped(_isLooped), running(false), lastUpdateTime(0.0f), maximalLength(0.0f), speed(_speed)
{
		for (unsigned int i = 0; i < _count; i++)
			maximalLength = max(maximalLength, _curves[i]->Length());
}

Animation::~Animation()
{
	for (unsigned int i = 0; i < count; i++)
		delete curves[i];

	if (curves != nullptr)
		delete[] curves;
}

void Animation::BindComponent(float* _locX, float* _locY, float* _locZ, float* _rotX, float* _rotY, float* _rotZ, float* _scaleX, float* _scaleY, float* _scaleZ)
{
	for (unsigned int i = 0; i < count; i++)
	{
		switch (curves[i]->Data()->Target())
		{
		case CurveTarget::LocationXCurveTarget:
			curves[i]->TargetValue(_locX);
			break;
		case CurveTarget::LocationYCurveTarget:
			curves[i]->TargetValue(_locY);
			break;
		case CurveTarget::LocationZCurveTarget:
			curves[i]->TargetValue(_locZ);
			break;
		case CurveTarget::RotationXCurveTarget:
			curves[i]->TargetValue(_rotX);
			break;
		case CurveTarget::RotationYCurveTarget:
			curves[i]->TargetValue(_rotY);
			break;
		case CurveTarget::RotationZCurveTarget:
			curves[i]->TargetValue(_rotZ);
			break;
		case CurveTarget::ScaleXCurveTarget:
			curves[i]->TargetValue(_scaleX);
			break;
		case CurveTarget::ScaleYCurveTarget:
			curves[i]->TargetValue(_scaleY);
			break;
		case CurveTarget::ScaleZCurveTarget:
			curves[i]->TargetValue(_scaleZ);
			break;
		}
	}
}

void Animation::Update(float _t)
{
	float timeInAnimation = (_t - startTime) * speed;
	if (timeInAnimation > maximalLength)
	{
		if (isLooped)
		{
			startTime += maximalLength * 1.0f / speed;
			timeInAnimation = (_t - startTime) * speed;
		}
		else
		{
			timeInAnimation = maximalLength * 1.0f / speed;
		}
	}

	if (running)
	{
		for (unsigned int i = 0; i < count; i++)
			curves[i]->Interpolate(timeInAnimation);
	}

	lastUpdateTime = _t;
}

void Animation::Start()
{
	startTime = lastUpdateTime;
	running = true;
}

void Animation::Stop()
{
	startTime = lastUpdateTime - startTime;
	running = false;
}

void Animation::Resume()
{
	startTime = lastUpdateTime - startTime;
	running = true;
}

inline std::string Animation::Name() const
{
	return name;
}

inline float Animation::Speed() const
{
	return speed;
}

inline void Animation::Speed(float _value)
{
	speed = _value;
}