#ifndef _ENGINE_SCENE_BOUNDINGS_BIH_BIH_HPP
#define _ENGINE_SCENE_BOUNDINGS_BIH_BIH_HPP

namespace Engine
{
	namespace Scene
	{
		namespace Boundings
		{
			namespace Bih
			{
				enum CubeFace {neg_x, pos_x, neg_y, pos_y, neg_z, pos_z};

				enum NodeType {x, y, z, leaf};
				
				union PlaneData
				{
					/* Clip values from origin for left (clip[0]) and right (clip[1]) subtree
					   only set if Node has Children */
					float clip[2];

					/* start and finish indizes in boundingObjects
					   only set if Node is a leaf */
					size_t objects_range[2];
				};

				struct BihNode
				{
					NodeType type;
					
					PlaneData data;
					
					/* Not set if Node is a leaf (check BihNode.type) */
					size_t children_idx[2]; /* in std::vec flattenedtree */
				};

				template<typename T>
				struct MinMaxBox
				{
					T x_min, x_max, y_min, y_max, z_min, z_max;

					NodeType GetMaxSpan() const
					{
						float tmp;
						return GetMaxSpan(tmp);
					}

					NodeType GetMaxSpan(T& outval) const
					{
						T spans[3];
						spans[0] = abs(x_max - x_min);
						spans[1] = abs(y_max - y_min);
						spans[2] = abs(z_max - z_min);
						
						int maxpos = 0;
						for(int i=1; i<3; i++)
							maxpos = (spans[i] > spans[maxpos]) ? i : maxpos;

						outval = spans[maxpos];
						switch(maxpos)
						{
							case 0: return NodeType::x;
							case 1: return NodeType::y;
							case 2: return NodeType::z;
						}
						throw std::logic_error("It should have returned x, y or z");
					}

					T Min(NodeType nt) const
					{
						switch(nt)
						{
							case NodeType::x : return x_min;
							case NodeType::y : return y_min;
							case NodeType::z : return z_min;
						}
						throw std::runtime_error("Parameter must be x, y or z");
					}

					T Max(NodeType nt) const
					{
						switch(nt)
						{
							case NodeType::x : return x_max;
							case NodeType::y : return y_max;
							case NodeType::z : return z_max;
						}
						throw std::runtime_error("Parameter must be x, y or z");
					}

					T& Min(NodeType nt)
					{
						switch(nt)
						{
							case NodeType::x : return x_min;
							case NodeType::y : return y_min;
							case NodeType::z : return z_min;
						}
						throw std::runtime_error("Parameter must be x, y or z");
					}

					T& Max(NodeType nt)
					{
						switch(nt)
						{
							case NodeType::x : return x_max;
							case NodeType::y : return y_max;
							case NodeType::z : return z_max;
						}
						throw std::runtime_error("Parameter must be x, y or z");
					}
				};

				typedef MinMaxBox<float> MinMaxBoxf;
			
			}
		}
	}
}

#endif
