#define DLL_EXPORT

#include "Box.hpp"
#include <cassert>
#include "../Cameras/Camera.hpp"
#include "Frustum.hpp"

using namespace Engine;
using namespace Engine::Math;
using namespace Engine::Scene;
using namespace Engine::Scene::Boundings;


Box::Box(const Vec4f& _location, const Vec3f& _size) : location(_location), size(_size)
{ }

BoundingObject* Box::Transform(const Math::Mat4f& _matrix, BoundingObject* _target)
{
	if (_target == nullptr)
	{
		_target = new TransformedBox(
		_matrix * location,
		_matrix * Vec4f(size.X(), 0.0f, 0.0f, 0.0f),
		_matrix * Vec4f(0.0f, size.Y(), 0.0f, 0.0f),
		_matrix * Vec4f(0.0f, 0.0f, size.Z(), 0.0f)
		);
	}
	else
	{
		*_target = TransformedBox(
			_matrix * location,
			_matrix * Vec4f(size.X(), 0.0f, 0.0f, 0.0f),
			_matrix * Vec4f(0.0f, size.Y(), 0.0f, 0.0f),
			_matrix * Vec4f(0.0f, 0.0f, size.Z(), 0.0f)
		);
	}

	return _target;
}

inline const Math::Vec3f& Box::Size() const
{
	return size;
}
inline const Math::Vec4f& Box::Location() const
{
	return location;
}

bool Box::IsVisible(const Cameras::Camera* const _camera) const
{
	return _camera->Frustum().Intersect(this);
}

SplittingResult Box::IntersectPlane(const Math::Vec4f& _plane) const
{
	//Calculate p and n vertex
	Vec4f p = location;
	Vec4f n = location;

	if (_plane.X() >= 0.0f)
		p.X(p.X() + size.X());
	else
		n.X(n.X() + size.X());
	if (_plane.Y() >= 0.0f)
		p.Y(p.Y() + size.Y());
	else
		n.Y(n.Y() + size.Y());
	if (_plane.Z() >= 0.0f)
		p.Z(p.Z() + size.Z());
	else
		n.Z(n.Z() + size.Z());

	if (Vec4f::Dot(p, _plane) < 0.01) //If p outside -> n outside
		return SplittingResult::Back;
	else if (Vec4f::Dot(n, _plane) < -0.01) //p inside and n outside
		return SplittingResult::Split;

	return SplittingResult::Front;
}

Math::Vec3f Box::GetCenterPoint() const
{
	return (Vec3f)location + size/2;
}

Bih::MinMaxBoxf Box::GetAabb() const
{
	Vec3f locppos = (Vec3f)location + size;
	Bih::MinMaxBoxf retval;
	
	if(location.X() < locppos.X())
	{
		retval.x_min = location.X();
		retval.x_max = locppos.X();
	}
	else
	{
		retval.x_min = locppos.X();
		retval.x_max = location.X();
	}
	
	if(location.Y() < locppos.Y())
	{
		retval.y_min = location.Y();
		retval.y_max = locppos.Y();
	}
	else
	{
		retval.y_min = locppos.Y();
		retval.y_max = location.Y();
	}
	
	if(location.Z() < locppos.Z())
	{
		retval.z_min = location.Z();
		retval.z_max = locppos.Z();
	}
	else
	{
		retval.z_min = locppos.Z();
		retval.z_max = location.Z();
	}

	return retval;
}