#ifndef _ENGINE_SCENE_BOUNDINGS_CONE_HPP
#define _ENGINE_SCENE_BOUNDINGS_CONE_HPP

#include "BoundingObject.hpp"
#include "../../Math/Math.hpp"
#include "SplittingResult.hpp"

/*
 * WARNING:
 * This is not a real cone, but one with a sphere segment as its bounding (radius is equal everywhere on the hat)
 * Default direction is -z
 */

namespace Engine
{
	namespace Scene
	{
		namespace Boundings
		{
			class _DLL_DECLSPEC Cone : public BoundingObject
			{
			public:
				Cone(float _angle, float _height);
				~Cone();

				BoundingObject* Transform(const Math::Mat4f& _matrix, BoundingObject* _target = nullptr);

				bool IsVisible(const Cameras::Camera* const _camera) const;

				SplittingResult IntersectPlane(const Math::Vec4f& _plane) const;

				const Math::Vec3f& Direction() const;
				const Math::Vec4f& Position() const;
				float Angle() const;
				void Angle(float _angle);
				float Height() const;
				void Height(float _height);

				Math::Vec3f GetCenterPoint() const;

				Bih::MinMaxBoxf GetAabb() const;

			private:
				Math::Vec3f direction;
				Math::Vec4f position;
				float angle;
				float height;
			};
		}
	}
}

#endif