#ifndef _ENGINE_SCENE_BOUNDINGS_FRUSTRUM_HPP
#define _ENGINE_SCENE_BOUNDINGS_FRUSTRUM_HPP

#include "../../dllconfig.hpp"
#include "../../Math/Math.hpp"
#include "TransformedBox.hpp"
#include "Sphere.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Cameras { class Camera; }

		namespace Boundings
		{
			enum FrustumProjectionType { Perspective, Orthographic };

			class _DLL_DECLSPEC Frustum
			{
			public:
				Frustum(float _fov, float _ratio, float _nearPlane, float _farPlane);
				Frustum(float _left, float _right, float _bottom, float _top, float _nearPlane, float _farPlane);

				void PlaneNormal(int _index, const Math::Vec3f& _normal);
				void PlaneD(int _index, float _d);

				bool Intersect(const BoundingObject* _bobject) const;

				void FromCameraParameter(const Math::Vec4f& _position, const Math::Vec4f& _right, const Math::Vec4f& _up, const Math::Vec4f& _view);

				void Resize(float _fov, float _ratio, float _nearPlane, float _farPlane);

			private:
				float hNear, hFar, wNear, wFar;
				float nearPlane, farPlane;
				float left, right, top, bottom;
				FrustumProjectionType projectionType;

				Math::Vec4f plane[6];
			};
		}
	}
}

#endif