#define DLL_EXPORT

#include "Sphere.hpp"
#include "../Cameras/Camera.hpp"
#include "Frustum.hpp"

using namespace Engine;
using namespace Engine::Math;
using namespace Engine::Scene;
using namespace Engine::Scene::Boundings;

Sphere::Sphere(const Vec4f& _center, float _radius) : center(_center), radius(_radius)
{
	center.W(1.0f);
}
/*
Sphere::Sphere(const Sphere& _sphere) : radius(_sphere.radius), center(_sphere.center), transformedSphere(_sphere.transformedSphere)
{ }*/

Sphere::~Sphere()
{ }

inline float Sphere::Radius() const
{
	return radius;
}

inline const Math::Vec4f& Sphere::Center() const
{
	return center;
}

BoundingObject* Sphere::Transform(const Math::Mat4f& _matrix, BoundingObject* _target)
{
	if (_target == nullptr)
		_target = new Sphere(_matrix * center, radius);
	else
		((Sphere*)_target)->center = _matrix * center;
	
	return _target;
}

/*const Sphere& Sphere::operator= (const Sphere& _sphere)
{
	radius = _sphere.radius;
	center = _sphere.center;

	return *this;
}*/

bool Sphere::IsVisible(const Cameras::Camera* const _camera) const
{
	return _camera->Frustum().Intersect(this);
}

SplittingResult Sphere::IntersectPlane(const Math::Vec4f& _plane) const
{
	float d = Vec4f::Dot(center, _plane);

	if (fabs(d) < radius)
		return SplittingResult::Split;
	
	if (d < 0.0f)
		return SplittingResult::Back;

	return SplittingResult::Front;
}

Math::Vec3f Sphere::GetCenterPoint() const
{
	return (Vec3f)center;
}

Bih::MinMaxBoxf Sphere::GetAabb() const
{
	Bih::MinMaxBoxf retval;

	retval.x_min = center.X() - radius;
	retval.x_max = center.X() + radius;
	retval.y_min = center.Y() - radius;
	retval.y_max = center.Y() + radius;
	retval.z_min = center.Z() - radius;
	retval.z_max = center.Z() + radius;

	return retval;
}
