#ifndef _ENGINE_SCENE_BOUNDINGS_TRANSFORMEDBOX_HPP
#define _ENGINE_SCENE_BOUNDINGS_TRANSFORMEDBOX_HPP

#include "../../dllconfig.hpp"
#include "../../Math/Math.hpp"
#include "SplittingResult.hpp"
#include "BoundingObject.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Boundings
		{
			class _DLL_DECLSPEC TransformedBox : public BoundingObject
			{
			public:
				TransformedBox();
				TransformedBox(const Math::Vec4f& _location, const Math::Vec4f& _axis1, const Math::Vec4f& _axis2, const Math::Vec4f& _axis3);
				TransformedBox(const TransformedBox& _box);

				const Math::Vec4f& Location() const;
				const Math::Vec4f& Axis1() const;
				const Math::Vec4f& Axis2() const;
				const Math::Vec4f& Axis3() const;

				const Math::Vec4f& NormalizedAxis1() const;
				const Math::Vec4f& NormalizedAxis2() const;
				const Math::Vec4f& NormalizedAxis3() const;

				const float* PlaneDistances() const;

				SplittingResult IntersectPlane(const Math::Vec4f& _plane) const;

				BoundingObject* Transform(const Math::Mat4f& _matrix, BoundingObject* _target = nullptr);

				const TransformedBox& operator = (const TransformedBox& _box);

				bool IsVisible(const Cameras::Camera* const _camera) const;

				Math::Vec3f GetCenterPoint() const;

				Bih::MinMaxBoxf GetAabb() const;

			private:
				float planeDistances[6];
				Math::Vec4f location;
				Math::Vec4f axis1, axis2, axis3;
				Math::Vec4f normalizedAxis1, normalizedAxis2, normalizedAxis3;
			};
		}
	}
}

#endif