#ifndef _ENGINE_SCENE_CAMERAS_CAMERA_HPP
#define _ENGINE_SCENE_CAMERAS_CAMERA_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"

#include "../../Math/Math.hpp"
#include "CameraType.hpp"

#include <vector>

namespace Engine
{
	class EngineManager;

	namespace Scene
	{
		class SceneObject;

		namespace Cameras
		{
			class _DLL_DECLSPEC Camera
			{
			public:
				Camera(SceneObject* _sceneObject, EngineManager* _manager);
				~Camera();

				void ResetVisibility();
				std::vector<char>& Visibility();
				void Visibility(int _index, bool _value);

				Scene::SceneObject* SceneObject() const;
				EngineManager* Manager() const;

				virtual void Update(const Math::Mat4f& _matrix) = 0;
				virtual void Render() = 0;

				virtual void Resize(unsigned int _width, unsigned int _height) = 0;

				virtual const Boundings::Frustum& Frustum() const = 0;
				virtual const Math::Mat4f& CameraMatrix() const = 0;
				virtual const Math::Vec4f& Position() const = 0;
				virtual const Math::Mat4f& InverseCameraMatrix() const = 0;

				virtual CameraType GetType() const = 0;

			private:
				Scene::SceneObject* sceneObject;
				EngineManager* manager;

				std::vector<char> visibility;
			};
		}
	}
}

#endif