#ifndef _ENGINE_SCENE_CAMERAS_GICAMERA_HPP
#define _ENGINE_SCENE_CAMERAS_GICAMERA_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"

#include "Camera.hpp"
#include "../Boundings/Frustum.hpp"
#include "../../Util/GLStopWatch.hpp"
#include "../../Util/StopWatch.hpp"
#include "../../Graphic/FrameBuffers/FrameBuffer.hpp"
#include "../../Graphic/Buffers/Texture2D.hpp"
#include "../../Graphic/Shaders/Shader.hpp"

#include "../../Graphic/Effects/DirectionalLightEffect.hpp"
#include "../../Graphic/Effects/PointLightEffect.hpp"

namespace Engine
{
	namespace Scene
	{
		class SceneObject;
		
		namespace Boundings
		{
			class BoxBspNode;
		}

		namespace Cameras
		{
			class _DLL_DECLSPEC GICamera : public Camera
			{
			public:
				GICamera(float _fov, unsigned int _width, unsigned int _height, float _nearPlane, float _farPlane, Scene::SceneObject* _sceneObject, Graphic::GraphicContext* _context,
					EngineManager* _manager, Components::VoxelOctreeComponent* octree);
				~GICamera();

				void Update(const Math::Mat4f& _matrix);
				void Render();

				virtual const Math::Mat4f& CameraMatrix() const;
				const Math::Mat4f& InverseCameraMatrix() const;
				const Math::Vec4f& Position() const;
				const Math::Mat4f& OctreeMatrix() const;

				float* ReadBuffer(unsigned int& _size);

				const Boundings::Frustum& Frustum() const;

				const unsigned int VisibleObjects() const;
				virtual CameraType GetType() const;
				virtual void Resize(unsigned int _width, unsigned int _height);

				float OpenGLTime();
				float CppTime();

			private:
				Util::GLStopWatch glStopWatch;
				Util::StopWatch cppStopWatch;
				float nearPlane, farPlane, fov;
				unsigned int visibleObjects;

				Boundings::Frustum frustum;

				Math::Mat4f projectionMatrix;
				Math::Mat4f cameraMatrix;
				Math::Mat4f inverseCameraMatrix;
				Math::Vec4f position;

				Graphic::FrameBuffers::FrameBuffer deferredBuffer;
				Graphic::Buffers::Texture2D depthBufferTexture;
				Graphic::Buffers::Texture2D normalTexture, tangentTexture, depthTexture;

				Graphic::FrameBuffers::FrameBuffer illuminationBuffer;
				Graphic::Buffers::Texture2D diffuseIllumination;
				Graphic::Buffers::Texture2D specularIllumination;

				EngineManager* manager;

				Graphic::Effects::DirectionalLightEffect directionalLightEffect;
				Graphic::Effects::PointLightEffect pointLightEffect;

				Graphic::GraphicContext* context;
				Components::VoxelOctreeComponent* octree;
			};
		}
	}
}

#endif