#ifndef _ENGINE_SCENE_CAMERAS_RENDERCAMERA_HPP
#define _ENGINE_SCENE_CAMERAS_RENDERCAMERA_HPP

#include "../../Prototypes.hpp"
#include "../../dllconfig.hpp"
#include "Camera.hpp"
#include "../Boundings/Frustum.hpp"
#include "../../Util/GLStopWatch.hpp"
#include "../../Util/StopWatch.hpp"
#include "../../Graphic/FrameBuffers/FrameBuffer.hpp"
#include "../../Graphic/Buffers/Texture2D.hpp"
#include "../../Graphic/Shaders/Shader.hpp"
#include "../../Graphic/Effects/DirectionalLightEffect.hpp"
#include "../../Graphic/Effects/SpotLightEffect.hpp"
#include "../../Graphic/Effects/PointLightEffect.hpp"
#include "../../Graphic/Effects/TonemapEffect.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Cameras
		{
			class _DLL_DECLSPEC RenderCamera : public Camera
			{
			public:
				RenderCamera(float _fov, unsigned int _width, unsigned int _height, float _nearPlane, float _farPlane, Scene::SceneObject* _sceneObject, Graphic::GraphicContext* _context,
					EngineManager* _manager);
				~RenderCamera();

				void Update(const Math::Mat4f& _matrix);
				void Render();

				virtual const Math::Mat4f& CameraMatrix() const;
				const Math::Mat4f& InverseCameraMatrix() const;
				const Math::Vec4f& Position() const;

				float* ReadBuffer(unsigned int& _size);

				const Boundings::Frustum& Frustum() const;

				const unsigned int VisibleObjects() const;
				virtual CameraType GetType() const;
				virtual void Resize(unsigned int _width, unsigned int _height);

				float OpenGLTime();
				float CppTime();

			private:
				Util::GLStopWatch glStopWatch;
				Util::StopWatch cppStopWatch;
				float nearPlane, farPlane, fov;
				unsigned int visibleObjects;

				Boundings::Frustum frustum;

				Math::Mat4f projectionMatrix;
				Math::Mat4f cameraMatrix;
				Math::Mat4f inverseCameraMatrix;
				Math::Vec4f position;

				Graphic::FrameBuffers::FrameBuffer deferredBuffer;
				Graphic::Buffers::Texture2D depthBufferTexture, depthTexture, normalTexture;

				Graphic::FrameBuffers::FrameBuffer lightAccumulationBuffer;
				Graphic::Buffers::Texture2D diffuseTexture, specularTexture;

				Graphic::FrameBuffers::FrameBuffer hdrBuffer;
				Graphic::Buffers::Texture2D hdrTexture;

				Graphic::Effects::DirectionalLightEffect directionalLightEffect;
				Graphic::Effects::PointLightEffect pointLightEffect;
				Graphic::Effects::SpotLightEffect spotLightEffect;
				Graphic::Effects::TonemapEffect tonemapEffect;

				Graphic::GraphicContext* context;

				void RenderTransparent(const Boundings::BoxBspNode& _node, unsigned int& _count);
			};
		}
	}
}

#endif