#define DLL_EXPORT

#include "AnimationComponent.hpp"
#include "../Animations/Animation.hpp"
#include "../../EngineManager.hpp"
#include "../SceneObject.hpp"
#include "PositionComponent.hpp"

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Animations;
using namespace Engine::Math;
using namespace Engine::Core::Collections;

AnimationComponent::AnimationComponent(Scene::SceneObject* _sceneObject, EngineManager* _manager, const Math::Vec3f& _translate, const Math::Vec3f& _rotate, const Math::Vec3f& _scale)
	: IComponent(_sceneObject), manager(_manager), loc(_translate), rot(_rotate), scale(_scale)
{ }
AnimationComponent::~AnimationComponent()
{
	for (auto it = animations.begin(); it != animations.end(); ++it)
		delete it->second;
}
				
void AnimationComponent::RegisterAnimation(Core::Collections::HashString _name, Animations::Animation* _animation)
{
	animations[_name] = _animation;
	_animation->BindComponent(loc.GetGlVector(), loc.GetGlVector() + 1, loc.GetGlVector() + 2,
		rot.GetGlVector(), rot.GetGlVector() + 1, rot.GetGlVector() + 2,
		scale.GetGlVector(), scale.GetGlVector() + 1, scale.GetGlVector() + 2);
}

void AnimationComponent::UpdateComponent()
{
	float t = manager->Clock().AbsoluteTime();

	for (auto it = animations.begin(); it != animations.end(); ++it)
		it->second->Update(t);

	SceneObject()->GetComponent<PositionComponent>()->Matrix(
		Mat4f::Translate(loc) * Mat4f::RotateZ(rot.Z()) * Mat4f::RotateY(rot.Y()) * Mat4f::RotateX(rot.X()) * Mat4f::Scale(scale));
}

ComponentType AnimationComponent::GetComponentType()
{
	return ComponentType::AnimationComponentType;
}
ComponentType AnimationComponent::ComponentTypeOf()
{
	return ComponentType::AnimationComponentType;
}

inline const std::unordered_map<Core::Collections::HashString, Animations::Animation*>& AnimationComponent::AnimationList() const
{
	return animations;
}