#define DLL_EXPORT

#include "BoundingComponent.hpp"
#include "../SceneObject.hpp"
#include "PositionComponent.hpp"
#include "../Boundings/BoundingObject.hpp"
#include "../../Util/Logger.hpp"

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Boundings;

BoundingComponent::BoundingComponent(Scene::SceneObject* _sceneObject, BoundingObject* _boundingObject) : IComponent(_sceneObject, false), boundingObject(_boundingObject), worldBoundingObject(nullptr),
	visibilityIndex(0), isSomewhereVisible(false)
{ }

BoundingComponent::~BoundingComponent()
{
	if (worldBoundingObject != nullptr)
		delete worldBoundingObject;
}

inline ComponentType BoundingComponent::GetComponentType()
{
	return ComponentType::BoundingComponentType;
}

ComponentType BoundingComponent::ComponentTypeOf()
{
	return ComponentType::BoundingComponentType;
}

void BoundingComponent::UpdateComponent()
{
	isSomewhereVisible = false;

	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();

	if (pc == nullptr)
	{
		Util::Logger::Fatal("BoundingComponent without PositionComponent found");
		return;
	}

	worldBoundingObject = boundingObject->Transform(pc->WorldMatrix(), worldBoundingObject);
}

inline Boundings::BoundingObject* BoundingComponent::Object() const
{
	return boundingObject;
}

Boundings::BoundingObject* BoundingComponent::WorldObject() const
{
	return worldBoundingObject;
}

inline const size_t BoundingComponent::VisibilityIndex() const
{
	return visibilityIndex;
}

inline void BoundingComponent::VisibilityIndex(size_t _index)
{
	visibilityIndex = _index;
}

inline bool BoundingComponent::IsVisible() const
{
	return isSomewhereVisible;
}

inline void BoundingComponent::AddVisible()
{
	isSomewhereVisible = true;
}