#define DLL_EXPORT

#include "FreeFlyControllerComponent.hpp"
#include "../../SceneObject.hpp"
#include "../../../EngineManager.hpp"
#include "../PositionComponent.hpp"
#include "../../../Core/Control/RenderControl.hpp"

#define _USE_MATH_DEFINES
#include <math.h>

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Components::ControllerComponents;
using namespace Engine::Core::IO;
using namespace Engine::Math;

FreeFlyControllerComponent::FreeFlyControllerComponent(Scene::SceneObject* _sceneObject, EngineManager* _manager) :
	IComponent(_sceneObject), manager(_manager), moveSpeed(2.0f), position(), rotation(), mouseSpeed(0.02f)
{ }

FreeFlyControllerComponent::~FreeFlyControllerComponent()
{ }

ComponentType FreeFlyControllerComponent::GetComponentType()
{
	return ComponentType::FreeFlyControllerComponentType;
}
ComponentType FreeFlyControllerComponent::ComponentTypeOf()
{
	return ComponentType::FreeFlyControllerComponentType;
}

void FreeFlyControllerComponent::UpdateComponent()
{
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();

	Math::Vec4f moveForward, moveRight;
	moveForward = pc->Matrix() * Math::Vec4f(0.0f, 0.0f, -moveSpeed * manager->Clock().RelativeTime(), 0.0f);
	moveRight = pc->Matrix() * Math::Vec4f(moveSpeed * manager->Clock().RelativeTime(), 0.0f, 0.0f, 0.0f);

	if (manager->RenderControl()->Keyboard().IsKeyPressed(Keys::W))
		position += moveForward;
	else if (manager->RenderControl()->Keyboard().IsKeyPressed(Keys::S))
		position -= moveForward;
	
	if (manager->RenderControl()->Keyboard().IsKeyPressed(Keys::A))
		position -= moveRight;
	else if (manager->RenderControl()->Keyboard().IsKeyPressed(Keys::D))
		position += moveRight;

	if (manager->RenderControl()->Mouse().IsButtonDoubleClick(MouseButton::Left))
	{
		manager->RenderControl()->Mouse().FixMousePosition(true);
		manager->RenderControl()->HasMouseFocus(true);
	}
	else if (manager->RenderControl()->Keyboard().IsKeyDown(Keys::Escape))
	{
		manager->RenderControl()->Mouse().FixMousePosition(false);
		manager->RenderControl()->HasMouseFocus(false);
	}

	if (manager->RenderControl()->HasMouseFocus())
	{
		POINT p = manager->RenderControl()->Mouse().RelativeMousePosition();

		rotation.X(rotation.X() - p.y * mouseSpeed);
		rotation.Y(rotation.Y() - p.x * mouseSpeed);

		if (rotation.X() < - (float)M_PI + 0.001f)
			rotation.X(-(float)M_PI + 0.001f);
		else if (rotation.X() > (float)M_PI - 0.001f)
			rotation.X((float)M_PI - 0.001f);
	}

	pc->Matrix(Mat4f::Translate(position) * Mat4f::RotateY(rotation.Y()) * Mat4f::RotateX(rotation.X()));
}

float FreeFlyControllerComponent::MoveSpeed() const
{
	return moveSpeed;
}

void FreeFlyControllerComponent::MoveSpeed(float _speed)
{
	moveSpeed = _speed;
}

float FreeFlyControllerComponent::MouseSpeed() const
{
	return mouseSpeed;
}

void FreeFlyControllerComponent::MouseSpeed(float _speed)
{
	mouseSpeed = _speed;
} 

void FreeFlyControllerComponent::OnEnabling()
{
	//Aktuelle Position berechnen
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();
	if (pc != nullptr)
	{
		position = pc->Matrix() * Vec4f(0.0f, 0.0f, 0.0f, 1.0f);

		float xangle = asinf(-pc->Matrix().M23());
		float yangle = acosf(pc->Matrix().M33() / cosf(xangle));

		if (fabsf(pc->Matrix().M23()) >= 0.98f)
			yangle = acosf(min(pc->Matrix().M32() / -pc->Matrix().M23(), 1.0f));

		rotation = Vec2f(xangle, yangle);
	}
}