#define DLL_EXPORT

#include "OrbitControllerComponent.hpp"

#include "../../SceneObject.hpp"
#include "../../../EngineManager.hpp"
#include "../PositionComponent.hpp"
#include "../../../Core/Control/RenderControl.hpp"

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Components::ControllerComponents;
using namespace Engine::Core::IO;
using namespace Engine::Math;

OrbitControllerComponent::OrbitControllerComponent(Scene::SceneObject* _sceneObject, EngineManager* _manager) :
	IComponent(_sceneObject), manager(_manager), moveSpeed(1.0f), position(), rotation(), mouseSpeed(0.005f), distance(8.0f), mouseWheelSpeed(0.7f)
{
	OnEnabling();
}

OrbitControllerComponent::~OrbitControllerComponent()
{ }

ComponentType OrbitControllerComponent::GetComponentType()
{
	return ComponentType::OrbitControllerComponentType;
}
ComponentType OrbitControllerComponent::ComponentTypeOf()
{
	return ComponentType::OrbitControllerComponentType;
}

void OrbitControllerComponent::UpdateComponent()
{
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();

	if (manager->RenderControl()->Keyboard().IsKeyPressed(Keys::Shift))
	{
		if (manager->RenderControl()->Mouse().IsButtonPressed(MouseButton::Left)) //When SHIFT is pressed -> Move the camera around
		{
			POINT mouse = manager->RenderControl()->Mouse().RelativeMousePosition();
			Math::Vec4f moveUp, moveRight;
			moveUp = pc->Matrix() * Math::Vec4f(0.0f, moveSpeed * mouse.y * manager->Clock().RelativeTime(), 0.0f, 0.0f);
			moveRight = pc->Matrix() * Math::Vec4f(-moveSpeed * mouse.x * manager->Clock().RelativeTime(), 0.0f, 0.0f, 0.0f);

			position += moveUp + moveRight;
		}
	}
	else //Otherwise rotate it
	{
		if (manager->RenderControl()->Mouse().IsButtonPressed(MouseButton::Left))
		{
			rotation *= Mat4f::RotateX(- manager->RenderControl()->Mouse().RelativeMousePosition().y * mouseSpeed) * Mat4f::RotateY(- manager->RenderControl()->Mouse().RelativeMousePosition().x * mouseSpeed);
		}
	}

	distance -= manager->RenderControl()->Mouse().MouseWheel() * mouseWheelSpeed;
	if (distance < 0.01f)
		distance = 0.01f;
	
	pc->Matrix(Mat4f::Translate(position) * rotation * Mat4f::Translate(Vec3f(0.0f, 0.0f, distance)));
}

inline float OrbitControllerComponent::MoveSpeed() const
{
	return moveSpeed;
}

inline void OrbitControllerComponent::MoveSpeed(float _speed)
{
	moveSpeed = _speed;
}

inline float OrbitControllerComponent::MouseSpeed() const
{
	return mouseSpeed;
}

inline void OrbitControllerComponent::MouseSpeed(float _speed)
{
	mouseSpeed = _speed;
} 

inline float OrbitControllerComponent::MouseWheelSpeed() const
{
	return mouseWheelSpeed;
}

inline void OrbitControllerComponent::MouseWheelSpeed(float _speed)
{
	mouseWheelSpeed = _speed;
} 

void OrbitControllerComponent::OnEnabling()
{
	//Aktuelle Position berechnen
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();
	if (pc != nullptr)
	{
		Vec4f r = pc->Matrix() * Vec4f(0.0f, 0.0f, -1.0f, 0.0f);

		position = pc->Matrix() * Vec4f(0.0f, 0.0f, 0.0f, 1.0f) + r * distance;

		const Mat4f& m = pc->Matrix();
		rotation = Mat4f(m.M11(), m.M12(), m.M13(), 0.0f, m.M21(), m.M22(), m.M23(), 0.0f, m.M31(), m.M32(), m.M33(), 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
	}
}