#define DLL_EXPORT

#include "IComponent.hpp"
#include "../SceneObject.hpp"
#include "../World.hpp"

using namespace Engine::Scene;
using namespace Engine::Scene::Components;

IComponent::IComponent(Engine::Scene::SceneObject* _sceneObject) : sceneObject(_sceneObject), isEnabled(true)
{
	_sceneObject->World()->RegisterUpdateComponent(this);

	OnEnabling();
}

IComponent::IComponent(Engine::Scene::SceneObject* _sceneObject, const bool _registerUpdate) : sceneObject(_sceneObject), isEnabled(true)
{
	if (_registerUpdate)
		_sceneObject->World()->RegisterUpdateComponent(this);

	OnEnabling();
}

IComponent::~IComponent()
{
	sceneObject->World()->RemoveUpdateComponent(this);
}

inline SceneObject* IComponent::SceneObject()
{
	return sceneObject;
}

inline bool IComponent::IsEnabled() const
{
	return isEnabled;
}

inline void IComponent::IsEnabled(bool _value)
{
	isEnabled = _value;

	if (isEnabled)
		OnEnabling();
}

void IComponent::Update()
{
	if (isEnabled)
		UpdateComponent();
}

void IComponent::OnEnabling()
{
}