#define DLL_EXPORT

#include "PointLightComponent.hpp"
#include "../../SceneObject.hpp"
#include "../PositionComponent.hpp"
#include "../../../Graphic/GraphicContext.hpp"
#include "../../../Core/System/SystemConfiguration.hpp"
#include "../../Lights/PointLight.hpp"
#include "../BoundingComponent.hpp"
#include "../../../Graphic/IconRenderer.hpp"
#include "../../../Graphic/Shaders/Shader.hpp"
#include "../../../Graphic/BasicBodies/LineSphere.hpp"


using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Lights;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Components::LightComponents;
using namespace Engine::Core::System;

PointLightComponent::PointLightComponent(Scene::SceneObject* _sceneObject, Lights::PointLight* _light, Graphic::GraphicContext* _context) : LightComponent(_sceneObject), light(_light), context(_context)
{
	bounding = new BoundingComponent(_sceneObject, &((Boundings::Sphere&)_light->Boundings()));
	_sceneObject->AddComponent(bounding);
}

PointLightComponent::~PointLightComponent()
{
	delete light;
}

BoundingComponent* PointLightComponent::Bounding() const
{
	return bounding;
}

ComponentType PointLightComponent::GetComponentType()
{
	return ComponentType::PointLightComponentType;
}
ComponentType PointLightComponent::ComponentTypeOf()
{
	return ComponentType::PointLightComponentType;
}

void PointLightComponent::UpdateComponent()
{
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();
	light->Update(pc->WorldMatrix());
}

Lights::Light* PointLightComponent::Light() const
{
	return light;
}

void PointLightComponent::RenderDebug(const Math::Mat4f& _cameraMatrix) const
{
	if (IsEnabled())
	{
		PointLight* pLight = (PointLight*)Light();

		context->IconRenderer()->Render(context->IconPointLight(), pLight->WorldPosition(), _cameraMatrix, SystemConfiguration::Get<Math::Vec2f>("IconSizeLight"));

		context->DepthTest().TestAndWrite();
		context->LineWidth().Set(2.0f);

		context->DebugLineShader()->Activate();
		context->DebugLineShader()->UniformVariable<Math::Mat4f>("modelMatrix") = pLight->LightMatrix();
		context->DebugLineShader()->UniformVariable<Math::Mat4f>("cameraMatrix") = _cameraMatrix;
		context->DebugLineShader()->UniformVariable<Math::Vec3f>("color") = SystemConfiguration::Get<Math::Vec3f>("LineColorLight");
		context->DebugLineShader()->ChangeValues();

		context->LineSphere()->Draw();
	}
}