#define DLL_EXPORT

#include "SpotLightComponent.hpp"
#include "../PositionComponent.hpp"
#include "../../SceneObject.hpp"
#include "../../../Graphic/GraphicContext.hpp"
#include "../../../Core/System/SystemConfiguration.hpp"
#include "../../Lights/SpotLight.hpp"
#include "../BoundingComponent.hpp"
#include "../../../Graphic/IconRenderer.hpp"
#include "../../../Graphic/Shaders/Shader.hpp"
#include "../../../Graphic/BasicBodies/LineCone.hpp"

using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Math;
using namespace Engine::Scene::Lights;
using namespace Engine::Scene::Components;
using namespace Engine::Scene::Components::LightComponents;
using namespace Engine::Core::System;

SpotLightComponent::SpotLightComponent(Scene::SceneObject* _sceneObject, Lights::SpotLight* _light, Graphic::GraphicContext* _context) : LightComponent(_sceneObject), light(_light), context(_context)
{
	bounding = new BoundingComponent(_sceneObject, &((Boundings::Cone&)_light->Boundings()));
	_sceneObject->AddComponent(bounding);
}
SpotLightComponent::~SpotLightComponent()
{
	delete light;
}

BoundingComponent* SpotLightComponent::Bounding() const
{
	return bounding;
}

ComponentType SpotLightComponent::GetComponentType()
{
	return ComponentType::SpotLightComponentType;
}
ComponentType SpotLightComponent::ComponentTypeOf()
{
	return ComponentType::SpotLightComponentType;
}

void SpotLightComponent::UpdateComponent()
{
	PositionComponent* pc = SceneObject()->GetComponent<PositionComponent>();
	light->Update(pc->WorldMatrix());
}

Lights::Light* SpotLightComponent::Light() const
{
	return light;
}

void SpotLightComponent::RenderDebug(const Math::Mat4f& _cameraMatrix) const
{
	if (IsEnabled())
	{
		SpotLight* slight = (SpotLight*)Light();

		context->IconRenderer()->Render(context->IconSpotLight(), light->WorldPosition(), _cameraMatrix, SystemConfiguration::Get<Math::Vec2f>("IconSizeLight"));

		context->DepthTest().TestAndWrite();
		context->LineWidth().Set(2.0f);

		context->DebugLineShader()->Activate();
		context->DebugLineShader()->UniformVariable<Math::Mat4f>("modelMatrix") = slight->LightMatrix();
		context->DebugLineShader()->UniformVariable<Math::Mat4f>("cameraMatrix") = _cameraMatrix;
		context->DebugLineShader()->UniformVariable<Math::Vec3f>("color") = SystemConfiguration::Get<Math::Vec3f>("LineColorLight");
		context->DebugLineShader()->ChangeValues();

		context->LineCone()->Draw();
	}
}