#ifndef _ENGINE_SCENE_COMPONENTS_MESHRENDERCOMPONENT_HPP
#define _ENGINE_SCENE_COMPONENTS_MESHRENDERCOMPONENT_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"

#include "RenderComponent.hpp"
#include "../../Util/SharedPointer.hpp"
#include "../../Graphic/Meshes/Mesh.hpp"
#include "../../Graphic/Materials/Material.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Components
		{
			class _DLL_DECLSPEC MeshRenderComponent : public RenderComponent
			{
			public:
				MeshRenderComponent(Engine::Scene::SceneObject* _sceneObject,
									Engine::Util::SharedPointer<Engine::Graphic::Meshes::Mesh> _mesh,
									Engine::Util::SharedPointer<Engine::Graphic::Materials::Material> _material,
									Graphic::GraphicContext* _context);
				~MeshRenderComponent();

				ComponentType GetComponentType();
				static ComponentType ComponentTypeOf(); //Muss immer implementiert werden

				void Render(Cameras::Camera* _camera);
				void RenderNormals(const Math::Mat4f& _cameraMatrix);
				void RenderDebug(const Math::Mat4f& _cameraMatrix);
				virtual void RenderOctree(const Math::Mat4f& _cameraMatrixX, const Math::Mat4f& _cameraMatrixY, const Math::Mat4f& _cameraMatrixZ);

				inline Engine::Util::SharedPointer<Engine::Graphic::Materials::Material> Material() const;
				inline void Material(Engine::Util::SharedPointer<Engine::Graphic::Materials::Material> _value);

				BoundingComponent* Bounding() const;

			protected:
				virtual void UpdateComponent();

			private:
				void ComponentChanged(void* _sender, ComponentChangedEventArgs& _args);

				Engine::Util::SharedPointer<Engine::Graphic::Meshes::Mesh> mesh;
				Engine::Util::SharedPointer<Engine::Graphic::Materials::Material> material;

				PositionComponent* positionComponent;
				BoundingComponent* bounding;

				Graphic::GraphicContext* context;
			};
		}
	}
}

#endif