#ifndef _ENGINE_SCENE_COMPONENTS_VOXELOCTREECOMPONENT_HPP
#define _ENGINE_SCENE_COMPONENTS_VOXELOCTREECOMPONENT_HPP

#include "../../dllconfig.hpp"
#include "../../Prototypes.hpp"
#include "IComponent.hpp"

#include "../../Math/Math.hpp"
#include "../../Util/GLStopWatch.hpp"
#include "../../Util/StopWatch.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Components
		{
			class _DLL_DECLSPEC VoxelOctreeComponent : public IComponent
			{
			public:
				VoxelOctreeComponent(Scene::SceneObject* _sceneObject, float _extend);
				virtual ~VoxelOctreeComponent();

				virtual Components::ComponentType GetComponentType();
				static Components::ComponentType ComponentTypeOf();

				const Math::Mat4f& MatrixX() const;
				const Math::Mat4f& MatrixY() const;
				const Math::Mat4f& MatrixZ() const;
				const Math::Mat4f& InverseMatrixZ() const;

				Graphic::GI::Generation::IGenerationStage* GenerationStage() const;
				void GenerationStage(Graphic::GI::Generation::IGenerationStage* _stage);
				Graphic::GI::Preparation::IPreparationStage* PreparationStage() const;
				void PreparationStage(Graphic::GI::Preparation::IPreparationStage* _stage);
				Graphic::GI::Illumination::IIlluminationStage* IlluminationStage() const;
				void IlluminationStage(Graphic::GI::Illumination::IIlluminationStage* _stage);
				Graphic::GI::Rendering::IRenderStage* DebugRenderStage() const;
				void DebugRenderStage(Graphic::GI::Rendering::IRenderStage* _stage);
				Graphic::GI::GIData* Data() const;
				void Data(Graphic::GI::GIData* _data);

				void Resize(unsigned int _entryCount);

				void Render();

				float OpenGLTime();
				float CppTime();

			protected:
				void UpdateComponent();

			private:
				Graphic::GI::Generation::IGenerationStage* generationStage;
				Graphic::GI::Preparation::IPreparationStage* preparationStage;
				Graphic::GI::Illumination::IIlluminationStage* illuminationStage;
				Graphic::GI::Rendering::IRenderStage* debugRenderStage;
				Graphic::GI::GIData* data;

				Math::Mat4f projectionMatrix;
				Math::Mat4f matrixX;
				Math::Mat4f matrixY;
				Math::Mat4f matrixZ, inverseMatrixZ;
				Math::Vec4f position;
				float extend;
				bool firstFrame;

				Util::StopWatch cppStopWatch;
				Util::GLStopWatch glStopWatch;
			};
		}
	}
}

#endif