#define DLL_EXPORT

#include "PointLight.hpp"
#include "../../Graphic/BasicBodies/IcoSphere.hpp"
#include <algorithm>

using namespace std;
using namespace Engine;
using namespace Engine::Graphic;
using namespace Engine::Graphic::BasicBodies;
using namespace Engine::Math;
using namespace Engine::Scene;
using namespace Engine::Scene::Lights;

#define MIN_INTENSITY 0.05f

PointLight::PointLight(const Math::Vec3f& _intensity, const Math::Vec3f& _attenuation) : intensity(_intensity), attenuation(_attenuation), worldPosition(),
	bounding(Math::Vec4f(0.0f, 0.0f, 0.0f), CalculateMaximumDistance())
{ }

LightType PointLight::GetType() const
{
	return LightType::Point;
}
				
void PointLight::Update(const Math::Mat4f& _matrix)
{
	worldPosition.X(_matrix.M14());
	worldPosition.Y(_matrix.M24());
	worldPosition.Z(_matrix.M34());

	lightMatrix = Mat4f::Translate(worldPosition) * IcoSphere::CreateMatrix(bounding.Radius());
}

inline const Math::Vec3f& PointLight::Intensity() const
{
	return intensity;
}
inline void PointLight::Intensity(const Math::Vec3f& _value)
{
	intensity = _value;
}

inline const Math::Vec3f& PointLight::Attenuation() const
{
	return attenuation;
}
inline void PointLight::Attenuation(const Math::Vec3f& _value)
{
	attenuation = _value;
}

inline const Math::Vec3f& PointLight::WorldPosition() const
{
	return worldPosition;
}

inline const Math::Mat4f& PointLight::LightMatrix() const
{
	return lightMatrix;
}

inline const Boundings::Sphere& PointLight::Boundings() const
{
	return bounding;
}

float PointLight::CalculateMaximumDistance() const
{
	float maxChannelIntensity = std::max(std::max(intensity.X(), intensity.Y()), intensity.Z());

	if (attenuation.Z() != 0.0f)
	{
		float dist = (-attenuation.Y() + sqrtf(-4.0f * attenuation.X() * attenuation.Z() + attenuation.Y() * attenuation.Y() + 4.0f / MIN_INTENSITY * attenuation.Z() * maxChannelIntensity)) / (2.0f * attenuation.Z());
		return dist;
	}
	else if (attenuation.Y() != 0.0f)
	{
		return (-attenuation.X() + 1.0f / MIN_INTENSITY * maxChannelIntensity) / attenuation.Y();
	}
	else
		return 500.0f;
}

inline float PointLight::GetInfluence(const Math::Vec3f& _target) const
{
	float dist = Vec3f::Length(_target - worldPosition);
	float att = attenuation.X() + dist * attenuation.Y() + dist * dist * attenuation.Z();

	Vec3f i = intensity / att;

	return max(i.X(), max(i.Y(), i.Z()));
}
