#define DLL_EXPORT

#include "SpotLight.hpp"
#include "../../Graphic/BasicBodies/Cone.hpp"
#include <algorithm>

#define _USE_MATH_DEFINES 
#include <math.h>

using namespace std;
using namespace Engine;
using namespace Engine::Scene;
using namespace Engine::Scene::Lights;
using namespace Engine::Math;

#define MIN_INTENSITY 0.05f

SpotLight::SpotLight(const Math::Vec3f& _intensity, float _angle, const Math::Vec3f& _attenuation, float _angularAttenuation) : intensity(_intensity), attenuation(_attenuation), angularAttenuation(_angularAttenuation), cone(_angle, CalculateMaximumDistance()),
	worldDirection(0.0f, 0.0f, 0.0f), worldPosition(0.0f, 0.0f, 0.0f, 1.0f)
{ }
SpotLight::~SpotLight()
{ }

LightType SpotLight::GetType() const
{
	return LightType::Spot;
}
				
void SpotLight::Update(const Math::Mat4f& _matrix)
{
	worldPosition.X(_matrix.M14());
	worldPosition.Y(_matrix.M24());
	worldPosition.Z(_matrix.M34());

	worldDirection.X(-_matrix.M13());
	worldDirection.Y(-_matrix.M23());
	worldDirection.Z(-_matrix.M33());
	worldDirection.Normalize();

	lightMatrix = Mat4f::RemoveScaling(_matrix) * Graphic::BasicBodies::Cone::CreateMatrix(cone.Angle(), cone.Height());
	//shadowMatrix =  * Mat4f::Translate(Vec3f(0.0f, 0.0f, -5.0f));// * Mat4f::Invert(Mat4f::RemoveScaling(_matrix)); Mat4f::Persp(cone.Angle(), 1.0, 0.1f, 50.0f)
	shadowMatrix = Mat4f::Persp(cone.Angle() * 2.0f, 1.0f, 1.0f, 50.0f) * Mat4f::Invert(Mat4f::RemoveScaling(_matrix));
}

inline const Math::Vec3f& SpotLight::WorldDirection() const
{
	return worldDirection;
}
inline const Math::Vec4f& SpotLight::WorldPosition() const
{
	return worldPosition;
}
inline const Math::Vec3f& SpotLight::Intensity() const
{
	return intensity;
}
inline void SpotLight::Intensity(const Math::Vec3f& _value)
{
	intensity = _value;
	cone.Height(CalculateMaximumDistance());
}
inline const Math::Vec3f& SpotLight::Attenuation() const
{
	return attenuation;
}
inline void SpotLight::Attenuation(const Math::Vec3f& _value)
{
	attenuation = _value;
	cone.Height(CalculateMaximumDistance());
}

inline const Math::Mat4f& SpotLight::LightMatrix() const
{
	return lightMatrix;
}

inline const Math::Mat4f& SpotLight::ShadowMatrix() const
{
	return shadowMatrix;
}


inline float SpotLight::AngularAttenuation() const
{
	return angularAttenuation;
}
inline void SpotLight::AngularAttenuation(float _value)
{
	angularAttenuation = _value;
}

inline float SpotLight::Angle() const
{
	return cone.Angle();
}
inline void SpotLight::Angle(float _value)
{
	cone.Angle(_value);
}

float SpotLight::CalculateMaximumDistance() const
{
	float maxChannelIntensity = std::max(std::max(intensity.X(), intensity.Y()), intensity.Z());

	if (attenuation.Z() != 0.0f)
	{
		float dist = (-attenuation.Y() + sqrtf(-4.0f * attenuation.X() * attenuation.Z() + attenuation.Y() * attenuation.Y() + 4.0f / MIN_INTENSITY * attenuation.Z() * maxChannelIntensity)) / (2.0f * attenuation.Z());
		return dist;
	}
	else if (attenuation.Y() != 0.0f)
	{
		return (-attenuation.X() + 1.0f / MIN_INTENSITY * maxChannelIntensity) / attenuation.Y();
	}
	else
		return 500.0f;
}

inline float SpotLight::GetInfluence(const Math::Vec3f& _target) const
{
	float dist = Vec3f::Length(_target - WorldPosition());
	float att = attenuation.X() + dist * attenuation.Y() + dist * dist * attenuation.Z();

	Vec3f i = intensity / att;

	return max(i.X(), max(i.Y(), i.Z()));
}

const Boundings::Cone& SpotLight::Boundings() const
{
	return cone;
}