#ifndef _ENGINE_SCENE_LIGHTS_SPOTLIGHT_HPP
#define _ENGINE_SCENE_LIGHTS_SPOTLIGHT_HPP

#include "Light.hpp"
#include "../../Math/Math.hpp"
#include "../Boundings/Cone.hpp"

namespace Engine
{
	namespace Scene
	{
		namespace Lights
		{
			class _DLL_DECLSPEC SpotLight : public Light
			{
			public:
				SpotLight(const Math::Vec3f& _intensity, float _angle, const Math::Vec3f& _attenuation, float _angularAttenuation);
				~SpotLight();

				virtual LightType GetType() const;
				
				virtual void Update(const Math::Mat4f& _matrix);

				const Math::Vec3f& WorldDirection() const;
				const Math::Vec4f& WorldPosition() const;

				const Math::Vec3f& Intensity() const;
				void Intensity(const Math::Vec3f& _value);

				const Math::Vec3f& Attenuation() const;
				void Attenuation(const Math::Vec3f& _value);
				const Math::Mat4f& LightMatrix() const;
				const Math::Mat4f& ShadowMatrix() const;
				
				float AngularAttenuation() const;
				void AngularAttenuation(float _value);
				float Angle() const;
				void Angle(float _value);

				float CalculateMaximumDistance() const;
				virtual float GetInfluence(const Math::Vec3f& _target) const;

				const Boundings::Cone& Boundings() const;

			private:

				Math::Vec3f worldDirection;
				Math::Vec4f worldPosition;
				Math::Vec3f intensity, attenuation;
				Math::Mat4f lightMatrix, shadowMatrix;
				float angularAttenuation;//, angle, height;
				Boundings::Cone cone;
			};
		}
	}
}

#endif