#ifndef _ENGINE_SCENE_SCENEOBJECT_HPP
#define _ENGINE_SCENE_SCENEOBJECT_HPP

#include "../dllconfig.hpp"
#include "../Prototypes.hpp"

#include "../Core/Concurrency/Lockable.hpp"
#include "../Core/EventHandling/Event.hpp"
#include "Components\ComponentType.hpp"

#include <map>
#include <unordered_map>
#include <vector>

namespace Engine
{
	namespace Scene
	{
		namespace Components { class IComponent; }

		class World;

		class _DLL_DECLSPEC SceneObject : public Engine::Core::Concurrency::Lockable
		{
		public:
			typedef std::vector<Engine::Scene::SceneObject*> ChildList;
			typedef std::unordered_map<Components::ComponentType, Engine::Scene::Components::IComponent*> ComponentList;
			typedef ComponentList::iterator ComponentListIterator;
			typedef std::pair<Components::ComponentType, Engine::Scene::Components::IComponent*> ComponentListEntry;
			typedef Engine::Core::EventHandling::Event<Components::ComponentChangedEventArgs> ComponentChangedEvent;
			typedef Engine::Core::EventHandling::Event<ParentChangedEventArgs> ParentChangedEvent;
		
			SceneObject(World* _world, std::string _name);
			~SceneObject();

			const ChildList& Children() const;
			void AddChild(SceneObject* _child);
			void RemoveChild(SceneObject* _child);
			void ClearChildren();

			const ComponentList& Components() const;
			void AddComponent(Components::IComponent* _component);
			void RemoveComponent(Components::IComponent* _component);
			template <typename T> T* GetComponent()
			{
				ComponentList::iterator it = components.find(T::ComponentTypeOf());
				if (it != components.end())
					return (T*)it->second;
				return nullptr;
			}

			SceneObject* Parent() const;
			World* World() const;

			unsigned short SubTreeCount() const;
			void SubTreeCount(unsigned short _count);

			const std::string& Name() const;

			ComponentChangedEvent ComponentChanged;
			ParentChangedEvent ParentChanged;

			static bool SortSubtreeCount(SceneObject* _first, SceneObject* _second);

		private:
			void Parent(SceneObject* _parent);

			unsigned short subTreeCount;
			SceneObject* parent;
			Scene::World* world;
			std::string name;
			ChildList children;
			ComponentList components;
		};
	}
}

#endif