#ifndef _ENGINE_SCENE_WORLD_HPP
#define _ENGINE_SCENE_WORLD_HPP

#include "../dllconfig.hpp"
#include "../Prototypes.hpp"

#include "Boundings\BoxBSPNode.hpp"
#include "../Core/Collections/HashString.hpp"
#include "../Core/Collections/UnorderedMapExtensions.hpp"

#include <unordered_map>
#include <vector>
#include <set>

namespace Engine
{
	namespace Scene
	{
		class SceneObject;

		class _DLL_DECLSPEC World
		{
		public:
			World();
			~World();

			const std::vector<SceneObject*>& SceneObjects() const;
			void RegisterSceneObject(SceneObject* _sceneObject);
			void RemoveSceneObject(SceneObject* _sceneObject);
			
			const std::vector<Components::RenderComponent*>& RenderObjects() const;

			Components::CameraComponent* ScreenCamera() const;
			void ScreenCamera(Components::CameraComponent* _screenCamera);

			const std::vector<Components::CameraComponent*>& Cameras() const;
			void RegisterCamera(Components::CameraComponent* _camera);

			const std::vector<Components::LightComponents::LightComponent*>& Lights() const;

			const std::vector<Components::IComponent*>& UpdateComponents() const;
			void RegisterUpdateComponent(Components::IComponent* _component);
			void RemoveUpdateComponent(Components::IComponent* _component);

			unsigned int SceneObjectCount() const;

			void Resize(unsigned int _width, unsigned int _height);

			void OrderTransparentObjects();
			const std::vector<Boundings::BoxBspNode>& TransparentObjectBsp() const;

			const std::vector<Components::BoundingComponent*>& BoundingObjects() const;
			const std::vector<Components::PositionComponent*>& PositionComponents() const;

			SceneObject* FindSceneObject(Core::Collections::HashString _name) const;

		private:
			unsigned int sceneObjectCount;

			std::vector<SceneObject*> sceneObjects;

			std::vector<Components::CameraComponent*> cameras;
			std::vector<Components::RenderComponent*> renderObjects;
			std::vector<Components::BoundingComponent*> boundingObjects;

			std::vector<Components::LightComponents::LightComponent*> lights;
			std::vector<Components::IComponent*> updateComponents;
			std::vector<Components::PositionComponent*> positionComponents;

			std::unordered_multimap<Core::Collections::HashString, SceneObject*> nameLookupTable;

			void OrderSceneTree();

			Components::CameraComponent* screenCamera;

			void SceneObject_ComponentsChanged(void* _sender, Components::ComponentChangedEventArgs& _args);
			void SceneObject_ParentChanged(void* _sender, ParentChangedEventArgs& _args);

			int OrderTransparentObjects(std::vector<int>& _readVector, std::vector<int>& _writeVector, size_t _start, size_t _end);

			std::vector<Boundings::BoxBspNode> transparentObjectBsp;
			std::vector<int> transparentCache1, transparentCache2;
		};
	}
}

#endif