#define DLL_EXPORT

#include "Clock.hpp"
#include <iostream>

using namespace Engine::Util;

Clock::Clock() : isRunning(false), lastStartTime(0), timeBeforeLastPause(0), lastTickTime(0), absoluteTime(0.0f), relativeTime(0.0f), tickTime(0)
{
	QueryPerformanceFrequency((LARGE_INTEGER*)&timerFrequency);	
}

Clock::~Clock()
{ }

void Clock::Start()
{
	isRunning = true;
	QueryPerformanceCounter((LARGE_INTEGER*)&lastStartTime);
	QueryPerformanceCounter((LARGE_INTEGER*)&lastTickTime);
}

void Clock::Pause()
{
	isRunning = false;
	timeBeforeLastPause += tickTime - lastStartTime;
}

void Clock::Tick()
{
	if (isRunning)
	{
		lastTickTime = tickTime;
		QueryPerformanceCounter((LARGE_INTEGER*)&tickTime);
		//std::cout << relativeTime << std::endl;

		absoluteTime = (double)(timeBeforeLastPause + tickTime - lastStartTime) / (double)timerFrequency;
		relativeTime = (double)(tickTime - lastTickTime) / (double)timerFrequency;
	}
	else
		relativeTime = 0;
}

float Clock::AbsoluteTime() const
{
	return (float)absoluteTime;
}

float Clock::RelativeTime() const
{
	return (float)relativeTime;
}

bool Clock::IsRunning() const
{
	return isRunning;
}