#ifndef _ENGINE_UTIL_CONVERT_HPP
#define _ENGINE_UTIL_CONVERT_HPP

#include "../dllconfig.hpp"

#include <string>
#include <sstream>
#include <gl\glew.h>

namespace Engine
{
	namespace Util
	{
		class _DLL_DECLSPEC Convert
		{
		public:
			template <typename T> static std::string ToString(T _value)
			{
				std::stringstream ss;
				ss << _value;
				return ss.str();
			}

			template <> static std::string ToString<GLenum>(GLenum _value)
			{
				switch (_value)
				{
				case GL_VERTEX_SHADER:
					return "GL_VERTEX_SHADER";
				case GL_FRAGMENT_SHADER:
					return "GL_FRAGMENT_SHADER";
				case GL_GEOMETRY_SHADER:
					return "GL_GEOMETRY_SHADER";
				case GL_TESS_CONTROL_SHADER:
					return "GL_TESS_CONTROL_SHADER";
				case GL_TESS_EVALUATION_SHADER:
					return "GL_TESS_EVALUATION_SHADER";
				case GL_FRAMEBUFFER_UNDEFINED:
					return "GL_FRAMEBUFFER_UNDEFINED";
				case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT:
					return "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT";
				case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT:
					return "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT";
				case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER:
					return "GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER";
				case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER:
					return "GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER";
				case GL_FRAMEBUFFER_UNSUPPORTED:
					return "GL_FRAMEBUFFER_UNSUPPORTED";
				case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE:
					return "GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE";
				case GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS:
					return "GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS";
				case GL_TEXTURE_1D:
					return "GL_TEXTURE_1D";
				case GL_TEXTURE_2D:
					return "GL_TEXTURE_2D";
				case GL_TEXTURE_3D:
					return "GL_TEXTURE_3D";
				case GL_DEBUG_CATEGORY_API_ERROR_AMD:
					return "GL_DEBUG_CATEGORY_API_ERROR_AMD";

				case GL_DEBUG_SOURCE_API:
					return "GL_DEBUG_SOURCE_API";
				case GL_DEBUG_CATEGORY_APPLICATION_AMD:
					return "GL_DEBUG_CATEGORY_APPLICATION_AMD";
				case GL_DEBUG_SOURCE_APPLICATION:
					return "GL_DEBUG_SOURCE_APPLICATION";
				case GL_DEBUG_CATEGORY_WINDOW_SYSTEM_AMD:
					return "GL_DEBUG_CATEGORY_WINDOW_SYSTEM_AMD";
				case GL_DEBUG_SOURCE_WINDOW_SYSTEM:
					return "GL_DEBUG_SOURCE_WINDOW_SYSTEM";
				case GL_DEBUG_CATEGORY_SHADER_COMPILER_AMD:
					return "GL_DEBUG_CATEGORY_SHADER_COMPILER_AMD";
				case GL_DEBUG_SOURCE_SHADER_COMPILER:
					return "GL_DEBUG_SOURCE_SHADER_COMPILER";
				case GL_DEBUG_SOURCE_THIRD_PARTY:
					return "GL_DEBUG_SOURCE_THIRD_PARTY";
				case GL_DEBUG_CATEGORY_OTHER_AMD:
					return "GL_DEBUG_CATEGORY_OTHER_AMD";
				case GL_DEBUG_SOURCE_OTHER:
					return "GL_DEBUG_SOURCE_OTHER";
				case GL_DEBUG_TYPE_ERROR:
					return "GL_DEBUG_TYPE_ERROR";
				case GL_DEBUG_CATEGORY_DEPRECATION_AMD:
					return "GL_DEBUG_CATEGORY_DEPRECATION_AMD";
				case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR:
					return "GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR";
				case GL_DEBUG_CATEGORY_UNDEFINED_BEHAVIOR_AMD:
					return "GL_DEBUG_CATEGORY_UNDEFINED_BEHAVIOR_AMD";
				case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR:
					return "GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR";
				case GL_DEBUG_TYPE_PORTABILITY_ARB:
					return "GL_DEBUG_TYPE_PORTABILITY_ARB";
				case GL_DEBUG_CATEGORY_PERFORMANCE_AMD:
					return "GL_DEBUG_CATEGORY_PERFORMANCE_AMD";
				case GL_DEBUG_TYPE_PERFORMANCE:
					return "GL_DEBUG_TYPE_PERFORMANCE";
				case GL_DEBUG_TYPE_OTHER:
					return "GL_DEBUG_TYPE_OTHER";
				case GL_DEBUG_SEVERITY_HIGH:
					return "GL_DEBUG_SEVERITY_HIGH";
				case GL_DEBUG_SEVERITY_MEDIUM:
					return "GL_DEBUG_SEVERITY_MEDIUM";
				case GL_DEBUG_SEVERITY_LOW:
					return "GL_DEBUG_SEVERITY_LOW";

				default:
					{
					std::stringstream ss;
					ss << _value;
					return ss.str();
					}
				}
			}
		};
	}
}

#endif