#define DLL_EXPORT

#include "Logger.hpp"

#undef Logger
#undef Debug
#undef Warning
#undef Error
#undef Fatal
#undef GLErrors
#undef MaxMessageType
#undef HasMessages
#undef ResetLogger
#undef InfoMessage
#undef EnableWarnings
#undef DisableWarnings

#include <gl\glew.h>
#include <iostream>
#include "../Core/System/SystemConfiguration.hpp"
#include "../Core/Control/SystemConsole.hpp"
#include <Windows.h>

using namespace Engine::Util;
using namespace Engine::Core::System;
using namespace Engine::Core::Control;

LoggerOutputType Logger::maxMessageType = LoggerOutputType::AllLoggerOutputType;
bool Logger::enableWarnings = true;

void Logger::Debug(std::string _message)
{
	if (enableWarnings && SystemConfiguration::Get<int>("LoggerReportLevel") <= LoggerOutputType::DebugLoggerOutputTye)
	{
		SystemConsole::SetColor(ConsoleColor::GrayConsoleColor);
		std::cout << _message << std::endl;

		if (maxMessageType < LoggerOutputType::DebugLoggerOutputTye)
			maxMessageType = LoggerOutputType::DebugLoggerOutputTye;
	}
}

void Logger::InfoMessage(std::string _message)
{
	if (enableWarnings && SystemConfiguration::Get<int>("LoggerReportLevel") <= LoggerOutputType::MessageLoggerOutputTye)
	{
		SystemConsole::SetColor(ConsoleColor::DefaultConsoleColor);
		std::cout << _message << std::endl;

		if (maxMessageType < LoggerOutputType::MessageLoggerOutputTye)
			maxMessageType = LoggerOutputType::MessageLoggerOutputTye;
	}
}

void Logger::Warning(std::string _message)
{
	if (enableWarnings && SystemConfiguration::Get<int>("LoggerReportLevel") <= LoggerOutputType::WarningLoggerOutputTye)
	{
		SystemConsole::SetColor(ConsoleColor::YellowConsoleColor);
		std::cout << "Warning: ";
		std::cout << _message << std::endl;

		if (maxMessageType < LoggerOutputType::WarningLoggerOutputTye)
			maxMessageType = LoggerOutputType::WarningLoggerOutputTye;
	}
}

void Logger::Error(std::string _message)
{
	if  (SystemConfiguration::Get<int>("LoggerReportLevel") <= LoggerOutputType::ErrorLoggerOutputTye)
	{
		SystemConsole::SetColor(ConsoleColor::RedConsoleColor);
		std::cout << "Error: " << _message << std::endl;
		SystemConsole::Focus();

		if (maxMessageType < LoggerOutputType::ErrorLoggerOutputTye)
			maxMessageType = LoggerOutputType::ErrorLoggerOutputTye;
	}
}

void Logger::Fatal(std::string _message)
{
	if  (SystemConfiguration::Get<int>("LoggerReportLevel") <= LoggerOutputType::FatalLoggerOutputTye)
	{
		SystemConsole::SetColor(ConsoleColor::RedConsoleColor);
		std::cout << "Fatal: " << _message << std::endl;
		SystemConsole::Focus();

		if (maxMessageType < LoggerOutputType::FatalLoggerOutputTye)
			maxMessageType = LoggerOutputType::FatalLoggerOutputTye;
	}
}

void Logger::GLErrors()
{
	GLenum error = glGetError();

	if (error != GL_NO_ERROR)
	{
		switch (error)
		{
			case GL_INVALID_ENUM:
				Logger::Fatal("GL_INVALID_ENUM: enum argument out of range.");
				break;
			case GL_INVALID_VALUE:
				Logger::Fatal("GL_INVALID_VALUE: Numeric argument out of range.");
				break;
			case GL_INVALID_OPERATION:
				Logger::Fatal("GL_INVALID_OPERATION: Operation illegal in current state.");
				break;
			case GL_INVALID_FRAMEBUFFER_OPERATION:
				Logger::Fatal("GL_INVALID_FRAMEBUFFER_OPERATION: Framebuffer object is not complete.");
				break;
			case GL_OUT_OF_MEMORY:
				Logger::Fatal("GL_OUT_OF_MEMORY: Not enough memory left to execute command.");
				break;
			default:
				Logger::Fatal("GL: Unknown error.");
		}
	}
}

LoggerOutputType Logger::MaxMessageType()
{
	return maxMessageType;
}

bool Logger::HasMessages()
{
	return (maxMessageType >= SystemConfiguration::Get<int>("LoggerErrorLevel"));
}

void Logger::ResetLogger()
{
	maxMessageType = LoggerOutputType::AllLoggerOutputType;
}

void Logger::DisableWarnings()
{
	enableWarnings = false;
}

void Logger::EnableWarnings()
{
	enableWarnings = true;
}