#ifndef _ENGINE_UTIL_LOGGER
#define _ENGINE_UTIL_LOGGER

#include "../dllconfig.hpp"
#include "FunctionDecorator.hpp"

#include <string>
#include <iostream>

namespace Engine
{
	namespace Util
	{
		enum LoggerOutputType : unsigned char
		{
			AllLoggerOutputType = 0,
			DebugLoggerOutputTye = 1,
			MessageLoggerOutputTye = 2,
			WarningLoggerOutputTye = 3,
			ErrorLoggerOutputTye = 4,
			FatalLoggerOutputTye = 5,
			NonLoggerOutputType = 6
		};

		class _DLL_DECLSPEC Logger
		{
		public:
			static void Debug(std::string _message);
			static void InfoMessage(std::string _message);
			static void Warning(std::string _message);
			static void Error(std::string _message);
			static void GLErrors();
			static void Fatal(std::string _message);
			static void DisableWarnings();
			static void EnableWarnings();

			static LoggerOutputType MaxMessageType();
			static bool HasMessages();

			static void ResetLogger();

			static bool enableWarnings;

		private:
			static LoggerOutputType maxMessageType;
			
		};

#ifndef NDEBUG
		FUNCTIONDECORATOR_START(Logger)
		FUNCTIONDECORATOR_FUNCTION_CALLERLOG_OPTIONAL(Logger, Debug, std::string, enableWarnings)
		FUNCTIONDECORATOR_FUNCTION(void, Logger, InfoMessage, std::string)
		FUNCTIONDECORATOR_FUNCTION_CALLERLOG_OPTIONAL(Logger, Warning, std::string, enableWarnings)
		FUNCTIONDECORATOR_FUNCTION(LoggerOutputType, Logger, MaxMessageType, )
		FUNCTIONDECORATOR_FUNCTION(bool, Logger, HasMessages, )
		FUNCTIONDECORATOR_FUNCTION_CALLERLOG(Logger, Error, std::string)
		FUNCTIONDECORATOR_FUNCTION(void, Logger, GLErrors, )
		FUNCTIONDECORATOR_FUNCTION(void, Logger, ResetLogger, )
		FUNCTIONDECORATOR_FUNCTION_CALLERLOG(Logger, Fatal, std::string)
		FUNCTIONDECORATOR_FUNCTION(void, Logger, EnableWarnings, )
		FUNCTIONDECORATOR_FUNCTION(void, Logger, DisableWarnings, )
		FUNCTIONDECORATOR_END()

		#undef Logger
		#define Logger Logger_functiondecorator
		#undef Debug
		#define Debug Debug(__FUNCTION__,__FILE__,__LINE__)
		#undef Warning
		#define Warning Warning(__FUNCTION__,__FILE__,__LINE__)
		#undef Error
		#define Error Error(__FUNCTION__,__FILE__,__LINE__)
		#undef GLErrors
		#define GLErrors GLErrors(__FUNCTION__,__FILE__,__LINE__)
		#undef MaxMessageType
		#define MaxMessageType MaxMessageType(__FUNCTION__,__FILE__,__LINE__)
		#undef HasMessages
		#define HasMessages HasMessages(__FUNCTION__,__FILE__,__LINE__)
		#undef ResetLogger
		#define ResetLogger ResetLogger(__FUNCTION__,__FILE__,__LINE__)
		#undef InfoMessage
		#define InfoMessage InfoMessage(__FUNCTION__,__FILE__,__LINE__)
		#undef Fatal
		#define Fatal Fatal(__FUNCTION__,__FILE__,__LINE__)
		#undef EnableWarnings
		#define EnableWarnings EnableWarnings(__FUNCTION__,__FILE__,__LINE__)
		#undef DisableWarnings
		#define DisableWarnings DisableWarnings(__FUNCTION__,__FILE__,__LINE__)
#endif
	}
}

#endif