#ifndef _ENGINE_UTIL_WEIGHTEDCONTAINER_HPP
#define _ENGINE_UTIL_WEIGHTEDCONTAINER_HPP

#include "../dllconfig.hpp"

namespace Engine
{
	namespace Util
	{
		template <typename T>
		class _DLL_DECLSPEC WeightedContainer
		{
		public:
			WeightedContainer(T _data, float _weight) : data(_data), weight(_weight) { }
			WeightedContainer(const WeightedContainer<T>& _copy) : data(_copy.data), weight(_copy.weight) { }
			~WeightedContainer() { }

			WeightedContainer<T>& operator = (const WeightedContainer<T>& _copy)
			{
				data = _copy.data;
				weight = _copy.weight;
				return *this;
			}

			T operator -> ()
			{
				return data;
			}

			T operator * ()
			{
				return data;
			}

			inline bool operator == (const WeightedContainer<T>& _container) const 
			{
				return weight == _container.weight;
			}

			inline bool operator != (const WeightedContainer<T>& _container) const 
			{
				return weight != _container.weight;
			}

			inline bool operator < (const WeightedContainer<T>& _container) const 
			{
				return weight < _container.weight;
			}

			inline bool operator <= (const WeightedContainer<T>& _container) const 
			{
				return weight <= _container.weight;
			}

			inline bool operator > (const WeightedContainer<T>& _container) const 
			{
				return weight > _container.weight;
			}

			inline bool operator >= (const WeightedContainer<T>& _container) const 
			{
				return weight >= _container.weight;
			}

			static bool OrderAscending(const WeightedContainer<T>& _c1, const WeightedContainer<T>& _c2) //Aufsteigend
			{
				return _c1 < c2;
			}

			static bool OrderDescending(const WeightedContainer<T>& _c1, const WeightedContainer<T>& _c2) //Absteigend
			{
				return _c1 > _c2;
			}

			inline const float Weight() const { return weight; }
			inline void Weight(float _weight) { weight = _weight; }

			inline const T Data() const { return data; }
			inline void Data(T _data) { data = _data; }

		private:
			T data;
			float weight;
		};
	}
}

#endif